; ==========
; Rocket Man
; ==========
; (c)1984 Software Farm, written by Julian Chappell.
; For the 16KB ZX81.
;
; Disassembled by Paul Farrow (www.fruitcake.plus.com), 06 July 2017 (revised 6 August 2017).

; ======================================================================================================================================================

; ====================
; Assembler Directives
; ====================
; For TASM assembler.

#define DEFB .BYTE
#define DEFW .WORD
#define DEFM .TEXT
#define DEFS .BLOCK
#define EQU  .EQU
#define ORG  .ORG
#define END  .END

; =======
; Options
; =======

; Include the following definition to create a P81 file, otherwise a P file will be generated.
;#define MAKE_P81

; ======================================================================================================================================================

; =======================
; Character Set Constants
; =======================

_SPACE                          EQU     $00     ; 0
_TOPLEFTBLACK                   EQU     $01     ; 1
_TOPRIGHTBLACK                  EQU     $02     ; 2
_TOPBLACK                       EQU     $03     ; 3
_BOTTOMLEFTBLACK                EQU     $04     ; 4
_LEFTBLACK                      EQU     $05     ; 5
_BOTTOMLEFTTOPRIGHT             EQU     $06     ; 6
_BOTTOMRIGHTWHITE               EQU     $07     ; 7
_CHEQUERBOARD                   EQU     $08     ; 8
_TOPWHITEBOTTOMCHEQUER          EQU     $09     ; 9
_TOPCHEQUERBOTTOMWHITE          EQU     $0A     ; 10
_QUOTE                          EQU     $0B     ; 11
_POUND                          EQU     $0C     ; 12
_DOLLAR                         EQU     $0D     ; 13
_COLON                          EQU     $0E     ; 14
_QUESTIONMARK                   EQU     $0F     ; 15
_OPENBRACKET                    EQU     $10     ; 16
_CLOSEBRACKET                   EQU     $11     ; 17
_GREATERTHAN                    EQU     $12     ; 18
_LESSTHAN                       EQU     $13     ; 19
_EQUALS                         EQU     $14     ; 20
_PLUS                           EQU     $15     ; 21
_MINUS                          EQU     $16     ; 22
_ASTERISK                       EQU     $17     ; 23
_DIVIDE                         EQU     $18     ; 24
_SEMICOLON                      EQU     $19     ; 25
_COMMA                          EQU     $1A     ; 26
_FULLSTOP                       EQU     $1B     ; 27
_0                              EQU     $1C     ; 28
_1                              EQU     $1D     ; 29
_2                              EQU     $1E     ; 30
_3                              EQU     $1F     ; 31
_4                              EQU     $20     ; 32
_5                              EQU     $21     ; 33
_6                              EQU     $22     ; 34
_7                              EQU     $23     ; 35
_8                              EQU     $24     ; 36
_9                              EQU     $25     ; 37
_A                              EQU     $26     ; 38
_B                              EQU     $27     ; 39
_C                              EQU     $28     ; 40
_D                              EQU     $29     ; 41
_E                              EQU     $2A     ; 42
_F                              EQU     $2B     ; 43
_G                              EQU     $2C     ; 44
_H                              EQU     $2D     ; 45
_I                              EQU     $2E     ; 46
_J                              EQU     $2F     ; 47
_K                              EQU     $30     ; 48
_L                              EQU     $31     ; 49
_M                              EQU     $32     ; 50
_N                              EQU     $33     ; 51
_O                              EQU     $34     ; 52
_P                              EQU     $35     ; 53
_Q                              EQU     $36     ; 54
_R                              EQU     $37     ; 55
_S                              EQU     $38     ; 56
_T                              EQU     $39     ; 57
_U                              EQU     $3A     ; 58
_V                              EQU     $3B     ; 59
_W                              EQU     $3C     ; 60
_X                              EQU     $3D     ; 61
_Y                              EQU     $3E     ; 62
_Z                              EQU     $3F     ; 63
_RND                            EQU     $40     ; 64
_INKEY                          EQU     $41     ; 65
_PI                             EQU     $42     ; 66
_CURSORUP                       EQU     $70     ; 112
_CURSORDOWN                     EQU     $71     ; 113
_CURSORLEFT                     EQU     $72     ; 114
_CURSORRIGHT                    EQU     $73     ; 115
_GRAPHICS                       EQU     $74     ; 116
_EDIT                           EQU     $75     ; 117
_NEWLINE                        EQU     $76     ; 118
_RUBOUT                         EQU     $77     ; 119
_MODE                           EQU     $78     ; 120
_FUNCTION                       EQU     $79     ; 121
_NUMBER                         EQU     $7E     ; 126
_CURSOR                         EQU     $7F     ; 127
_BLACK                          EQU     $80     ; 128
_TOPLEFTWHITE                   EQU     $81     ; 129
_TOPRIGHTWHITE                  EQU     $82     ; 130
_BOTTOMBLACK                    EQU     $83     ; 131
_BOTTOMLEFTWHITE                EQU     $84     ; 132
_RIGHTBLACK                     EQU     $85     ; 133
_TOPLEFTBOTTOMRIGHT             EQU     $86     ; 134
_BOTTOMRIGHTBLACK               EQU     $87     ; 135
_INVCHEQUERBOARD                EQU     $88     ; 136
_TOPBLACKBOTTOMCHEQUER          EQU     $89     ; 137
_TOPCHEQUERBOTTOMBLACK          EQU     $8A     ; 138
_INVQUOTE                       EQU     $8B     ; 139
_INVPOUND                       EQU     $8C     ; 140
_INVDOLLAR                      EQU     $8D     ; 141
_INVCOLON                       EQU     $8E     ; 142
_INVQUESTIONMARK                EQU     $8F     ; 143
_INVOPENBRACKET                 EQU     $90     ; 144
_INVCLOSEBRACKET                EQU     $91     ; 145
_INVGREATERTHAN                 EQU     $92     ; 146
_INVLESSTHAN                    EQU     $93     ; 147
_INVEQUALS                      EQU     $94     ; 148
_INVPLUS                        EQU     $95     ; 149
_INVMINUS                       EQU     $96     ; 150
_INVASTERISK                    EQU     $97     ; 151
_INVDIVIDE                      EQU     $98     ; 152
_INVSEMICOLON                   EQU     $99     ; 153
_INVCOMMA                       EQU     $9A     ; 154
_INVFULLSTOP                    EQU     $9B     ; 155
_INV0                           EQU     $9C     ; 156
_INV1                           EQU     $9D     ; 157
_INV2                           EQU     $9E     ; 158
_INV3                           EQU     $9F     ; 159
_INV4                           EQU     $A0     ; 160
_INV5                           EQU     $A1     ; 161
_INV6                           EQU     $A2     ; 162
_INV7                           EQU     $A3     ; 163
_INV8                           EQU     $A4     ; 164
_INV9                           EQU     $A5     ; 165
_INVA                           EQU     $A6     ; 166
_INVB                           EQU     $A7     ; 167
_INVC                           EQU     $A8     ; 168
_INVD                           EQU     $A9     ; 169
_INVE                           EQU     $AA     ; 170
_INVF                           EQU     $AB     ; 171
_INVG                           EQU     $AC     ; 172
_INVH                           EQU     $AD     ; 173
_INVI                           EQU     $AE     ; 174
_INVJ                           EQU     $AF     ; 175
_INVK                           EQU     $B0     ; 176
_INVL                           EQU     $B1     ; 177
_INVM                           EQU     $B2     ; 178
_INVN                           EQU     $B3     ; 179
_INVO                           EQU     $B4     ; 180
_INVP                           EQU     $B5     ; 181
_INVQ                           EQU     $B6     ; 182
_INVR                           EQU     $B7     ; 183
_INVS                           EQU     $B8     ; 184
_INVT                           EQU     $B9     ; 185
_INVU                           EQU     $BA     ; 186
_INVV                           EQU     $BB     ; 187
_INVW                           EQU     $BC     ; 188
_INVX                           EQU     $BD     ; 189
_INVY                           EQU     $BE     ; 190
_INVZ                           EQU     $BF     ; 191
_DOUBLEQUOTE                    EQU     $C0     ; 192
_AT                             EQU     $C1     ; 193
_TAB                            EQU     $C2     ; 194
_CODE                           EQU     $C4     ; 196
_VAL                            EQU     $C5     ; 197
_LEN                            EQU     $C6     ; 198
_SIN                            EQU     $C7     ; 199
_COS                            EQU     $C8     ; 200
_TAN                            EQU     $C9     ; 201
_ASN                            EQU     $CA     ; 202
_ACS                            EQU     $CB     ; 203
_ATN                            EQU     $CC     ; 204
_LN                             EQU     $CD     ; 205
_EXP                            EQU     $CE     ; 206
_INT                            EQU     $CF     ; 207
_SQR                            EQU     $D0     ; 208
_SGN                            EQU     $D1     ; 209
_ABS                            EQU     $D2     ; 210
_PEEK                           EQU     $D3     ; 211
_USR                            EQU     $D4     ; 212
_STR                            EQU     $D5     ; 213
_CHR                            EQU     $D6     ; 214
_NOT                            EQU     $D7     ; 215
_DOUBLEASTERISK                 EQU     $D8     ; 216
_OR                             EQU     $D9     ; 217
_AND                            EQU     $DA     ; 218
_LESSTHANEQUAL                  EQU     $DB     ; 219
_GREATERTHANEQUAL               EQU     $DC     ; 220
_NOTEQUAL                       EQU     $DD     ; 221
_THEN                           EQU     $DE     ; 222
_TO                             EQU     $DF     ; 223
_STEP                           EQU     $E0     ; 224
_LPRINT                         EQU     $E1     ; 225
_LLIST                          EQU     $E2     ; 226
_STOP                           EQU     $E3     ; 227
_SLOW                           EQU     $E4     ; 228
_FAST                           EQU     $E5     ; 229
_NEW                            EQU     $E6     ; 230
_SCROLL                         EQU     $E7     ; 231
_CONT                           EQU     $E8     ; 232
_DIM                            EQU     $E9     ; 233
_REM                            EQU     $EA     ; 234
_FOR                            EQU     $EB     ; 235
_GOTO                           EQU     $EC     ; 236
_GOSUB                          EQU     $ED     ; 237
_INPUT                          EQU     $EE     ; 238
_LOAD                           EQU     $EF     ; 239
_LIST                           EQU     $F0     ; 240
_LET                            EQU     $F1     ; 241
_PAUSE                          EQU     $F2     ; 242
_NEXT                           EQU     $F3     ; 243
_POKE                           EQU     $F4     ; 244
_PRINT                          EQU     $F5     ; 245
_PLOT                           EQU     $F6     ; 246
_RUN                            EQU     $F7     ; 247
_SAVE                           EQU     $F8     ; 248
_RAND                           EQU     $F9     ; 249
_IF                             EQU     $FA     ; 250
_CLS                            EQU     $FB     ; 251
_UNPLOT                         EQU     $FC     ; 252
_CLEAR                          EQU     $FD     ; 253
_RETURN                         EQU     $FE     ; 254
_COPY                           EQU     $FF     ; 255

; ======================================================================================================================================================

; ===============
; Hi-Res Patterns
; ===============
; The pixel patterns for I value of $0C.
;
; $00 = .#..#### ($4F)
; $01 = ....###. ($0E)
; $02 = ..#...#. ($22)
; $03 = ....#..# ($09)
; $04 = .#...... ($40)
; $05 = ....#.#. ($0A)
; $06 = .#.##### ($5F)
; $07 = .#...#.# ($45)
; $08 = .#..##.. ($4C)
; $09 = .......# ($01)
; $0A = ##.####. ($DE)
; $0B = ........ ($00)
; $0C = .....##. ($06)
; $0D = ..#.###. ($2E)
; $0E = ....###. ($0E)
; $0F = ##....## ($C3)
; $10 = .....##. ($06)
; $11 = ....###. ($0E)
; $12 = .#####.. ($7C)
; $13 = .....##. ($06)
; $14 = .....##. ($06)
; $15 = ........ ($00)
; $16 = ....#### ($0F)
; $17 = ..#.##.. ($2C)
; $18 = ....#.#. ($0A)
; $19 = ######## ($FF)
; $1A = #######. ($FE)
; $1B = ....##.# ($0D)
; $1C = ........ ($00)
; $1D = ###....# ($E1)
; $1E = .#..###. ($4E)
; $1F = ..#...## ($23)
; $20 = .#..#### ($4F)
; $21 = .....##. ($06)
; $22 = ##.##### ($DF)
; $23 = .#.#..## ($53)
; $24 = ##...... ($C0)
; $25 = #######. ($FE)
; $26 = ...###.# ($1D)
; $27 = .#...##. ($46)
; $28 = ..##.##. ($36)
; $29 = ##..###. ($CE)
; $2A = ######.# ($FD)
; $2B = #.#..##. ($A6)
; $2C = #.##.##. ($B6)
; $2D = ...#..#. ($12)
; $2E = ##..##.# ($CD)
; $2F = .#.#.##. ($56)
; $30 = .####.#. ($7A)
; $31 = ####.#.# ($F5)
; $32 = ...##... ($18)
; $33 = .###.##. ($76)
; $34 = ....##.# ($0D)
; $35 = .......# ($01)
; $36 = ###.#### ($EF)
; $37 = ....##.. ($0C)
; $38 = ....##.# ($0D)
; $39 = ....##.# ($0D)
; $3A = ###..... ($E0)
; $3B = .#...... ($40)
; $3C = ....#..# ($09)
; $3D = ....#..# ($09)
; $3E = ###.#.## ($EB)
; $3F = ###.#.## ($EB)
; $80 = #.##.... ($B0)
; $81 = ####...# ($F1)
; $82 = ##.###.# ($DD)
; $83 = ####.##. ($F6)
; $84 = #.###### ($BF)
; $85 = ####.#.# ($F5)
; $86 = #.#..... ($A0)
; $87 = #.###.#. ($BA)
; $88 = #.##..## ($B3)
; $89 = #######. ($FE)
; $8A = ..#....# ($21)
; $8B = ######## ($FF)
; $8C = #####..# ($F9)
; $8D = ##.#...# ($D1)
; $8E = ####...# ($F1)
; $8F = ..####.. ($3C)
; $90 = #####..# ($F9)
; $91 = ####...# ($F1)
; $92 = #.....## ($83)
; $93 = #####..# ($F9)
; $94 = #####..# ($F9)
; $95 = ######## ($FF)
; $96 = ####.... ($F0)
; $97 = ##.#..## ($D3)
; $98 = ####.#.# ($F5)
; $99 = ........ ($00)
; $9A = .......# ($01)
; $9B = ####..#. ($F2)
; $9C = ######## ($FF)
; $9D = ...####. ($1E)
; $9E = #.##...# ($B1)
; $9F = ##.###.. ($DC)
; $A0 = #.##.... ($B0)
; $A1 = #####..# ($F9)
; $A2 = ..#..... ($20)
; $A3 = #.#.##.. ($AC)
; $A4 = ..###### ($3F)
; $A5 = .......# ($01)
; $A6 = ###...#. ($E2)
; $A7 = #.###..# ($B9)
; $A8 = ##..#..# ($C9)
; $A9 = ..##...# ($31)
; $AA = ......#. ($02)
; $AB = .#.##..# ($59)
; $AC = .#..#..# ($49)
; $AD = ###.##.# ($ED)
; $AE = ..##..#. ($32)
; $AF = #.#.#..# ($A9)
; $B0 = #....#.# ($85)
; $B1 = ....#.#. ($0A)
; $B2 = ###..### ($E7)
; $B3 = #...#..# ($89)
; $B4 = ####..#. ($F2)
; $B5 = #######. ($FF)
; $B6 = ...#.... ($10)
; $B7 = ####..## ($F3)
; $B8 = ####..#. ($F2)
; $B9 = ####..#. ($F2)
; $BA = ...##### ($1F)
; $BB = #.###### ($BF)
; $BC = ####.##. ($F6)
; $BD = ####.##. ($F6)
; $BE = ...#.#.. ($14)
; $BF = ...#.#.. ($14)

; ======================================================================================================================================================

; ============
; Program Name
; ============
; The ZX81 program name consists of one or more non-inverted characters, with the last character inverted.

#ifdef MAKE_P81

        ORG  $4000

        DEFB _R
        DEFB _O
        DEFB _C
        DEFB _K
        DEFB _E
        DEFB _T
        DEFB _M
        DEFB _A
        DEFB _N + $80

#endif

; ======================================================================================================================================================


; ----------------
; System Variables
; ----------------

        ORG  $4009

VERSN:  DEFB $00           ; 16393               $4009
E_PPC:  DEFW $0064         ; 16394               $400A
D_FILE: DEFW L78CE         ; 16396               $400C
DF_CC:  DEFW L78CE+$0001   ; 16398               $400E
VARS:   DEFW L7BE7         ; 16400               $4010
DEST:   DEFW L7BE7         ; 16402               $4012
E_LINE: DEFW L7BE8         ; 16404               $4014
CH_ADD: DEFW L78BC-$0001   ; 16406               $4016
X_PTR:  DEFW $C000         ; 16408               $4018
STKBOT: DEFW L7BE7+$0001   ; 16410               $401A
STKEND: DEFW L7BE7+$0001   ; 16412               $401C
BERG:   DEFB $00           ; 16414               $401E
MEM:    DEFW MEMBOT        ; 16415               $401F
SPARE1: DEFB $06           ; 16417 - unused      $4021
DF_SZ:  DEFB $02           ; 16418               $4022
S_TOP:  DEFW $0064         ; 16419               $4023
LAST_K  DEFW $FFFF         ; 16421               $4025
DBOUNC: DEFB $FF           ; 16423               $4027
MARGIN: DEFB $37           ; 16424               $4028
NXTLIN: DEFW L78BC         ; 16425               $4029
OLDPPC: DEFW $FFFE         ; 16427               $402B
FLAGX:  DEFB $00           ; 16429               $402D
STRLEN: DEFW $CC66         ; 16430               $402E
T_ADDR: DEFW $0C8D         ; 16432               $4030
SEED:   DEFW $6E43         ; 16434               $4032
FRAMES: DEFW $D8A6         ; 16436               $4034
COORDS: DEFW $0000         ; 16438               $4036
PR_CC:  DEFB $BC           ; 16440               $4038
S_POSN: DEFB $21           ; 16441               $4039
        DEFB $18           ; 16442               $403A
CDFLAG: DEFB $40           ; 16443               $403B
PRBUFF: DEFW $0000, $0000  ; 16444               $403C
        DEFW $0000, $0000
        DEFW $0000, $0000
        DEFW $0000, $0000
        DEFW $0000, $0000
        DEFW $0000, $0000
        DEFW $0000, $0000
        DEFW $0000, $0000
        DEFB _NEWLINE      ; 16476               $405C
MEMBOT: DEFW $0000, $0000  ; 16477               $405D
        DEFW $0000, $0088
        DEFW $0000, $A084
        DEFW $0000, $8000
        DEFW $8080, $8080
        DEFW $8480, $9295
        DEFW $A690, $0000
        DEFW $0000
SPARE2: DEFB $00           ; 16507 - unused      $407B
SPARE3: DEFB $00           ; 16508 - unused      $407C

; ======================================================================================================================================================

; ------------
; BASIC Line 1
; ------------

; ===================
; Hi-Res Display File
; ===================

L407D:  DEFB 1 >> 8
        DEFB 1 & $FF
        DEFW L5942 - 1 - $
        DEFB _REM

;       <---0--><---1--><---2--><---3--><---4--><---5--><---6--><---7--><---8--><---9--><--10--><--11--><--12--><--13--><--14--><--15--><--16--><--17--><--18--><--19--><--20--><--21--><--22--><--23--><--24--><--25--><--26--><--27--><--28--><--29--><--30--><--31-->
;
;   0 - ################################################################################################################################################################################################################################################################
;   1 - ################################################################################################################################################################################################################################################################
;   2 - ################################################################################################################################################################################################################################################################
;   3 - ################################################################################################################################################################################################################################################################
;   4 - ################################################################################################################################################################################################################################################################
;   5 - ################################################################################################################################################################################################################################################################
;   6 - ################################################################################################################################################################################################################################################################
;   7 - ################################################################################################################################################################################################################################################################
;   8 - ################################################################################################################################################################################################################################################################
;   9 - ################################################################################################################################################################################################################################################################
;  10 - ################################################################################################################################################################################################################################################################
;  11 - ################################################################################################################################################################################################################################################################
;  12 - ################################################################################################################################################################################################################################################################
;  13 - ################################################################################################################################################################################################################################################################
;  14 - ################################################################################################################################################################################################################################################################
;  15 - ################################################################################################################################################################################################################################################################
;  16 - ################################################.................................................#..########..#.................................################........................................####....################################################
;  17 - ################################################..#############...#############...#############..#..########..#...#############...#############.################..####....####....#############...####..####..#.################################################
;  18 - ################################################.#............#..#............#..#...............#..########..#..#.....................#........################.#............#..#............#..#......####..#.################################################
;  19 - ################################################.#............#..#............#..#...............#..########..#..#.....................#........################.#............#..#............#..#......####..#.################################################
;  20 - ################################################.#..########..#..#..########..#..#..############.#..####.........#..##################.#..######################.#..##.....#..#..#..########..#..#..##.....#..#.################################################
;  21 - ################################################.#..########..#..#..########..#..#..############.#..####.........#..##################.#..######################.#..##.....#..#..#..########..#..#..##.....#..#.################################################
;  22 - ################################################.#..########..#..#..########..#..#..############.#.............#.#..........##########.#..######################.#..##.....#..#..#..########..#..#..##.....#..#.################################################
;  23 - ################################################.#............#..#..########..#..#..############..##############..#########.##########.#..######################.#..##.....#..#..#............#..#..##.....#..#.################################################
;  24 - ################################################..#############..#..########..#..#..############.#......########.#..........##########.#..######################.#..##.....#..#...#############..#..##.....#..#.################################################
;  25 - ################################################.#...............#..########..#..#..############.#.............#.#..........##########.#..######################.#..##.....#..#..#............#..#..##.....#..#.################################################
;  26 - ################################################.#...............#..########..#..#..############.#..####.........#..##################.#..######################.#..##.....#..#..#............#..#..##.....#..#.################################################
;  27 - ################################################.#..####...#####.#..########..#..#..############.#..####.........#..##################.#..######################.#..##.....#..#..#..########..#..#..##.....#..#.################################################
;  28 - ################################################.#..####...#####.#............#..#...............#..########..#..#..............######.#..######################.#..########..#..#..########..#..#..####......#.################################################
;  29 - ################################################.#..#####....#.#.#............#..#...............#..########..#..#..............######.#..######################.#..########..#..#..########..#..#..####......#.################################################
;  30 - ################################################.#..#####....#.#..#############...#############..#..########..#...#############.######.#..######################.#..########..#..#..########..#..#..####..####..################################################
;  31 - ################################################....#####....#.#.................................#..########..#.................######.#..######################....########........########..#.....####........################################################
;  32 - ################################################################################################################################################################################################################################################################
;  33 - ################################################################################################################################################################################################################################################################
;  34 - ################################################################################################################################################################################################################################################################
;  35 - ################################################################################################################################################################################################################################################################
;  36 - ################################################################################################################################################################################################################################################################
;  37 - ################################################################################################################################################################################################################################################################
;  38 - ################################################################################################################################################################################################################################################################
;  39 - ################################################################################################################################################################################################################################################################
;  40 - ################################################################################################################################################################################################################################################################
;  41 - ################################################################################################################################################################################################################################################################
;  42 - ################################################################################################################################################################################################################################################################
;  43 - ################################################################################################################################################################################################################################################################
;  44 - ################################################################################################################################################################################################################################################################
;  45 - ################################################################################################################################################################################################################################################################
;  46 - ################################################################################################################################################################################################################################################################
;  47 - ################################################################################################################################################################################################################################################################
;  48 - ##########################################################################################################################.#..##################################################################################################################################
;  49 - #########################################################################################################################.#..##.################################################################################################################################
;  40 - #########################################################################################################################.#.##..#.##############################################################################################################################
;  51 - ##########################################################################################################################.#..##.#.#############################################################################################################################
;  52 - ##################################################################################################################.#..##.###.##..#.##..#########################################################################################################################
;  53 - #################################################################################################################.#.#..##.##..###.#.#..#########################################################################################################################
;  54 - ################################################################################################################.#.#.##..#.#..##.#..#..##.######################################################################################################################
;  55 - ################################################################################################################.#.#..##.#..##..###.#.##.#.#####################################################################################################################
;  56 - #################################################################################################################.##.##.####.#.#....#.#..#..####################################################################################################################
;  57 - ##################################################################################################################..###..#.#.##.##..##.#.#.#####################################################################################################################
;  58 - #########################################################################################################################.#.#..##.##.##..#..####################################################################################################################
;  59 - ##################################################################################################################.#..####..##.###..#..####.####################################################################################################################
;  60 - ################################################################################################################.#.#..##..##..#..#.##..#.#..####################################################################################################################
;  61 - #################################################################################################################.#.##..##.#..##.#.#..###.######################################################################################################################
;  62 - ################################################################################################################.#.#.##...#.##....##.##....#####################################################################################################################
;  63 - #################################################################################################################.#.#..#.#.#.##.##..#..#.#.#####################################################################################################################
;  64 - ###############################################################################################################.#.#..##.#.#..##.#.#..##..#.#####################################################################################################################
;  65 - ###############################################################################################################.##..##.#.#.##..#.#.##..##.######################################################################################################################
;  66 - ################################################################################################################.#.#.##.#.##.##..#..#..#..######################################################################################################################
;  67 - ####################################################################################################################.#.#.#..#..##.##.##.##.#####################################################################################################################
;  68 - ###############################################################################################################.########.#.#.##..#.#.##..#.#####################################################################################################################
;  69 - #################################################################################################################.#..##.#.#.#..##.#.#..###.#####################################################################################################################
;  70 - ##################################################################################################################..##.#..##..#.#.##.##.#.######################################################################################################################
;  71 - ################################################################################################################.#.#.##.##..##.#.#..#..#########################################################################################################################
;  72 - ################################################################################################################.#.##..#.#.##..##.#..##.########################################################################################################################
;  73 - ################################################################################################################.#.#####..##..#.##..##.#########################################################################################################################
;  74 - #################################################################################################################.##.##.#.##.##.###.#.##########################################################################################################################
;  75 - ##################################################################################################################..##.#.###.##..#.#..##########################################################################################################################
;  76 - ##################################################################################################################..#..#..##..#..#.#############################################################################################################################
;  77 - ####################################################################################################################.#.##.###.#.#.##############################################################################################################################
;  78 - ##################################################################################################################.####.##....##..#....#########################################################################################################################
;  79 - ########################################################################################################################.........#......########################################################################################################################
;  80 - #######################################################################################################################........#..#.......######################################################################################################################
;  81 - #######################################################################################################################..........#........######################################################################################################################
;  82 - #####################################################################################################################..#.......#.#.........#####################################################################################################################
;  83 - ####################################################################################################################...........#.#.........#####################################################################################################################
;  84 - ####################################################################################################################......##..#..#..........####################################################################################################################
;  85 - ####################################################################################################################.....###.##..#..##......####################################################################################################################
;  86 - ####################################################################################################################......##.##..#..###....#####################################################################################################################
;  87 - ##################################################################################################################.###.....#..#...#.###....#####################################################################################################################
;  88 - #################################################################################################################.#.##....#....#..#.......######################################################################################################################
;  89 - ##########################################################################################################################......##.#####.#.#####################################################################################################################
;  90 - ################################################################################################################........................########################################################################################################################
;  91 - ################################################################################################################..................................##############################################################################################################
;  92 - ###############################################################################################################.########................########...#############################################################################################################
;  93 - ###############################################################################################################.....######..........########.......#############################################################################################################
;  94 - ################################################################################################################.......####........########.......##############################################################################################################
;  95 - ################################################################################################################........####......########......################################################################################################################
;  96 - ############################################################################################################....##......####.......#####.......#..##############################################################################################################
;  97 - ###########################################################################################################.......####....####....####.......##....#############################################################################################################
;  98 - ##########################################################################################################.............#...####..#####....####......############################################################################################################
;  99 - ############################################################################################################........#.#..#..########..####........##############################################################################################################
; 100 - ###########################################################################################################.........##.#.#..#######.#.##.#........##############################################################################################################
; 101 - ##########################################################################################################..........##..#.#..##.###...#..#.........#############################################################################################################
; 102 - ########################################################################################################...........#..#.#.#..##.##..###.#.#.........############################################################################################################
; 103 - ########################################################################################################...........###.#..#...####..##.#.#.........#############################################################################################################
; 104 - ##########################################################################################################..........##....#...##..##.##.#.#.......##############################################################################################################
; 105 - ###########################################################################################################........#..#..#..#..#.#.##..#..#.........############################################################################################################
; 106 - ############################################################################################################........##...#..#..###.#...##.##........############################################################################################################
; 107 - #############################################################################################################..#...#..#..#.##..##.#..##...#........#############################################################################################################
; 108 - ##############################################################################################################.####.##.#.#.##..#..####..#.##.......#############################################################################################################
; 109 - ###################################################################################################################.#.##.#...#.#.#...#.#.#..####..##############################################################################################################
; 110 - ###################################################################################################################...#.###....#.#...##.#.######################################################################################################################
; 111 - ###################################################################################################################.#.###.#.....#.....##.#.#####################################################################################################################
; 112 - ################################################################################################################..##.##.#.#.........#.#.#.##....################################################################################################################
; 113 - ################################################################################################################..#.##...#..........###.###.....################################################################################################################
; 114 - ##################################################################################################################..#..##.#.........#.#.#.##....################################################################################################################
; 115 - ##################################################################################################################.#..##.#..........##.##.##...#################################################################################################################
; 116 - ##################################################################################################################..##.##.#..........##..#..####################################################################################################################
; 117 - ###################################################################################################################.#.##.#..........##.#...#####################################################################################################################
; 118 - ####################################################################################################################.#.##.#.........#..#.#.#####################################################################################################################
; 119 - ######################################################################################################################.#...##.......###.#.######################################################################################################################
; 120 - ################################################################################################################################################################################################################################################################
; 121 - ################################################################################################################################################################################################################################################################
; 122 - ################################################################################################################################################################################################################################################################
; 123 - ################################################################################################################################################################################################################################################################
; 124 - ################################################################################################################################################################################################################################################################
; 125 - ################################################################################################################################################################################################################################################################
; 126 - ################################################################################################################################################################################################################################################################
; 127 - ################################################################################################################################################################################################################################################################
; 128 - ################################################################################################################################################################################################################################################################
; 129 - ################################################################################################################################################################################################################################################################
; 130 - ################################################################################################################################################################################################################################################################
; 131 - ################################################################################################################################################################################################################################################################
; 132 - ################################################################################################################################################################################################################################################################
; 133 - ################################################################################################################################################################################################################################################################
; 134 - ################################################################################################################################################################################################################################################################
; 135 - ################################################################################################################################################################################################################################################################
; 136 - ################################################################################################...............................#########..........####..........################################################################################################
; 137 - ################################################################################################...............................#########..........####..........################################################################################################
; 138 - ################################################################################################..######..####....####...#..#..############..###..####....######################################################################################################
; 139 - ################################################################################################..######..####....####...#..#..############..###............####################################################################################################
; 140 - ################################################################################################....####..........####...#..#..############..###............####################################################################################################
; 141 - ################################################################################################....####..........####....####..###########..###..####....######################################################################################################
; 142 - ################################################################################################..######..##...#..........####..###########..###..####..........################################################################################################
; 143 - ################################################################################################..######..####............####..###########..###..####..........################################################################################################
; 144 - ################################################################################################################################################################################################################################################################
; 145 - ################################################################################################################################################################################################################################################################
; 146 - ################################################################################################################################################################################################################################################################
; 147 - ################################################################################################################################################################################################################################################################
; 148 - ################################################################################################################################################################################################################################################################
; 149 - ################################################################################################################################################################################################################################################################
; 150 - ################################################################################################################################################################################################################################################################
; 151 - ################################################################################################################################################################################################################################################################
; 152 - ################################################################################################################################################################################################################################################################
; 153 - ################################################################################################################################################################################################################################################################
; 154 - ################################################################################################################################################################################################################################################################
; 155 - ################################################################################################################################################################################################################################################################
; 156 - ################################################################################################################################################################################################################################################################
; 157 - ################################################################################################################################################################################################################################################################
; 158 - ################################################################################################################################################################################################################################################################
; 159 - ################################################################################################################################################################################################################################################################
; 160 - ################################################################.......#.......#.......#..........#....#.......#########.......#.......#.......#..##...#.......#.......#.......#...#############################################################################
; 161 - ################################################################.......#.......#.......#..........#....#.......#########.......#.......#.......#..##...#.......#.......#.......#...#############################################################################
; 162 - ################################################################.......#.......#.......#..........#....#.......#########.......#.......#.......#..##...#.......#.......#.......#...#############################################################################
; 163 - ################################################################.......#.......#.......#..........#....#.......#########.......#.......#.......#..##...#.......#.......#.......#...#############################################################################
; 164 - ################################################################...#####..##...#...#####...#..#...#....#...#############...#####..##...#...#####..##...#...#####..##...#...#####...#############################################################################
; 165 - ################################################################...#####..##...#...#####...#..#...#....#...#############...#####..##...#...#####..##...#...#####..##...#...#####...#############################################################################
; 166 - ################################################################...#####..##...#.......#...#..#...#....#...#############...#####..##...#...#####.......#....####..##...#....####...#############################################################################
; 167 - ################################################################...#####..##...#.......#...#..#...#....#...#############...#####..##...#...#####....####....####..##...#....####...#############################################################################
; 168 - ################################################################...#####..##...#.......#...#..#...#....#...#############...#####..##...#...#####....####....####..##...#....####...#############################################################################
; 169 - ################################################################...#####..##...#.......#...#..#...#....#...#############...#####..##...#...#####.......#....####.......#....####...#############################################################################
; 170 - ################################################################...#####..##...#####...#...####...#....#...#############...#####..##...#...#####..##...#...#####....####...#####...#############################################################################
; 171 - ################################################################...#####..##...#####...#...####...#....#...#############...#####..##...#...#####..##...#...#####....####...#####...#############################################################################
; 172 - ################################################################.......#.......#.......#...####...#....#.......#########.......#.......#.......#..##...#.......#..##...#.......#.......#########################################################################
; 173 - ################################################################.......#.......#.......#...####...#....#.......#########.......#.......#.......#..##...#.......#..##...#.......#.......#########################################################################
; 174 - ################################################################.......#.......#.......#...####...#....#.......#########.......#.......#.......#..##...#.......#..##...#.......#.......#########################################################################
; 175 - ################################################################.......#.......#.......#...####...#....#.......#########.......#.......#.......#..##...#.......#..##...#.......#.......#########################################################################
; 176 - ################################################################################################################################################################################################################################################################
; 177 - ################################################################################################################################################################################################################################################################
; 178 - ################################################################################################################################################################################################################################################################
; 179 - ################################################################################################################################################################################################################################################################
; 180 - ################################################################################################################################################################################################################################################################
; 181 - ################################################################################################################################################################################################################################################################
; 182 - ################################################################################################################################################################################################################################################################
; 183 - ################################################################################################################################################################################################################################################################
; 184 - ################################################################################################################################################################################################################################################################
; 185 - ################################################################################################################################################################################################################################################################
; 186 - ################################################################################################################################################################################################################################################################
; 187 - ################################################################################################################################################################################################################################################################
; 188 - ################################################################################################################################################################################################################################################################
; 189 - ################################################################################################################################################################################################################################################################
; 190 - ################################################################################################################################################################################################################################################################
; 191 - ################################################################################################################################################################################################################################################################

L4082:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 0
L40A3:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 1
L40C4:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 2
L40E5:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 3
L4106:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 4
L4127:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 5
L4148:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 6
L4169:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 7
L418A:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 8
L41AB:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 9
L41CC:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 10
L41ED:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 11
L420E:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 12
L422F:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 13
L4250:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 14
L4271:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 15
L4292:  DEFB $19, $19, $19, $19, $19, $19, $0B, $0B, $0B, $0B, $0B, $0B, $00, $B9, $0B, $0B, $0B, $0B, $19, $19, $0B, $0B, $0B, $0B, $0B, $96, $19, $19, $19, $19, $19, $19, $C9        ; Line 16
L42B3:  DEFB $19, $19, $19, $19, $19, $19, $A4, $25, $A4, $25, $A4, $25, $00, $B9, $A4, $1A, $A4, $1A, $19, $19, $8F, $8F, $A4, $25, $8F, $9B, $19, $19, $19, $19, $19, $19, $C9        ; Line 17
L42D4:  DEFB $19, $19, $19, $19, $19, $19, $3B, $AA, $04, $AA, $04, $0B, $00, $B9, $3B, $0B, $09, $0B, $19, $19, $3B, $AA, $3B, $AA, $3B, $9B, $19, $19, $19, $19, $19, $19, $C9        ; Line 18
L42F5:  DEFB $19, $19, $19, $19, $19, $19, $3B, $AA, $04, $AA, $04, $0B, $00, $B9, $3B, $0B, $09, $0B, $19, $19, $3B, $AA, $3B, $AA, $3B, $9B, $19, $19, $19, $19, $19, $19, $C9        ; Line 19
L4316:  DEFB $19, $19, $19, $19, $19, $19, $20, $9B, $00, $9B, $00, $19, $00, $0B, $20, $19, $2A, $A4, $19, $19, $08, $2D, $20, $9B, $08, $2D, $19, $19, $19, $19, $19, $19, $C9        ; Line 20
L4337:  DEFB $19, $19, $19, $19, $19, $19, $20, $9B, $00, $9B, $00, $19, $00, $0B, $20, $19, $2A, $A4, $19, $19, $08, $2D, $20, $9B, $08, $2D, $19, $19, $19, $19, $19, $19, $C9        ; Line 21
L4358:  DEFB $19, $19, $19, $19, $19, $19, $20, $9B, $00, $9B, $00, $19, $04, $09, $3B, $16, $2A, $A4, $19, $19, $08, $2D, $20, $9B, $08, $2D, $19, $19, $19, $19, $19, $19, $C9        ; Line 22
L4379:  DEFB $19, $19, $19, $19, $19, $19, $3B, $AA, $00, $9B, $00, $19, $A4, $19, $A4, $36, $2A, $A4, $19, $19, $08, $2D, $3B, $AA, $08, $2D, $19, $19, $19, $19, $19, $19, $C9        ; Line 23
L439A:  DEFB $19, $19, $19, $19, $19, $19, $A4, $25, $00, $9B, $00, $19, $04, $19, $3B, $16, $2A, $A4, $19, $19, $08, $2D, $A4, $25, $08, $2D, $19, $19, $19, $19, $19, $19, $C9        ; Line 24
L43BB:  DEFB $19, $19, $19, $19, $19, $19, $3B, $0B, $00, $9B, $00, $19, $04, $09, $3B, $16, $2A, $A4, $19, $19, $08, $2D, $3B, $AA, $08, $2D, $19, $19, $19, $19, $19, $19, $C9        ; Line 25
L43DC:  DEFB $19, $19, $19, $19, $19, $19, $3B, $0B, $00, $9B, $00, $19, $00, $0B, $20, $19, $2A, $A4, $19, $19, $08, $2D, $3B, $AA, $08, $2D, $19, $19, $19, $19, $19, $19, $C9        ; Line 26
L43FD:  DEFB $19, $19, $19, $19, $19, $19, $20, $BA, $00, $9B, $00, $19, $00, $0B, $20, $19, $2A, $A4, $19, $19, $08, $2D, $20, $9B, $08, $2D, $19, $19, $19, $19, $19, $19, $C9        ; Line 27
L441E:  DEFB $19, $19, $19, $19, $19, $19, $20, $BA, $04, $AA, $04, $0B, $00, $B9, $3B, $0B, $2A, $A4, $19, $19, $20, $9B, $20, $9B, $20, $AA, $19, $19, $19, $19, $19, $19, $C9        ; Line 28
L443F:  DEFB $19, $19, $19, $19, $19, $19, $20, $B0, $04, $AA, $04, $0B, $00, $B9, $3B, $0B, $2A, $A4, $19, $19, $20, $9B, $20, $9B, $20, $AA, $19, $19, $19, $19, $19, $19, $C9        ; Line 29
L4460:  DEFB $19, $19, $19, $19, $19, $19, $20, $B0, $A4, $25, $A4, $25, $00, $B9, $A4, $1A, $2A, $A4, $19, $19, $20, $9B, $20, $9B, $20, $8F, $19, $19, $19, $19, $19, $19, $C9        ; Line 30
L4481:  DEFB $19, $19, $19, $19, $19, $19, $16, $B0, $0B, $0B, $0B, $0B, $00, $B9, $0B, $0B, $2A, $A4, $19, $19, $16, $96, $16, $9B, $16, $0B, $19, $19, $19, $19, $19, $19, $C9        ; Line 31
L44A2:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 32
L44C3:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 33
L44E4:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 34
L4505:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 35
L4526:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 36
L4547:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 37
L4568:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 38
L4589:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 39
L45AA:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 40
L45CB:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 41
L45EC:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 42
L460D:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 43
L462E:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 44
L464F:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 45
L4670:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 46
L4691:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 47
L46B2:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $97, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 48
L46D3:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2B, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 49
L46F4:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $A3, $84, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 50
L4715:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $97, $06, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 51
L4736:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $97, $33, $AB, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 52
L4757:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $AF, $88, $AF, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 53
L4778:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2F, $23, $AC, $84, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 54
L4799:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $23, $08, $3E, $06, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 55
L47BA:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2C, $31, $18, $00, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 56
L47DB:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $29, $2F, $2E, $06, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 57
L47FC:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $AF, $2C, $20, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 58
L481D:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $97, $2E, $A8, $36, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 59
L483E:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $23, $AE, $AB, $00, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 60
L485F:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $A3, $97, $23, $84, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 61
L4880:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2F, $17, $28, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 62
L48A1:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $AF, $2F, $A8, $06, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 63
L48C2:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $1A, $2B, $2B, $2B, $06, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 64
L48E3:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $1A, $2E, $AB, $AB, $84, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 65
L4904:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2F, $2C, $AC, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 66
L4925:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $31, $AC, $2C, $22, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 67
L4946:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $1A, $19, $2F, $2F, $06, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 68
L4967:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2B, $AF, $AF, $22, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 69
L4988:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2E, $AE, $2C, $84, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 70
L49A9:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2F, $2E, $AC, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 71
L49CA:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $AB, $AB, $2B, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 72
L49EB:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $06, $AE, $2E, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 73
L4A0C:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2C, $2C, $3E, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 74
L4A2D:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2E, $33, $23, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 75
L4A4E:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $A8, $AE, $06, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 76
L4A6F:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $31, $87, $84, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 77
L4A90:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $0A, $0F, $8A, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 78
L4AB1:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $0B, $04, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 79
L4AD2:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $1A, $09, $A2, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 80
L4AF3:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $1A, $0B, $04, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 81
L4B14:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $A1, $09, $04, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 82
L4B35:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $96, $09, $04, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 83
L4B56:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $96, $AE, $04, $16, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 84
L4B77:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $96, $33, $08, $16, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 85
L4B98:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $96, $28, $1E, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 86
L4BB9:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $9F, $2D, $0D, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 87
L4BDA:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $A3, $8A, $A2, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 88
L4BFB:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $24, $22, $06, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 89
L4C1C:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $0B, $0B, $0B, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 90
L4C3D:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $0B, $0B, $0B, $0B, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 91
L4C5E:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $1A, $19, $0B, $0B, $19, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 92
L4C7F:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $1A, $16, $24, $16, $96, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 93
L4CA0:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $09, $3A, $BA, $3A, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 94
L4CC1:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $0B, $96, $A4, $24, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 95
L4CE2:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $96, $24, $96, $BA, $09, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 96
L4D03:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $3A, $8F, $8F, $8F, $0C, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 97
L4D24:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $24, $09, $9D, $12, $8F, $16, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 98
L4D45:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $96, $18, $00, $B7, $24, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 99
L4D66:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $3A, $38, $00, $3E, $04, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 100
L4D87:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $24, $37, $2B, $A6, $04, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 101
L4DA8:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $0B, $2D, $2B, $29, $86, $16, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 102
L4DC9:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $0B, $26, $1F, $2E, $04, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 103
L4DEA:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $24, $37, $1F, $28, $86, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 104
L4E0B:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $3A, $2D, $AC, $AB, $A2, $16, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 105
L4E2C:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $96, $37, $AC, $8D, $80, $16, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 106
L4E4D:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $8C, $2D, $AB, $2B, $A2, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 107
L4E6E:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2A, $AD, $AB, $8F, $A0, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 108
L4E8F:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $3E, $07, $07, $00, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 109
L4EB0:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $A6, $1D, $27, $84, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 110
L4ED1:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $3E, $86, $92, $06, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 111
L4EF2:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $28, $86, $05, $80, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 112
L4F13:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $17, $04, $11, $3A, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 113
L4F34:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $A8, $86, $18, $80, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 114
L4F55:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $97, $04, $1B, $9E, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 115
L4F76:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2E, $86, $21, $00, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 116
L4F97:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $3E, $04, $34, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 117
L4FB8:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $31, $86, $3C, $06, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 118
L4FD9:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $2A, $32, $01, $84, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 119
L4FFA:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 120
L501B:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 121
L503C:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 122
L505D:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 123
L507E:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 124
L509F:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 125
L50C0:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 126
L50E1:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 127
L5102:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 128
L5123:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 129
L5144:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 130
L5165:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 131
L5186:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 132
L51A7:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 133
L51C8:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 134
L51E9:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 135
L520A:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $0B, $0B, $0B, $09, $19, $0B, $8F, $0B, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 136
L522B:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $0B, $0B, $0B, $09, $19, $0B, $8F, $0B, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 137
L524C:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $A4, $8F, $8F, $AC, $19, $B2, $8F, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 138
L526D:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $A4, $8F, $8F, $AC, $19, $B2, $0B, $16, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 139
L528E:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $16, $0B, $8F, $AC, $19, $B2, $0B, $16, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 140
L52AF:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $16, $0B, $8F, $8F, $19, $B2, $8F, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 141
L52D0:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $A4, $A9, $0B, $8F, $19, $B2, $8F, $0B, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 142
L52F1:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $A4, $8F, $0B, $8F, $19, $B2, $8F, $0B, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 143
L5312:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 144
L5333:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 145
L5354:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 146
L5375:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 147
L5396:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 148
L53B7:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 149
L53D8:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 150
L53F9:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 151
L541A:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 152
L543B:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 153
L545C:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 154
L547D:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 155
L549E:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 156
L54BF:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 157
L54E0:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 158
L5501:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 159
L5522:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $09, $09, $09, $0B, $8A, $09, $19, $09, $09, $09, $A9, $09, $09, $09, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 160
L5543:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $09, $09, $09, $0B, $8A, $09, $19, $09, $09, $09, $A9, $09, $09, $09, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 161
L5564:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $09, $09, $09, $0B, $8A, $09, $19, $09, $09, $09, $A9, $09, $09, $09, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 162
L5585:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $09, $09, $09, $0B, $8A, $09, $19, $09, $09, $09, $A9, $09, $09, $09, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 163
L55A6:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $BA, $A9, $BA, $2D, $8A, $BA, $19, $BA, $A9, $BA, $A9, $BA, $A9, $BA, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 164
L55C7:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $BA, $A9, $BA, $2D, $8A, $BA, $19, $BA, $A9, $BA, $A9, $BA, $A9, $BA, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 165
L55E8:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $BA, $A9, $09, $2D, $8A, $BA, $19, $BA, $A9, $BA, $09, $16, $A9, $16, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 166
L5609:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $BA, $A9, $09, $2D, $8A, $BA, $19, $BA, $A9, $BA, $16, $16, $A9, $16, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 167
L562A:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $BA, $A9, $09, $2D, $8A, $BA, $19, $BA, $A9, $BA, $16, $16, $A9, $16, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 168
L564B:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $BA, $A9, $09, $2D, $8A, $BA, $19, $BA, $A9, $BA, $09, $16, $09, $16, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 169
L566C:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $BA, $A9, $8E, $9D, $8A, $BA, $19, $BA, $A9, $BA, $A9, $BA, $16, $BA, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 170
L568D:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $BA, $A9, $8E, $9D, $8A, $BA, $19, $BA, $A9, $BA, $A9, $BA, $16, $BA, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 171
L56AE:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $09, $09, $09, $9D, $8A, $09, $19, $09, $09, $09, $A9, $09, $A9, $09, $09, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 172
L56CF:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $09, $09, $09, $9D, $8A, $09, $19, $09, $09, $09, $A9, $09, $A9, $09, $09, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 173
L56F0:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $09, $09, $09, $9D, $8A, $09, $19, $09, $09, $09, $A9, $09, $A9, $09, $09, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 174
L5711:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $09, $09, $09, $9D, $8A, $09, $19, $09, $09, $09, $A9, $09, $A9, $09, $09, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 175
L5732:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 176
L5753:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 177
L5774:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 178
L5795:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 179
L57B6:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 180
L57D7:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 181
L57F8:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 182
L5819:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 183
L583A:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 184
L585B:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 185
L587C:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 186
L589D:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 187
L58BE:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 188
L58DF:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 189
L5900:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 190
L5921:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $C9        ; Line 191

; -------------------
; End of BASIC Line 1
; -------------------

L5942:  DEFB _NEWLINE

; ------------
; BASIC Line 2
; ------------

L5943:  DEFB 2 >> 8
        DEFB 2 & $FF
        DEFW L6426 - 1 - $
        DEFB _REM

; -------------------------------------
; 'Execute' Line of Hi-Res Display File
; -------------------------------------
; This is called from the hi-res display driver at L612F.

L5948:  JP   (HL)                       ; 'Execute' the display file line.

; ===============
; Sprite Graphics
; ===============

; ------------------------------------
; Player Left Column Background Buffer
; -----------------------------------

L5949:  DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00

; ---------------------------
; Player Facing Right Graphic
; ---------------------------
; The background buffer above forms the first column of the graphic, allowing the player graphic
; to always be drawn as 2 columns wide.
;
; --------#.....##
; --------....####
; --------.......#
; --------#.....##
; --------###.####
; --------#.....##
; --------#.....##
; --------#.....##
; --------#.....##
; --------#.....##
; --------###.####
; --------###.####
; --------###.####
; --------###.####
; --------###..###
; --------###..###

L5959:  DEFB $92, $16, $09, $92, $36, $92, $92, $92, $92, $92, $36, $36, $36, $36, $B2, $B2

; -------------------------------
; Player Walking Right #1 Graphic
; -------------------------------
;
; #######.....####
; #######...######
; #######.....####
; #######.....####
; ########..######
; #######.....####
; #######.###..###
; #######...#...##
; #######.....#..#
; #######.....####
; ########..######
; #######.....####
; #######.###.####
; #####..####.####
; ####..#####.####
; #####..####..###

L5969:  DEFB $89, $89, $89, $89, $19, $89, $89, $89, $89, $89, $19, $89, $89, $8C, $B7, $8C
        DEFB $16, $A4, $16, $16, $A4, $16, $B2, $1F, $03, $16, $A4, $16, $36, $36, $36, $B2

; -------------------------------
; Player Walking Right #2 Graphic
; -------------------------------
;
; ####......######
; ####....########
; ####.......#####
; ####......######
; #######.########
; ####...........#
; ####........####
; ####....########
; ####......######
; ####......######
; #######.########
; ##........######
; ##....##..######
; ##.#######.#####
; ##.########.####
; ###########..###

L5989:  DEFB $96, $96, $96, $96, $89, $96, $96, $96, $96, $96, $89, $24, $0F, $22, $22, $19
        DEFB $A4, $19, $BA, $A4, $19, $09, $16, $19, $A4, $A4, $19, $A4, $A4, $22, $36, $B2

; -------------------------------
; Player Walking Right #3 Graphic
; -------------------------------
;
; ###.....########
; ###....#########
; ###.......######
; ###.....########
; #####..#########
; ###.....########
; ###.......######
; ###..###..######
; ###.....##.#####
; ###.....########
; #####..#########
; ###.......######
; ##..###...######
; ...#####..######
; ..########.#####
; ...#############

L59A9:  DEFB $3A, $1D, $3A, $3A, $8C, $3A, $3A, $B2, $3A, $3A, $8C, $3A, $29, $BA, $A4, $BA
        DEFB $19, $19, $A4, $19, $19, $19, $A4, $A4, $22, $19, $19, $A4, $A4, $A4, $22, $19

; ------------------------------
; Player Walking Left #1 Graphic
; ------------------------------
;
; ########....####
; ########.#..####
; #######.....####
; ########....####
; #########.######
; ########....####
; #######.....####
; #######.###.####
; #####..#....####
; ########....####
; #########.######
; #######.....####
; #######.####..##
; #######.#####..#
; #####..########.
; #############..#

L59C9:  DEFB $19, $19, $89, $19, $19, $19, $89, $89, $8C, $19, $19, $89, $89, $89, $8C, $19
        DEFB $16, $00, $16, $16, $84, $16, $16, $36, $16, $16, $84, $16, $B7, $A1, $89, $A1

; ------------------------------
; Player Walking Left #2 Graphic
; ------------------------------
;
; #######.....####
; ########....####
; ####........####
; #######.....####
; ########..######
; ##..........####
; ##..........####
; ########....####
; #######.....####
; #######.....####
; ########..######
; #######........#
; #######........#
; #####..#######.#
; ####..##########
; ###..###########

L59E9:  DEFB $1A, $19, $96, $1A, $19, $24, $24, $19, $1A, $1A, $19, $1A, $1A, $A1, $B7, $B2
        DEFB $16, $16, $16, $16, $A4, $16, $16, $16, $16, $16, $A4, $09, $09, $2A, $19, $19

; ------------------------------
; Player Walking Left #3 Graphic
; ------------------------------
;
; ####......######
; ####......######
; ###.......######
; ####......######
; #######.########
; ####......######
; ###.......######
; ##.###....######
; ...##.....######
; ####......######
; #######.########
; ####......######
; ####..##..######
; ####..##...#####
; ####..#####.####
; ##....##...#####

L5A09:  DEFB $96, $96, $3A, $96, $89, $96, $3A, $9F, $32, $96, $89, $96, $B7, $B7, $B7, $0F
        DEFB $A4, $A4, $A4, $A4, $19, $A4, $A4, $A4, $A4, $A4, $19, $A4, $A4, $BA, $36, $BA

; --------------------------
; Player Facing Left Graphic
; --------------------------
; The background buffer below forms the second column of the graphic, allowing the player graphic
; to always be drawn as 2 columns wide.
;
; ##....##--------
; ###....#--------
; .......#--------
; ##....##--------
; ####..##--------
; ###....#--------
; ##......--------
; ##......--------
; ##......--------
; ###....#--------
; ####..##--------
; ####..##--------
; ####..##--------
; ####..##--------
; ##....##--------
; ##....##--------

L5A29:  DEFB $0F, $1D, $09, $0F, $B7, $1D, $24, $24, $24, $1D, $B7, $B7, $B7, $B7, $0F, $0F

; -------------------------------------
; Player Right Column Background Buffer
; -------------------------------------

L5A39:  DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00

; --------------------------
; Player Climbing #1 Graphic
; --------------------------
;
; #.######
; ..#...##
; ..#...##
; ..#...##
; ..######
; #.....##
; .......#
; .#..##..
; ##..##.#
; ###..###
; #.....##
; #.###..#
; ...###.#
; #####..#
; #####..#
; ####....

L5A49:  DEFB $BB, $1F, $1F, $1F, $A4, $92, $09, $08, $2E, $B2, $92, $A7, $26, $8C, $8C, $96

; --------------------------
; Player Climbing #2 Graphic
; --------------------------
;
; #######.
; #.#.....
; #.#.....
; #.#.....
; ####.##.
; ##......
; .......#
; #.##..##
; ###..###
; ###..###
; ##....##
; #.###..#
; #.###..#
; #.######
; #.######
; #.######

L5A59:  DEFB $25, $86, $86, $86, $83, $24, $09, $88, $B2, $B2, $0F, $A7, $A7, $84, $84, $A4

; ----------------------------------------------
; Player Rocket Pack Right #1 (Column 1) Graphic
; ----------------------------------------------
;
; ########
; ########
; ########
; ########
; #######.
; ######.#
; ######.#
; ######.#
; ######.#
; #######.
; ######.#
; ########

L5A69:  DEFB $19, $19, $19, $19, $89, $2A, $2A, $2A, $2A, $89, $2A, $19

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; ########
; ########
; ########
; ########

L5A75:  DEFB $19, $19, $19, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; ######.#
; ######.#
; #######.
; #######.

L5A79:  DEFB $2A, $2A, $89, $89

; ----------------------------------------------
; Player Rocket Pack Right #1 (Column 2) Graphic
; ----------------------------------------------
;
; ##....##
; ##..##.#
; ##....##
; ###..###
; ##....##
; .#.#..##
; .#.#..##
; .#.#..##
; .#.#..##
; ##....##
; .#.#..##
; ###..###

L5A7D:  DEFB $0F, $2E, $0F, $B2, $0F, $23, $23, $23, $23, $0F, $23, $B2

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; ###..###
; ....####
; ..######
; ########

L5A89:  DEFB $B2, $16, $A4, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; ###..###
; ###..###
; ###..###
; ###....#

L5A8D:  DEFB $B2, $B2, $B2, $1D

; ----------------------------------------------
; Player Rocket Pack Right #2 (Column 1) Graphic
; ----------------------------------------------
;
; ########
; ########
; ########
; ########
; #####..#
; ####...#
; ####...#
; ####...#
; ####...#
; ########
; ####...#
; ########

L5A91:  DEFB $19, $19, $19, $19, $8C, $81, $81, $81, $81, $19, $81, $19

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; ########
; ######.#
; ######.#
; ########
        
L5A9D:  DEFB $19, $2A, $2A, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; ####..##
; ####..##
; ########
; ####..##

L5AA1:  DEFB $B7, $B7, $19, $B7

; ----------------------------------------------
; Player Rocket Pack Right #2 (Column 2) Graphic
; ----------------------------------------------
;
; ....####
; ...#####
; ....####
; #.######
; ....####
; .#..####
; .#..####
; .#..####
; .#..####
; ....####
; ....####
; #.######

L5AA5:  DEFB $16, $BA, $16, $84, $16, $00, $00, $00, $00, $16, $16, $84

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; ..######
; ..######
; ########
; ########
        
L5AB1:  DEFB $A4, $A4, $19, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;       
; ..######
; ..######
; ..######
; ...#####

L5AB5:  DEFB $A4, $A4, $A4, $BA

; ----------------------------------------------
; Player Rocket Pack Right #3 (Column 1) Graphic
; ----------------------------------------------
;
; #######.
; #######.
; #######.
; ########
; ####..##
; ###.##.#
; ###.##.#
; ###.##.#
; ###.##.#
; ####..##
; ##......
; #######.

L5AB9:  DEFB $89, $89, $89, $19, $B7, $8D, $8D, $8D, $8D, $B7, $24, $89

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; #######.
; ####....
; ####..##
; ########

L5AC5:  DEFB $89, $96, $B7, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; ##..###.
; ##..###.
; ####.##.
; ####.##.

L5AC9:  DEFB $29, $29, $83, $83

; ----------------------------------------------
; Player Rocket Pack Right #3 (Column 2) Graphic
; ----------------------------------------------
;
; ...#####
; ..######
; ...#####
; ########
; ..######
; ..######
; ..######
; ..######
; ..######
; ..######
; ..######
; ########

L5ACD:  DEFB $BA, $A4, $BA, $19, $A4, $A4, $A4, $A4, $A4, $A4, $A4, $19

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; ########
; ########
; ########
; ########

L5AD9:  DEFB $19, $19, $19, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; ########
; ########
; ########
; ..######

L5ADD:  DEFB $19, $19, $19, $A4
        
; ----------------------------------------------
; Player Rocket Pack Right #4 (Column 1) Graphic
; ----------------------------------------------
;
; ####....
; ####...#
; ####....
; #####..#
; #.##....
; ...#.#..
; ...#.#..
; ...#.#..
; ...#.#..
; #.##....
; ...#....
; #####..#

L5AE1:  DEFB $96, $81, $96, $8C, $A0, $BF, $BF, $BF, $BF, $A0, $B6, $8C

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; #####..#
; ##....##
; ##.#####
; ########

L5AED:  DEFB $8C, $0F, $22, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; ....#..#
; .#..#..#
; #.###..#
; ...##...

L5AF1:  DEFB $03, $AC, $A7, $32

; ----------------------------------------------
; Background Buffer for Rocket Pack Right Column
; ----------------------------------------------
; Holds the background graphics where the right column of the rocket pack graphic is currently displayed.

L5AF5:  DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00

; Base 1 - Rocket Pack Inactive
; -----------------------------

L5B01:  DEFB $00, $00, $00, $00

; Base 2 - Rocket Pack Active
; ---------------------------

L5B05:  DEFB $00, $00, $00, $00

; ---------------------------------------------
; Background Buffer for Rocket Pack Left Column
; ---------------------------------------------
; Holds the background graphics where the left column of the rocket pack graphic is currently displayed.

L5B09:  DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00

; Base 1 - Rocket Pack Inactive
; -----------------------------

L5B15:  DEFB $00, $00, $00, $00

; Base 2 - Rocket Pack Active
; ---------------------------

L5B19:  DEFB $00, $00, $00, $00

; ---------------------------------------------
; Player Rocket Pack Left #1 (Column 2) Graphic
; ---------------------------------------------
;
; ....####
; .#..####
; ....####
; #.######
; ####.#.#
; ....#.#.
; ....#.#.
; ....#.#.
; ....#.#.
; ....##.#
; ....#.#.
; #.######

L5B1D:  DEFB $16, $20, $16, $84, $31, $B1, $B1, $B1, $B1, $34, $B1, $84

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; #.######
; ##....##
; ######.#
; ########

L5B29:  DEFB $84, $0F, $2A, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; #.##.##.
; #.##.##.
; #.###..#
; ..######

L5B2D:  DEFB $2C, $2C, $A7, $A4

; ---------------------------------------------
; Player Rocket Pack Left #2 (Column 1) Graphic
; ---------------------------------------------
;
; #######.
; ########
; #######.
; ########
; ######.#
; ######.#
; ######.#
; ######.#
; ######.#
; ######.#
; ######.#
; #######.

L5B31:  DEFB $25, $19, $25, $19, $2A, $2A, $2A, $2A, $2A, $2A, $2A, $25

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; #######.
; ########
; ########
; ########

L5B3D:  DEFB $25, $19, $19, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; #######.
; #######.
; #######.
; ######.#

L5B41:  DEFB $1A, $1A, $1A, $2A

; ---------------------------------------------
; Player Rocket Pack Left #2 (Column 2) Graphic
; ---------------------------------------------
;
; ...#####
; ...#####
; ...#####
; ########
; ..######
; ..#...##
; ..#...##
; ..#...##
; ..#...##
; ..######
; ..#...##
; ########

L5B45:  DEFB $BA, $BA, $BA, $19, $A4, $1F, $1F, $1F, $1F, $A4, $1F, $19

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; ########
; ....####
; ###.####
; ########

L5B51:  DEFB $19, $16, $36, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; ###.#.##
; ###.#.##
; ####..##
; ########

L5B55:  DEFB $3F, $3F, $B7, $19

; ---------------------------------------------
; Player Rocket Pack Left #3 (Column 1) Graphic
; ---------------------------------------------
;
; ####....
; ####....
; ####....
; #####..#
; ####....
; ####..#.
; ####..#.
; ####..#.
; ####..#.
; ####....
; ####....
; #####..#

L5B59:  DEFB $96, $96, $96, $8C, $96, $9B, $9B, $9B, $9B, $96, $96, $8C

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; #####..#
; #######.
; ########
; ########

L5B65:  DEFB $8C, $1A, $19, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; #####..#
; #####..#
; #####..#
; ####..##

L5B69:  DEFB $8C, $8C, $8C, $B7

; ---------------------------------------------
; Player Rocket Pack Left #3 (Column 2) Graphic
; ---------------------------------------------
;
; ########
; ########
; ########
; ########
; ##.#####
; ....####
; ....####
; ....####
; ....####
; ##.#####
; ....####
; ########

L5B6D:  DEFB $19, $19, $19, $19, $22, $16, $16, $16, $16, $22, $16, $19

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; ########
; ..######
; #.######
; ########

L5B79:  DEFB $19, $A4, $84, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; ....####
; ....####
; #.######
; ##.#####

L5B7D:  DEFB $16, $16, $84, $22

; ---------------------------------------------
; Player Rocket Pack Left #4 (Column 1) Graphic
; ---------------------------------------------
;
; ##....##
; ##....##
; ##....##
; ###..###
; ##....##
; ##......
; ##......
; ##......
; ##......
; ##....##
; ##......
; ###.####

L5B81:  DEFB $0F, $0F, $0F, $B2, $0F, $24, $24, $24, $24, $0F, $24, $36

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; ###.####
; ####....
; #######.
; ########

L5B8D:  DEFB $36, $96, $1A, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; ###.####
; ###.####
; ###.####
; ###.####

L5B91:  DEFB $36, $36, $36, $36

; ---------------------------------------------
; Player Rocket Pack Left #4 (Column 2) Graphic
; ---------------------------------------------
;
; ########
; ########
; ########
; ########
; ########
; ..######
; ..######
; ..######
; ########
; ..######
; ########

L5B95:  DEFB $19, $19, $19, $19, $19, $A4, $A4, $A4, $A4, $19, $A4, $19

; Base 1 - Rocket Pack Inactive
; -----------------------------
;
; ########
; ########
; ########
; ########

L5BA1:  DEFB $19, $19, $19, $19

; Base 2 - Rocket Pack Active
; ---------------------------
;
; ...#####
; #.######
; ...#####
; #.######

L5BA5:  DEFB $BA, $84, $BA, $84

; --------------------------------
; Vulture Walking Right #1 Graphic
; --------------------------------
;
; ################
; ################
; #######.########
; ####....#.###..#
; #######.#.###..#
; ####......######
; ######.#...###.#
; ####......#.###.
; ##....##..#....#
; ..........#...##
; #....#.#....####
; ####.......#####
; #######.########
; #######.########
; #######.########
; #######...######

L5BA9:  DEFB $19, $19, $1A, $96, $1A, $96, $2A, $96, $0F, $0B, $B0, $96, $1A, $1A, $1A, $1A
        DEFB $19, $19, $19, $A7, $A7, $A4, $26, $0D, $8A, $1F, $16, $BA, $19, $19, $19, $A4

; --------------------------------
; Vulture Walking Right #2 Graphic
; --------------------------------
;
; ########################
; ########################
; ##############.#########
; ###########....####..###
; ###############.###..###
; ##########......###.####
; ########..####..###.####
; ######.##.....##....####
; #######.............####
; #######.####.......#####
; ##############.#..######
; ##############.#########
; ##############.#########
; ###############.########
; ########################
; ########################

L5BC9:  DEFB $19, $19, $19, $19, $19, $19, $19, $2A, $1A, $1A, $19, $19, $19, $19, $19, $19
        DEFB $19, $19, $2A, $1D, $1A, $1D, $1A, $24, $8F, $92, $0B, $96, $2A, $2A, $2A, $1A
        DEFB $19, $19, $19, $B2, $B2, $36, $36, $16, $16, $BA, $A4, $19, $19, $19, $19, $19

; --------------------------------
; Vulture Walking Right #3 Graphic
; --------------------------------
;
; ########################
; ########################
; ###########.############
; ########....####..######
; ###########..###...#####
; ########....#####.######
; ###########..#####.#####
; ####.##.........###.####
; ####....####.......#####
; ####..............######
; #####..#........########
; ##########....##########
; ###########.############
; ###########.############
; ###########.############
; ###########..###########

L5BF9:  DEFB $19, $19, $19, $19, $19, $19, $19, $83, $96, $96, $8C, $19, $19, $19, $19, $19
        DEFB $19, $19, $36, $16, $B2, $16, $B2, $0B, $96, $0B, $0B, $0F, $36, $36, $36, $B2
        DEFB $19, $19, $19, $A4, $BA, $84, $22, $36, $BA, $A4, $19, $19, $19, $19, $19, $19

; --------------------------------
; Vulture Walking Right #4 Graphic
; --------------------------------
;
; ########################
; ########################
; #########.##############
; ######.#...#####..######
; ##########.#####...#####
; ######.#...####.########
; ##########..###.########
; #####..#.......#########
; ##.#..##.......#########
; ###.....####...#########
; ###.........############
; ######.#...#############
; #########.##############
; #########.##############
; #########.##############
; ##########.#############

L5C29:  DEFB $19, $19, $19, $2A, $19, $2A, $19, $8C, $97, $3A, $3A, $2A, $19, $19, $19, $19
        DEFB $19, $19, $84, $BA, $22, $9D, $29, $09, $09, $81, $16, $BA, $84, $84, $84, $22
        DEFB $19, $19, $19, $A4, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19

; -------------------------------
; Vulture Flying Right #1 Graphic
; -------------------------------
;
; #####..########.
; #####..######..#
; #############..#
; #####..#####..##
; #####..#####..##
; .####.#.###..###
; ............####
; #....#.#....####
; ##.........#####
; ######.#...#####
; ###.......######
; ######.#..######
; ####......######
; #######.########
; ####....########
; #######.########

L5C59:  DEFB $8C, $8C, $19, $8C, $8C, $30, $0B, $B0, $24, $2A, $3A, $2A, $96, $1A, $96, $1A
        DEFB $1A, $8C, $8C, $B7, $B7, $B2, $16, $16, $BA, $BA, $A4, $A4, $A4, $19, $19, $19

; -------------------------------
; Vulture Flying Right #2 Graphic
; -------------------------------
;
; ###########..###########
; ###########..######..###
; ###################..###
; ###########..######.####
; ###########..######.####
; ######.###.#..####.#####
; #######............#####
; #######.....###...######
; ########......#.########
; ############.#.#########
; ########.......#########
; ############.#.#########
; #########.....##########
; ############..##########
; ##########....##########
; ############..##########

L5C79:  DEFB $19, $19, $19, $19, $19, $2A, $1A, $1A, $19, $19, $19, $19, $19, $19, $19, $19
        DEFB $B2, $B2, $19, $B2, $B2, $97, $0B, $01, $AA, $85, $09, $85, $92, $B7, $0F, $B7
        DEFB $19, $B2, $B2, $36, $36, $22, $BA, $A4, $19, $19, $19, $19, $19, $19, $19, $19

; -------------------------------
; Vulture Flying Right #3 Graphic
; -------------------------------
;
; ########..##############
; ########..##############
; ###################.####
; ########..#########..###
; ########..#########.####
; ###.####....####...#####
; ####..............######
; #####..#####....########
; ######.#.......#########
; ############..##########
; #######.#.....##########
; ###########..###########
; ########....############
; ###########.############
; ########....############
; ##########.#############

L5CA9:  DEFB $19, $19, $19, $19, $19, $36, $96, $8C, $2A, $19, $1A, $19, $19, $19, $19, $19
        DEFB $A4, $A4, $19, $A4, $A4, $16, $0B, $96, $09, $B7, $92, $B2, $16, $36, $16, $22
L5CC9:  DEFB $19, $19, $36, $B2, $36, $BA, $A4, $19, $19, $19, $19, $19, $19, $19, $19, $19

; -------------------------------
; Vulture Flying Right #4 Graphic
; -------------------------------
;
; ########..##############
; ########..######..######
; ################...#####
; ########..####..########
; #########.###..#########
; ##.####.....#..#########
; ##........#...##########
; ###.....##....##########
; ####........############
; ########.#..############
; #####..#....############
; ########.#..############
; ######.#...#############
; ########..##############
; ######.#..##############
; #######.################

L5CD9:  DEFB $19, $19, $19, $19, $19, $0A, $24, $3A, $96, $19, $8C, $19, $2A, $19, $2A, $1A
        DEFB $A4, $A4, $19, $8F, $A7, $03, $1F, $0F, $16, $20, $16, $20, $BA, $A4, $A4, $19
        DEFB $19, $A4, $BA, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19

; -------------------------------
; Vulture Walking Left #1 Graphic
; -------------------------------
;
; ################
; ################
; #######.########
; ...####...######
; ...###.#########
; ##.###....######
; #.##.##.########
; #....#.#...###.#
; #....#.#..#...##
; #...#..#..#...##
; ##..........####
; ####......######
; #######.########
; #######.########
; #######.########
; ######.#########

L5D09:  DEFB $19, $19, $1A, $9D, $26, $9F, $2C, $B0, $B0, $B3, $24, $96, $1A, $1A, $1A, $2A
        DEFB $19, $19, $19, $A4, $19, $A4, $19, $26, $1F, $1F, $16, $A4, $19, $19, $19, $19

; -------------------------------
; Vulture Walking Left #2 Graphic
; -------------------------------
;
; ########################
; ########################
; #############..#########
; #####..#####...#########
; #####..#####..##########
; #######.####....########
; #######.####..##########
; ######.####.....###.####
; #######....#.......#####
; #######....#.......#####
; ########..........######
; ###########.....########
; ########################
; ########################
; ########################
; ########################

L5D29:  DEFB $19, $19, $19, $8C, $8C, $1A, $1A, $2A, $1A, $1A, $19, $19, $19, $19, $19, $19
        DEFB $19, $19, $8C, $81, $B7, $96, $B7, $3A, $B6, $B6, $0B, $3A, $19, $19, $19, $19
        DEFB $19, $19, $19, $19, $19, $19, $19, $36, $BA, $BA, $A4, $19, $19, $19, $19, $19

; -------------------------------
; Vulture Walking Left #3 Graphic
; -------------------------------
;
; ########################
; ########################
; ###########.############
; #####..####..###########
; ####...###.#############
; ######.###....##########
; ####..####.#############
; ####.##...#...####.#####
; #####..#.#####....######
; #####..#.#........######
; ######.#........########
; ##########....##########
; ###########.############
; ###########.############
; ###########.############
; ##########.#############

L5D59:  DEFB $19, $19, $19, $8C, $81, $2A, $B7, $83, $8C, $8C, $2A, $19, $19, $19, $19, $19
        DEFB $19, $19, $36, $B2, $22, $0F, $22, $1F, $12, $04, $0B, $0F, $36, $36, $36, $22
        DEFB $19, $19, $19, $19, $19, $19, $19, $22, $A4, $A4, $19, $19, $19, $19, $19, $19

; -------------------------------
; Vulture Walking Left #4 Graphic
; -------------------------------
;
; ################
; ################
; #########.######
; ###..###...#####
; ###..###..######
; ########....####
; ###.####..######
; ##.###.#.....##.
; ###....#.......#
; ###....#.......#
; ####......#...##
; #######.....####
; #########.######
; #########.######
; #########.######
; ########..######

L5D89:  DEFB $19, $19, $19, $B2, $B2, $19, $36, $82, $1D, $1D, $96, $1A, $19, $19, $19, $19
        DEFB $19, $19, $84, $BA, $A4, $16, $A4, $0C, $09, $09, $1F, $16, $84, $84, $84, $A4

; ------------------------------
; Vulture Flying Left #1 Graphic
; ------------------------------
;
; ################..######
; #########.######..######
; ########...#############
; ##########.#####..######
; ##########.#####..######
; ###########.##.#...####.
; ###########............#
; ############...#.......#
; #############..#..#...##
; ##############.#...#####
; ##############.#....####
; ###############...######
; ###############....#####
; ################..######
; ################...#####
; #################.######

L5DA9:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19
        DEFB $19, $84, $BA, $22, $22, $AD, $3A, $81, $8C, $2A, $2A, $89, $89, $19, $19, $19
        DEFB $A4, $A4, $19, $A4, $A4, $9D, $09, $09, $1F, $BA, $16, $A4, $BA, $A4, $BA, $84

; ------------------------------
; Vulture Flying Left #2 Graphic
; ------------------------------
;
; #############..#########
; #####..######..#########
; ####...#################
; #######.#####..#########
; ########.####.#.########
; #########.##.##.#####..#
; ##########........#...##
; ##########........#...##
; ###########...#.....####
; ############..#.########
; ############..#...######
; #############..#########
; #############..#..######
; ##############.#########
; ##############.#..######
; ###############.########

L5DD9:  DEFB $19, $8C, $81, $1A, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19
L5DE9:  DEFB $8C, $8C, $19, $8C, $30, $2C, $24, $24, $A6, $B8, $B8, $8C, $8C, $2A, $2A, $1A
L5DF9:  DEFB $19, $19, $19, $19, $19, $8C, $1F, $1F, $16, $19, $A4, $19, $A4, $19, $A4, $19

; ------------------------------
; Vulture Flying Left #3 Graphic
; ------------------------------
;
; ############..##########
; #####..#####..##########
; ####...#################
; ######.#####..##########
; ######.#####..#####.####
; #######.##.#...#...#####
; #######............#####
; ########........#.######
; ##########......########
; ##########..###.########
; ##########......########
; ###########..###########
; ###########.....########
; ###########..###########
; ############...#########
; ##############.#########

L5E09:  DEFB $19, $8C, $81, $2A, $2A, $1A, $1A, $19, $19, $19, $19, $19, $19, $19, $19, $19
        DEFB $B7, $B7, $19, $B7, $B7, $8D, $0B, $0B, $24, $29, $24, $B2, $3A, $B2, $81, $2A
        DEFB $19, $19, $19, $19, $19, $36, $BA, $BA, $84, $19, $19, $19, $19, $19, $19, $19

; ------------------------------
; Vulture Flying Left #4 Graphic
; ------------------------------
;
; ###########..###########
; ###..######..###########
; ####..##################
; ###.#######..###########
; ###..######..###########
; ####..##.#.##..###.#####
; ####..............######
; ######.#..........######
; #######...#.....########
; ########..#.###.########
; ########..#...##########
; #########.##############
; ##########....##########
; ##########.#############
; ##########....##########
; ###########.############

L5E39:  DEFB $19, $B2, $B7, $36, $B2, $B7, $96, $2A, $1A, $19, $19, $19, $19, $19, $19, $19
        DEFB $B2, $B2, $19, $B2, $B2, $AB, $0B, $0B, $A2, $0D, $1F, $84, $0F, $22, $0F, $36
        DEFB $19, $19, $19, $19, $19, $22, $A4, $A4, $19, $19, $19, $19, $19, $19, $19, $19

; --------
; Not Used
; --------

; ########
; ########
; ########
; ########
; ########
; ###.####
; ...#####
; ...#####
; #.######
; ########
; ########
; ########
; ########
; ########
; ########
; ########

L5E69:  DEFB $19, $19, $19, $19, $19, $36, $BA, $BA, $84, $19, $19, $19, $19, $19, $19, $19     ; [Redundant]

; -----------------------------
; Background Buffer for Vulture
; -----------------------------
; Holds the background graphics where the flying vulture graphic is currently displayed.

; ###########..###########
; ###..######..###########
; ####..##################
; ###.#######..###########
; ###..######..###########
; ####..##.#.##..###.#####
; ####..............######
; ######.#..........######
; #######...#.....########
; ########..#.###.########
; ########..#...##########
; #########.##############
; ##########....##########
; ##########.#############
; ##########....##########
; ###########.############

L5E79:  DEFB $19, $B2, $B7, $36, $B2, $B7, $96, $2A, $1A, $19, $19, $19, $19, $19, $19, $19
L5E89:  DEFB $B2, $B2, $19, $B2, $B2, $AB, $0B, $0B, $A2, $0D, $1F, $84, $0F, $22, $0F, $36
L5E99:  DEFB $19, $19, $19, $19, $19, $22, $A4, $A4, $19, $19, $19, $19, $19, $19, $19, $19

; ------------------------
; Bubloid Set 1 Graphic #1
; ------------------------
;
; ##....##
; #...#..#
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; #...#..#
; ##....##

L5EA9:  DEFB $0F, $B3, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $B3, $0F

; ------------------------
; Bubloid Set 1 Graphic #2
; ------------------------
;
; ########....####
; #######.###..###
; ######.#..#...##
; #####..#.......#
; #####..#.......#
; #####..#.......#
; #####..#.......#
; #####..#.......#
; #####..#.......#
; #####..#.......#
; #####..#.......#
; #####..#.......#
; #####..#.......#
; ######.#..#...##
; #######.###..###
; ########....####

L5EB9:  DEFB $19, $1A, $2A, $8C, $8C, $8C, $8C, $8C, $8C, $8C, $8C, $8C, $8C, $2A, $1A, $19
        DEFB $16, $B2, $1F, $09, $09, $09, $09, $09, $09, $09, $09, $09, $09, $1F, $B2, $16

; ------------------------
; Bubloid Set 1 Graphic #3
; ------------------------
;
; ######.#..######
; #####..#...#####
; ####........####
; ###.....###..###
; ##........#...##
; ##........#...##
; ##........#...##
; ##........#...##
; ##........#...##
; ##........#...##
; ##........#...##
; ##........#...##
; ###.....###..###
; ####........####
; #####..#...#####
; ######.#..######

L5ED9:  DEFB $2A, $8C, $96, $3A, $24, $24, $24, $24, $24, $24, $24, $24, $3A, $96, $8C, $2A
        DEFB $A4, $BA, $16, $B2, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $B2, $16, $BA, $A4

; ------------------------
; Bubloid Set 1 Graphic #4
; ------------------------
;
; ####....########
; ##........######
; ............####
; ............####
; ............####
; ............####
; ............####
; ............####
; ............####
; ............####
; ............####
; ............####
; ............####
; ............####
; ##........######
; ####....########

L5EF9:  DEFB $96, $24, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $24, $96
        DEFB $19, $A4, $16, $16, $16, $16, $16, $16, $16, $16, $16, $16, $16, $16, $A4, $19

; ------------------------
; Bubloid Set 1 Graphic #5
; ------------------------
;
; ###..###
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ###..###

L5F19:  DEFB $B2, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $B2

; ------------------------
; Bubloid Set 2 Graphic #1
; ------------------------
;
; ##....##
; #...#..#
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; #...#..#
; ##....##

L5F29:  DEFB $0F, $B3, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $B3, $0F

; ------------------------
; Bubloid Set 2 Graphic #2
; ------------------------
;
; ########....####
; ######.#..#...##
; #####..#.......#
; ####............
; ####............
; #####..#.......#
; ######.#..#...##
; ########....####
; ########....####
; ######.#..#...##
; #####..#.......#
; ####............
; ####............
; #####..#.......#
; ######.#..#...##
; ########....####

L5F39:  DEFB $19, $2A, $8C, $96, $96, $8C, $2A, $19, $19, $2A, $8C, $96, $96, $8C, $2A, $19
        DEFB $16, $1F, $09, $0B, $0B, $09, $1F, $16, $16, $1F, $09, $0B, $0B, $09, $1F, $16

; ------------------------
; Bubloid Set 2 Graphic #3
; ------------------------
;
; ######.#..######
; ####........####
; ##........#...##
; ##........#...##
; ##........#...##
; ####........####
; ######.#..######
; ################
; ################
; ######.#..######
; ####........####
; ##........#...##
; ##........#...##
; ##........#...##
; ####........####
; ######.#..######

L5F59:  DEFB $2A, $96, $24, $24, $24, $96, $2A, $19, $19, $2A, $96, $24, $24, $24, $96, $2A
        DEFB $A4, $16, $1F, $1F, $1F, $16, $A4, $19, $19, $A4, $16, $1F, $1F, $1F, $16, $A4

; ------------------------
; Bubloid Set 2 Graphic #4
; ------------------------
;
; ################
; ################
; ################
; ################
; ################
; ################
; ################
; ####....########
; ##........######
; ##........######
; ............####
; ............####
; ............####
; ##........######
; ##........######
; ####....########

L5F79:  DEFB $19, $19, $19, $19, $19, $19, $19, $96, $24, $24, $0B, $0B, $0B, $24, $24, $96
        DEFB $19, $19, $19, $19, $19, $19, $19, $19, $A4, $A4, $16, $16, $16, $A4, $A4, $19

; ------------------------
; Bubloid Set 2 Graphic #5
; ------------------------
;
; ########
; ########
; ########
; ########
; ##....##
; #...#..#
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; #...#..#
; ##....##

L5F99:  DEFB $19, $19, $19, $19, $0F, $B3, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $B3, $0F

; ------------------------
; Bubloid Set 3 Graphic #1
; ------------------------
;
; ##....##
; #...#..#
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; ........
; #...#..#
; ##....##

L5FA9:  DEFB $0F, $B3, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $0B, $B3, $0F

; ------------------------
; Bubloid Set 3 Graphic #2
; ------------------------
;
; ################
; ########....####
; #######.###..###
; ######.#..#...##
; #####..#.......#
; #####..#.......#
; #####..#.......#
; #####..#.......#
; #####..#.......#
; #####..#.......#
; ######.#..#...##
; #######.###..###
; ########....####
; ################
; ################

L5FB9:  DEFB $19, $19, $1A, $2A, $8C, $8C, $8C, $8C, $8C, $8C, $8C, $2A, $1A, $19, $19, $19
        DEFB $19, $16, $B2, $1F, $09, $09, $09, $09, $09, $09, $09, $1F, $B2, $16, $19, $19

; ------------------------
; Bubloid Set 3 Graphic #3
; ------------------------
;
; ################
; ################
; ################
; ################
; ######.#..######
; #####..#...#####
; ####........####
; ###.....###..###
; ##........#...##
; ##........#...##
; ###.....###..###
; ####........####
; #####..#...#####
; ######.#..######
; ################
; ################

L5FD9:  DEFB $19, $19, $19, $19, $2A, $8C, $96, $3A, $24, $24, $3A, $96, $8C, $2A, $19, $19
        DEFB $19, $19, $19, $19, $A4, $BA, $16, $B2, $1F, $1F, $B2, $16, $BA, $A4, $19, $19

; ------------------------
; Bubloid Set 3 Graphic #4
; ------------------------
;
; ################
; ################
; ################
; ################
; ################
; ####....########
; ##........######
; ............####
; ............####
; ............####
; ##........######
; ####....########
; ################
; ################
; ################
; ################

L5FF9:  DEFB $19, $19, $19, $19, $19, $96, $24, $0B, $0B, $0B, $24, $96, $19, $19, $19, $19
        DEFB $19, $19, $19, $19, $19, $19, $A4, $16, $16, $16, $A4, $19, $19, $19, $19, $19

; ------------------------
; Bubloid Set 3 Graphic #5
; ------------------------
;
; ###..###
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ##....##
; ###..###

L6019:  DEFB $B2, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $B2

; ---------------------------------
; Background Buffer for the Bubloid
; ---------------------------------

L6029:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19
L6039:  DEFB $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19, $19

; --------
; Not Used
; --------
;
; ################
; ################
; ################
; ################
; ################
; ####....########
; ##........######
; ............####
; ............####
; ............####
; ##........######
; ####....########
; ################
; ################
; ################
; ################

L6049:  DEFB $19, $19, $19, $19, $19, $96, $24, $0B, $0B, $0B, $24, $96, $19, $19, $19, $19     ; [Redundant]
        DEFB $19, $19, $19, $19, $19, $19, $A4, $16, $16, $16, $A4, $19, $19, $19, $19, $19

; ###..###--------
; ##....##--------
; ##....##--------
; ##....##--------
; ##....##--------
; ##....##--------
; ##....##--------
; ##....##--------
; ##....##--------
; ##....##--------
; ##....##--------
; ##....##--------
; ##....##--------
; ##....##--------
; ##....##--------
; ###..###--------

L6069:  DEFB $B2, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $B2
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00

; -------------------------------------------
; Player Caught Left Column Background Buffer
; -------------------------------------------

L6089:  DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00

; ------------------------
; Player Caught #1 Graphic
; ------------------------
; The background buffer above forms the first column of the graphic, allowing the player caught
; graphic to always be drawn as 2 columns wide.
;
; --------##....##
; --------#...#..#
; --------...##...
; --------...##...
; --------........
; --------..####..
; --------..####..
; --------..####..
; --------..####..
; --------...##...
; --------...##...
; --------...##...
; --------........
; --------...##...
; --------#...#..#
; --------##....##

L6099:  DEFB $0F, $B3, $32, $32, $0B, $8F, $8F, $8F, $8F, $32, $32, $32, $0B, $32, $B3, $0F

; ------------------------
; Player Caught #2 Graphic
; ------------------------
;
; ########....####
; #######........#
; #######.##....##
; ####....##....##
; ####...........#
; ###.#########..#
; ####....##....##
; ####....##....##
; ####....##....##
; ####....##....##
; ####...#..#...##
; ####...#..#...##
; #####..#..#...##
; ######.#..#...##
; #######.....####
; ########...#####

L60A9:  DEFB $19, $1A, $1A, $96, $96, $36, $96, $96, $96, $96, $81, $81, $8C, $2A, $1A, $19
        DEFB $16, $09, $0F, $0F, $09, $8C, $0F, $0F, $0F, $0F, $1F, $1F, $1F, $1F, $16, $BA

; ------------------------
; Player Caught #3 Graphic
; ------------------------
;
; #######...######
; ####.......#####
; ####........####
; ###....#.......#
; ###....#.......#
; ###....#....##.#
; ###....#....#..#
; ###.##.#####...#
; ###.##.#####...#
; ###....#....#..#
; ###....#....##.#
; ###....#.......#
; ###....#.......#
; ####........####
; ####.......#####
; #######...######

L60C9:  DEFB $1A, $96, $96, $1D, $1D, $1D, $1D, $AD, $AD, $1D, $1D, $1D, $1D, $96, $96, $1A
        DEFB $A4, $BA, $16, $09, $09, $34, $03, $81, $81, $03, $34, $09, $09, $16, $BA, $A4

; ------------------------
; Player Caught #4 Graphic
; ------------------------
;
; ####....########
; ###.......######
; ##........######
; ##.........#####
; ##..........####
; ##.########.####
; ##....##....####
; ##....##....####
; ##....##....####
; ##....##....####
; ##.########.####
; ##..........####
; ##....##...#####
; ##....##..######
; ###.......######
; ####....########

L60E9:  DEFB $96, $3A, $24, $24, $24, $22, $0F, $0F, $0F, $0F, $22, $24, $0F, $0F, $3A, $96
        DEFB $19, $A4, $A4, $BA, $16, $36, $16, $16, $16, $16, $36, $16, $BA, $A4, $A4, $19

; ------------------------
; Player Caught #5 Graphic
; ------------------------
; The background buffer below forms the second column of the graphic, allowing the player caught
; graphic to always be drawn as 2 columns wide.
;
; ##....##--------
; #...#..#--------
; ........--------
; .#......--------
; .#......--------
; .#......--------
; .#......--------
; .###.##.--------
; .###.##.--------
; .#......--------
; .#......--------
; .#......--------
; .#......--------
; ........--------
; ##....##--------

L6109:  DEFB $0F, $B3, $0B, $04, $04, $04, $04, $33, $33, $04, $04, $04, $04, $0B, $B3, $0F

; --------------------------------------------
; Player Caught Right Column Background Buffer
; --------------------------------------------

L6119:  DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00

; ---------------
; Diamond Graphic
; ---------------
;
; ##....##
; #....#.#
; ...##...
; #.#.#..#
; ##....##
; ########

L6129:  DEFB $0F, $B0, $32, $AF, $0F, $19

; ----------------------------
; Pseudo Hi-Res Display Driver
; ----------------------------

L612F:  DI                                              ; Disable interrupts.    
        LD      B,$16                                   ; Delay counter.

L6132:  DJNZ    L6132                                   ; Delay until time to output the next scan line.

        LD      B,$C0                                   ; The number of display lines.
        LD      C,$FE                                   ; C holds the port, a read from which will begin a HSync pulse.
        LD      HL,$C061                                ; The address of the hi-res display file with bit 15 set to point to the shadow copy in high memory.
        LD      DE,$0021                                ; There are 32 characters + a terminator per line.

L613E:  IN      A,(C)                                   ; Start a VSync pulse. This resets the scan line counter.
        OUT     ($FF),A                                 ; Stop the VSync pulse.

        ADD     HL,DE                                   ; Point to the next display line.
        CALL    L5948                                   ; Display the line by 'executing' it.

        DEC     B                                       ; Decrement the line counter.
        JP      NZ,L613E                                ; Jump back if there are more lines to output.

        CALL    $0292                                   ; Generate the bottom border lines.
        CALL    $0220                                   ; Generate the VSync pulse and the top border lines.

        LD      IX,L612F                                ; Reset the video handler routine pointer.
        JP      $02A4                                   ; Jump to the ROM to restore the main registers and return to the game.

; ------------------------
; General Platform Graphic
; ------------------------
; The top for a normal platform appears the same as that for a diamond platform yet they use different character codes.
; This allows the program to easily differentiate between them.
;
; ........
; ........
; .#.#.##.
; #.#.#..#

L6157:  DEFB $0B, $0B, $2F, $AF

; ------------------------
; Diamond Platform Graphic
; ------------------------
;
; ........
; ........
; .#.#.##.
; #.#.#..#

L615B:  DEFB $1C, $1C, $2F, $AF

; ----------------------
; Ladder Graphic Segment
; ----------------------
;
; ..####..
; ..####..
; ........
; ..####..

L615F:  DEFB $8F, $8F, $0B, $8F

; ---------------
; Number Graphics
; ---------------

; ##....##
; #.###..#
; #.###..#
; #.###..#
; #.###..#
; #.###..#
; ##....##

L6163:  DEFB $0F, $A7, $A7, $A7, $A7, $A7, $A7, $0F

; ###..###
; ###..###
; ###..###
; ###..###
; ###..###
; ###..###
; ###..###
; ##....##

L616B:  DEFB $B2, $B2, $B2, $B2, $B2, $B2, $B2, $0F

; #.....##
; #####..#
; #####..#
; #####..#
; #.....##
; #.######
; #.######
; .......#

L6173:  DEFB $92, $8C, $8C, $8C, $92, $84, $84, $09

; #.....##
; #####..#
; #####..#
; #####..#
; ##....##
; #####..#
; #####..#
; #.....##

L617B:  DEFB $92, $8C, $8C, $8C, $0F, $8C, $8C, $92

; ####..##
; ##.#..##
; #.##..##
; ####..##
; ........
; ####..##
; ####..##
; ####..##

L6183:  DEFB $B7, $97, $88, $B7, $0B, $B7, $B7, $B7

; .......#
; #.######
; #.######
; #.....##
; #####..#
; #####..#
; #####..#
; #.....##

L618B:  DEFB $09, $84, $84, $92, $8C, $8C, $8C, $92

; ##....##
; #.######
; #.######
; #.....##
; #.###.#.
; #.###.#.
; #.###.#.
; ##....##

L6193:  DEFB $0F, $84, $84, $92, $87, $87, $87, $0F

; ##......
; #####..#
; #####..#
; ####..##
; ####..##
; ####..##
; ###..###
; ###..###

L619B:  DEFB $24, $8C, $8C, $B7, $B7, $B7, $B2, $B2

; ##....##
; #.###..#
; #.###..#
; ##....##
; #.###..#
; #.###..#
; #.###..#
; ##....##

L61A3:  DEFB $0F, $A7, $A7, $0F, $A7, $A7, $A7, $0F

; ##....##
; #.###..#
; #.###..#
; #.###..#
; ##....##
; #####..#
; #####..#
; ##....##

L61AB:  DEFB $0F, $A7, $A7, $A7, $0F, $8C, $8C, $0F

; -------
; "SCORE"
; -------
;
; <--+0--><--+1--><--+2--><--+3-->
;
; ####....########################
; ###.######..###..####.#.##....##
; ###.#####.###..##.##.##.#.######
; ####...##.###..##.##.##.#.######
; #######.#.###..##.##...###....##
; #######.#.###..##.##.##.#.######
; #######.#.###..##.##.##.#.######
; ###....###..###..###.##.##....##

L61B3:  DEFB $96, $36, $36, $8E, $89, $89, $89, $1D
        DEFB $19, $29, $A7, $A7, $A7, $A7, $A7, $29
        DEFB $19, $30, $2C, $2C, $9E, $2C, $2C, $33
        DEFB $19, $0F, $84, $84, $0F, $84, $84, $0F

; ----
; "HI"
; ----
;
; <--+0--><--+1-->
;
; ################
; ###.##.#...#####
; ###.##.##.######
; ###.##.##.######
; ###....##.######
; ###.##.##.######
; ###.##.##.######
; ###.##.#...#####

L61D3:
        DEFB $19, $AD, $AD, $AD, $1D, $AD, $AD, $AD
        DEFB $19, $BA, $84, $84, $84, $84, $84, $BA

; -------
; "BONUS"
; -------
;
; <--+0--><--+1--><--+2--><--+3--><--+4-->
;
; ########################################
; ###....###..###..###.##.##.###....######
; ###.##.##.##.##..###.##.##.###.#########
; ###.##.##.##.##...##.##.##.###.#########
; ###...#.#.##.##..#.#.##.##.###....######
; ###.##.##.##.##..#...##.##.######.######
; ###.##.##.##.##..###.##.##.######.######
; ###....###..###..###.##....##.....######

L61E3:  DEFB $19, $1D, $AD, $AD, $A6, $AD, $AD, $1D 
        DEFB $19, $29, $2C, $2C, $2C, $2C, $2C, $29 
        DEFB $19, $33, $33, $28, $2F, $27, $33, $33 
        DEFB $19, $9F, $82, $82, $9F, $22, $22, $32 
        DEFB $19, $A4, $19, $19, $A4, $84, $84, $A4

; --------------
; Life Indicator
; --------------
;
; ...##...
; ...##...
; ...##...
; ..####..
; ..####..
; ..####..
; ..####..
; ...##...
; ...##...
; ...##...
; ..####..

L620B:  DEFB $32, $32, $32, $8F, $8F, $8F, $8F, $32, $32, $32, $8F

; ------------
; Wave Graphic
; ------------
;
; #######.########
; #####..#..######
; #####..#...#####
; ##......##....##

L6216:  DEFB $89, $8C, $8C, $24                         ; Column 1
L621A:  DEFB $19, $A4, $BA, $0F                         ; Column 2

; -------------------
; Rocket Pack Graphic
; -------------------
;
; ###..###
; ##....##
; #....#.#
; #....#.#
; #....#.#
; #....#.#
; #....#.#
; #....#.#
; #....#.#
; #....#.#
; #....#.#
; #....#.#
; ###..###
; ##....##
; #....#.#
; ########

L621E:  DEFB $B2, $0F, $B0, $B0, $B0, $B0, $B0, $B0, $B0, $B0, $B0, $B0, $B2, $0F, $B0, $19

; -----------------
; Splash #1 Graphic
; -----------------
;
; ################
; ################
; ################
; ################
; #######.########
; #######.########
; #######..#####..
; ##.###....######
; .#####.....#####
; ####.#.#...#####
; #####..#....####
; ####.........##.
; ###..........##.
; ##.......#.#..##
; ##.............#
; ................

L622E:  DEFB $19, $19, $19, $19, $25, $25, $25, $9F, $12, $31, $8C, $96, $3A, $24, $24, $0B
        DEFB $19, $19, $19, $19, $19, $19, $12, $A4, $BA, $BA, $16, $0C, $0C, $23, $09, $0B

; -----------------
; Splash #2 Graphic
; -----------------
;
; ###..######..###
; ##....####....##
; ................
; ................
; ................
; ...#####...##...
; #...#..#...###.#
; ##.###....####..
; #######...######
; #######...######
; ##.###.....#####
; .#####.....#####
; #####..#....####
; ####.........##.
; ##...........##.
; ................

L624E:  DEFB $B2, $0F, $0B, $0B, $0B, $BA, $B3, $9F, $89, $89, $9F, $12, $8C, $96, $24, $0B
        DEFB $B2, $0F, $0B, $0B, $0B, $32, $26, $8F, $A4, $A4, $BA, $BA, $16, $0C, $0C, $0B

; -----------------
; Splash #3 Graphic
; -----------------
;
; ################
; ################
; ################
; ################
; ################
; ###.############
; ####.##.###.####
; ###..#######..##
; ###..#######...#
; ###..########..#
; ################
; #######.########
; #######..#####..
; ##.###....######
; #####..#...#####
; ####........####

L626E:  DEFB $19, $19, $19, $19, $19, $36, $83, $B2, $B2, $B2, $19, $89, $89, $9F, $8C, $96
        DEFB $19, $19, $19, $19, $19, $19, $36, $B7, $91, $8C, $19, $19, $12, $A4, $BA, $16

; ----------------
; Fuel Can Graphic
; ----------------
;
; ##......
; #.....##
; ..#...#.
; ..#...#.
; ..#...#.
; ..#.....
; .......#
; ########

L628E:  DEFB $24, $92, $02, $02, $02, $A2, $09, $19

; -------------------
; Leg of Lamb Graphic
; -------------------
;
; #.######
; .#..###.
; .#......
; .#...##.
; #.######
; ########

L6296:  DEFB $84, $1E, $04, $27, $84, $19

; =======================
; Screen Definition Table
; =======================
; Each screen level is defined using the following format:
; - 1 byte: Number of platforms.
; - For each platform:
;   - 2 bytes: Display file address of the left end of the platform.
;   - 1 byte : Width of the platform.
; - 1 byte: Number of ladders.
; - For each ladder:
;   - 2 bytes: Display file address of the top of the ladder.
;   - 1 byte : Number of segments forming the ladder.

; -------------------------
; Screen Level 1 Definition
; -------------------------

L629C:  DEFB $0D                                        ; There are 13 platforms.
        DEFW L462E+$0011                                ; Line 44 column 17.
        DEFB $06                                        ; 6 columns wide.
        DEFW L462E+$0018                                ; Line 44 column 24.
        DEFB $03                                        ; 3 columns wide.
        DEFW L462E+$001C                                ; Line 44 column 28.
        DEFB $03                                        ; 3 columns wide.
        DEFW L49CA+$0012                                ; Line 72 column 18.
        DEFB $05                                        ; 5 columns wide.
        DEFW L49CA+$0018                                ; Line 72 column 24.
        DEFB $02                                        ; 2 columns wide.
        DEFW L49CA+$001B                                ; Line 72 column 27.
        DEFB $02                                        ; 2 columns wide.
        DEFW L4D66+$0011                                ; Line 100 column 17.
        DEFB $09                                        ; 9 columns wide.
        DEFW L4F97+$001A                                ; Line 117 column 26.
        DEFB $03                                        ; 3 columns wide.
        DEFW L5102+$0010                                ; Line 128 column 16.
        DEFB $05                                        ; 5 columns wide.
        DEFW L5102+$0016                                ; Line 128 column 22.
        DEFB $04                                        ; 4 columns wide.
        DEFW L549E+$0011                                ; Line 156 column 17.
        DEFB $05                                        ; 5 columns wide.
        DEFW L549E+$0017                                ; Line 156 column 23.
        DEFB $02                                        ; 2 columns wide.
        DEFW L549E+$001A                                ; Line 156 column 26.
        DEFB $04                                        ; 4 columns wide.

        DEFB $05                                        ; There are 5 ladders.
        DEFW L4589+$0019                                ; Line 39 column 25.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4925+$0014                                ; Line 67 column 20.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4CC1+$0012                                ; Line 95 column 18.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L505D+$0017                                ; Line 123 column 23.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4925+$001D                                ; Line 67 column 29.
        DEFB $18                                        ; 24 segments tall.

; -------------------------
; Screen Level 2 Definition
; -------------------------

        DEFB 0C                                         ; There are 12 platforms.
        DEFW L462E+$0010                                ; Line 44 column 16.
        DEFB $03                                        ; 3 columns wide.
        DEFW L462E+$0018                                ; Line 44 column 24.
        DEFB $02                                        ; 2 columns wide.
        DEFW L49CA+$0010                                ; Line 72 column 16.
        DEFB $05                                        ; 5 columns wide.
        DEFW L49CA+$0016                                ; Line 72 column 22.
        DEFB $02                                        ; 2 columns wide.
        DEFW L49CA+$001B                                ; Line 72 column 27.
        DEFB $03                                        ; 3 columns wide.
        DEFW L4D66+$0012                                ; Line 100 column 18.
        DEFB $04                                        ; 4 columns wide.
        DEFW L4D66+$0017                                ; Line 100 column 23.
        DEFB $03                                        ; 3 columns wide.
        DEFW L5102+$0012                                ; Line 128 column 18.
        DEFB $03                                        ; 3 columns wide.
        DEFW L5102+$0018                                ; Line 128 column 24.
        DEFB $03                                        ; 3 columns wide.
        DEFW L549E+$0010                                ; Line 156 column 16.
        DEFB $05                                        ; 5 columns wide.
        DEFW L549E+$0017                                ; Line 156 column 23.
        DEFB $04                                        ; 4 columns wide.
        DEFW L549E+$001C                                ; Line 156 column 28.
        DEFB $02                                        ; 2 columns wide.

        DEFB $08                                        ; There are 8 ladders.
        DEFW L4589+$0013                                ; Line 39 column 19.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4589+$0017                                ; Line 39 column 23.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4589+$001A                                ; Line 39 column 26.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4925+$001D                                ; Line 67 column 29.
        DEFB $18                                        ; 24 segments tall.
        DEFW L4CC1+$0011                                ; Line 95 column 11.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4CC1+$001A                                ; Line 95 column 26.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L505D+$0014                                ; Line 123 column 20.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L505D+$0017                                ; Line 123 column 23.
        DEFB $0A                                        ; 10 segments tall.

; -------------------------
; Screen Level 3 Definition
; -------------------------

        DEFB $0D                                        ; There are 13 platforms.
        DEFW L462E+$0011                                ; Line 44 column 17.
        DEFB $02                                        ; 2 columns wide.
        DEFW L462E+$0014                                ; Line 44 column 20.
        DEFB $03                                        ; 3 columns wide.
        DEFW L462E+$001A                                ; Line 44 column 26.
        DEFB $02                                        ; 2 columns wide.
        DEFW L462E+$001D                                ; Line 44 column 29.
        DEFB $01                                        ; 1 column wide.
        DEFW L49CA+$0013                                ; Line 72 column 19.
        DEFB $04                                        ; 4 columns wide.
        DEFW L49CA+$001A                                ; Line 72 column 26.
        DEFB $04                                        ; 4 columns wide.
        DEFW L4D66+$0010                                ; Line 100 column 16.
        DEFB $06                                        ; 6 columns wide.
        DEFW L4D66+$001B                                ; Line 100 column 27.
        DEFB $03                                        ; 3 columns wide.
        DEFW L5102+$0011                                ; Line 128 column 17.
        DEFB $03                                        ; 3 columns wide.
        DEFW L5102+$0015                                ; Line 128 column 21.
        DEFB $02                                        ; 2 columns wide.
        DEFW L5102+$001B                                ; Line 128 column 27.
        DEFB $02                                        ; 2 columns wide.
        DEFW L549E+$0011                                ; Line 156 column 17.
        DEFB $06                                        ; 6 columns wide.
        DEFW L549E+$001A                                ; Line 128 column 26.
        DEFB $04                                        ; 4 columns wide.

        DEFB $05                                        ; There are 5 ladders.
        DEFW L4589+$0018                                ; Line 39 column 24.
        DEFB $20                                        ; 32 segments tall.
        DEFW L4925+$0015                                ; Line 67 column 21.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4925+$001C                                ; Line 67 column 28.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4CC1+$0011                                ; Line 95 column 17.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L505D+$001D                                ; Line 123 column 29.
        DEFB $0A                                        ; 10 segments tall.

; -------------------------
; Screen Level 4 Definition
; -------------------------

        DEFB $0B                                        ; There are 11 platforms.
        DEFW L462E+$0011                                ; Line 44 column 17.
        DEFB $06                                        ; 6 columns wide.
        DEFW L462E+$001C                                ; Line 44 column 28.
        DEFB $02                                        ; 2 columns wide.
        DEFW L49CA+$0010                                ; Line 72 column 16.
        DEFB $03                                        ; 3 columns wide.
        DEFW L49CA+$0018                                ; Line 44 column 24.
        DEFB $03                                        ; 3 columns wide.
        DEFW L4D66+$0014                                ; Line 100 column 20.
        DEFB $03                                        ; 3 columns wide.
        DEFW L4D66+$001C                                ; Line 100 column 28.
        DEFB $02                                        ; 2 columns wide.
        DEFW L5102+$0010                                ; Line 128 column 16.
        DEFB $03                                        ; 3 columns wide.
        DEFW L5102+$0018                                ; Line 128 column 24.
        DEFB $03                                        ; 3 columns wide.
        DEFW L549E+$0010                                ; Line 156 column 16.
        DEFB $03                                        ; 3 columns wide.
        DEFW L549E+$0014                                ; Line 156 column 20.
        DEFB $03                                        ; 3 columns wide.
        DEFW L549E+$0018                                ; Line 156 column 24.
        DEFB $03                                        ; 3 columns wide.

        DEFB $07                                        ; There are 7 ladders.
        DEFW L4589+$0019                                ; Line 39 column 25.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4925+$0015                                ; Line 67 column 21.
        DEFB $04                                        ; 4 segments tall.
        DEFW L4925+$001D                                ; Line 67 column 29.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4CC1+$0011                                ; Line 95 column 17.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4CC1+$0019                                ; Line 95 column 15.
        DEFB $04                                        ; 4 segments tall.
        DEFW L501B+$0015                                ; Line 121 column 21.
        DEFB $04                                        ; 4 segments tall.
        DEFW L501B+$001D                                ; Line 121 column 29.
        DEFB $0C                                        ; 12 segments tall.

; -------------------------
; Screen Level 5 Definition
; -------------------------

        DEFB $0E                                        ; There are 14 platforms.
        DEFW L462E+$0010                                ; Line 44 column 16.
        DEFB $02                                        ; 2 columns wide.
        DEFW L462E+$0015                                ; Line 44 column 21.
        DEFB $04                                        ; 4 columns wide.
        DEFW L462E+$001A                                ; Line 44 column 26.
        DEFB $04                                        ; 4 columns wide.
        DEFW L49CA+$0010                                ; Line 72 column 16.
        DEFB $02                                        ; 2 columns wide.
        DEFW L49CA+$0017                                ; Line 72 column 23.
        DEFB $02                                        ; 2 columns wide.
        DEFW L49CA+$001A                                ; Line 72 column 26.
        DEFB $02                                        ; 2 columns wide.
        DEFW L4D66+$0012                                ; Line 100 column 18.
        DEFB $02                                        ; 2 columns wide.
        DEFW L4D66+$0015                                ; Line 100 column 21.
        DEFB $02                                        ; 2 columns wide.
        DEFW L4D66+$0018                                ; Line 100 column 24.
        DEFB $02                                        ; 2 columns wide.
        DEFW L4D66+$001B                                ; Line 100 column 27.
        DEFB $01                                        ; 1 columns wide.
        DEFW L5102+$0010                                ; Line 128 column 16.
        DEFB $02                                        ; 2 columns wide.
        DEFW L5102+$001C                                ; Line 128 column 28.
        DEFB $02                                        ; 2 columns wide.
        DEFW L549E+$0010                                ; Line 156 column 16.
        DEFB $02                                        ; 2 columns wide.
        DEFW L549E+$001C                                ; Line 156 column 28.
        DEFB $02                                        ; 2 columns wide.

        DEFB $05                                        ; There are 5 ladders.
        DEFW L4589+$0013                                ; Line 39 column 19.
        DEFB $11                                        ; 17 segments tall.
        DEFW L4CC1+$0010                                ; Line 95 column 16.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L4CC1+$001D                                ; Line 95 column 29.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L505D+$0013                                ; Line 123 column 19.
        DEFB $0A                                        ; 10 segments tall.
        DEFW L505D+$001A                                ; Line 123 column 26.
        DEFB $0A                                        ; 10 segments tall.

; -------------------------
; Screen Level 6 Definition
; -------------------------

        DEFB $15                                        ; There are 21 platforms.
        DEFW L462E+$0010                                ; Line 44 column 16.
        DEFB $02                                        ; 2 columns wide.
        DEFW L462E+$0013                                ; Line 44 column 19.
        DEFB $01                                        ; 1 columns wide.
        DEFW L462E+$0015                                ; Line 44 column 21.
        DEFB $01                                        ; 1 columns wide.
        DEFW L462E+$0019                                ; Line 44 column 25.
        DEFB $01                                        ; 1 columns wide.
        DEFW L462E+$001B                                ; Line 44 column 27.
        DEFB $01                                        ; 1 columns wide.
        DEFW L462E+$001D                                ; Line 44 column 29.
        DEFB $01                                        ; 1 columns wide.
        DEFW L49CA+$0012                                ; Line 72 column 18.
        DEFB $01                                        ; 1 columns wide.
        DEFW L49CA+$0014                                ; Line 72 column 20.
        DEFB $01                                        ; 1 columns wide.
        DEFW L49CA+$001C                                ; Line 72 column 28.
        DEFB $02                                        ; 2 columns wide.
        DEFW L4D66+$0010                                ; Line 100 column 16.
        DEFB $02                                        ; 2 columns wide.
        DEFW L4D66+$0015                                ; Line 100 column 21.
        DEFB $01                                        ; 1 columns wide.
        DEFW L4D66+$001A                                ; Line 100 column 26.
        DEFB $02                                        ; 2 columns wide.
        DEFW L5102+$0012                                ; Line 128 column 18.
        DEFB $01                                        ; 1 columns wide.
        DEFW L5102+$0014                                ; Line 128 column 20.
        DEFB $01                                        ; 1 columns wide.
        DEFW L5102+$001C                                ; Line 128 column 28.
        DEFB $02                                        ; 2 columns wide.
        DEFW L549E+$0010                                ; Line 156 column 16.
        DEFB $02                                        ; 2 columns wide.
        DEFW L549E+$0013                                ; Line 156 column 19.
        DEFB $01                                        ; 1 columns wide.
        DEFW L549E+$0015                                ; Line 156 column 21.
        DEFB $01                                        ; 1 columns wide.
        DEFW L549E+$0019                                ; Line 156 column 25.
        DEFB $01                                        ; 1 columns wide.
        DEFW L549E+$001B                                ; Line 156 column 27.
        DEFB $01                                        ; 1 columns wide.
        DEFW L549E+$001D                                ; Line 156 column 29.
        DEFB $01                                        ; 1 columns wide.

        DEFB $01                                        ; There is 1 ladder.
        DEFW L4589+$0017                                ; Line 39 column 23.
        DEFB $20                                        ; 32 segments tall.

; -----------------
; Program Variables
; -----------------

L6401:  DEFW $0000                                      ; Holds the address within the display file of the top left of the player.
L6403:  DEFB $00                                        ; Player vertical speed when flying via the rocket pack ($00=No speed to $07=Maximum speed).
                                                        ; Also holds the number of vertical offsets to apply when the player is jumping ($00 to $08).
L6404:  DEFB $00                                        ; Player horizontal speed when flying via a rocket pack ($00=No speed to $07=Maximum speed).
                                                        ; Also holds the player vertical speed when flying via a vulture ($00=No speed to $05=Maximum speed),
                                                        ; with bit 7 used to indicate whether the jump key was previously being pressed (set=Jump not pressed).
L6405:  DEFB $00                                        ; Player vertical movement timer. Holds the FRAMES low byte number when it is time to move the player vertically when flying.
L6406:  DEFB $00                                        ; Player horizontal movement timer. Holds the FRAMES number when it is time to move the player horizontally when flying.
L6407:  DEFB $00                                        ; Movement flags:
                                                        ;   Bits 3-7: Not used (always 0).
                                                        ;   Bit 2   : 1=Flying right, 0=Flying left.
                                                        ;   Bit 1   : 1=Rocket pack active, 0=Rocket pack inactive.
                                                        ;   Bit 0   : 1=Player faces right, 0=Player faces left.
L6408:  DEFB $00                                        ; Holds the frame number for the player, when caught by the Bubloid, and for the splash.
L6409:  DEFB $00                                        ; Holds a set of flags:
                                                        ;   Bit 7: 1=Fallen into the sea, i.e. draw the splash graphic.
                                                        ;   Bit 6: 1=Flying is via a rocket pack.
                                                        ;   Bit 5: 1=Flying is via a vulture.
                                                        ;   Bit 4: 1=Player has been caught by the Bubloid.
                                                        ;   Bit 3: 1=Player is on a ladder.
                                                        ;   Bit 2: 1=Left of player is on a ladder.
                                                        ;   Bit 1: 1=Right of player is on a ladder.
                                                        ;   Bit 0: 1=Player is on a platform.
L640A:  DEFB $00                                        ; Holds the timer value used to control when to toggle the phase of the wave.
L640B:  DEFB $AA                                        ; Controls the wave animation. The value is rotated left and resultant state of the carry flag determines the phase of the wave.
L640C:  DEFB $00                                        ; Not used [Redundant].
L640D:  DEFB $00                                        ; Bubloid acceleration rate ($01=Fast acceleration to $04=Slow acceleration).
L640E:  DEFB $00                                        ; Holds the previous value of the 10000s digit of the score, used to determine when to award an extra life.
L640F:  DEFB $00                                        ; Not used [Redundant].
L6410:  DEFB $04                                        ; Holds the Bubloid frame number ($01 to $05).
L6411:  DEFW $0000                                      ; Holds the location of Bubloid in the display file.
L6413:  DEFB $00, $00, $00, $00, $00, $00, $00, $00     ; Not used [Redundant].
        DEFB $00, $00, $00                              ;
L641E:  DEFB $03                                        ; Bubloid movement flags:
                                                        ;   Bits 2-7: Not used (always 0).
                                                        ;   Bit 1   : 1=Bubloid is moving up, 0=Bubloid is moving down.
                                                        ;   Bit 0   : 1=Bubloid is moving right, 0=Bubloid is moving left.
L641F:  DEFB $04                                        ; Bubloid vertical acceleration ($01=Fast acceleration to $04=Slow acceleration).
L6420:  DEFB $04                                        ; Bubloid vertical speed ($01=Low speed to $0F=High speed).
L6421:  DEFB $FF                                        ; Bubloid vertical movement timer. Holds the FRAMES low byte number when it is time to move the Bubloid vertically.
L6422:  DEFB $04                                        ; Bubloid horizontal acceleration ($01=Fast acceleration to $04=Slow acceleration).
L6423:  DEFB $04                                        ; Bubloid horizontal speed ($01=Low speed to $0F=High speed).
L6424:  DEFB $FF                                        ; Bubloid horizontal movement timer. Holds the FRAMES low byte number when it is time to move the Bubloid horizontally.
L6425:  DEFB $00                                        ; Holds the level number ($01-$06).

; -------------------
; End of BASIC Line 2
; -------------------

L6426:  DEFB _NEWLINE

; ------------
; BASIC Line 3
; ------------

L6427:  DEFB 3 >> 8
        DEFB 3 & $FF
        DEFW L68C8 - 1 - $
        DEFB _REM

; -----------------
; Control Keys Text
; -----------------
; 'q=up'
; 'z=down'
; 'b=left'
; 'm=right'
; 'a=jump'

L642C:  DEFB _INVQ, _INVEQUALS, _INVU, _INVP, _BLACK, _BLACK, _BLACK
        DEFB _INVZ, _INVEQUALS, _INVD, _INVO, _INVW, _INVN, _BLACK
        DEFB _INVB, _INVEQUALS, _INVL, _INVE, _INVF, _INVT, _BLACK
        DEFB _INVM, _INVEQUALS, _INVR, _INVI, _INVG, _INVH, _INVT
        DEFB _INVA, _INVEQUALS, _INVJ, _INVU, _INVM, _INVP, _BLACK

; --------------------
; 'r to redefine keys'
; --------------------

L644F:  DEFB _INVR, _BLACK, _INVT, _INVO, _BLACK, _INVR, _INVE, _INVD, _INVE, _INVF, _INVI, _INVN, _INVE
        DEFB _BLACK, _INVK, _INVE, _INVY, _INVS

; ------------------------
; 'any other key to start'
; ------------------------

L6461:  DEFB _INVA, _INVN, _INVY, _BLACK, _INVO, _INVT, _INVH, _INVE, _INVR, _BLACK
        DEFB _INVK, _INVE, _INVY, _BLACK, _INVT, _INVO, _BLACK, _INVS, _INVT, _INVA, _INVR, _INVT

; ------------------
; 'press a key for:'
; ------------------

L6477:  DEFB _INVP, _INVR, _INVE, _INVS, _INVS, _BLACK, _INVA, _BLACK, _INVK, _INVE, _INVY, _BLACK
        DEFB _INVF, _INVO, _INVR, _INVCOLON

; ------------------------------
; Fill Hi-Res Display File Black
; ------------------------------

L6487:  LD      HL,L4082-$0001                          ; Point to the hi-res display file.
        LD      B,$C0                                   ; There are 192 lines.
        LD      A,$C9                                   ; The end of the end of line terminator.

L648E:  INC     HL                                      ; Advance to the next display file location.
        CP      (HL)                                    ; Has the end of the line been reached?
        JR      Z,L6496                                 ; Jump ahead if so.

; The end of the line has not been reached.

        LD      (HL),$19                                ; Set the location black.
        JR      L648E                                   ; Jump back to fill the next location.

; The end of the line has been reached.

L6496:  DJNZ    L648E                                   ; Loop back if there are further lines to fill.

        RET     

; --------------------------------
; Fill Standard Display File Black
; --------------------------------

L6499:  LD      B,$18                                   ; There are 24 rows to fill.
        LD      HL,(D_FILE)                             ; Fetch the location of the standard display file.
        LD      A,_NEWLINE                              ; The code of the end of row terminator.

L64A0:  INC     HL                                      ; Advance to the next display file location.
        CP      (HL)                                    ; Has the end of the row been reached?
        JR      Z,L64A8                                 ; Jump ahead if so.

; The end of the row has not been reached.

        LD      (HL),_BLACK                             ; Set the location black.
        JR      L64A0                                   ; Jump back to fill the next location.

; The end of the row has been reached.

L64A8:  DJNZ    L64A0                                   ; Loop back if there are further rows to fill.

        RET     

; ------------
; Score Digits
; ------------
; Each digit is based from 1, i.e. '0' is represented by $01, '1' by $02, etc.
; All digits are initialised to $01 at the start of each game.

L64AB:  DEFB $00                                        ; Most significant digit.
        DEFB $00
        DEFB $00
        DEFB $00
        DEFB $00
        DEFB $00
L64B1:  DEFB $00                                        ; Least significant digit.

; -----------------
; High Score Digits
; -----------------
; Each digit is based from 1, i.e. '0' is represented by $01, '1' by $02, etc.
; The initial value is "0000000".

L64B2:  DEFB $01                                        ; Most significant digit.
        DEFB $01
        DEFB $01
        DEFB $01
        DEFB $01
        DEFB $01
L64B8:  DEFB $01                                        ; Least significant digit.

; ------------
; Bonus Digits
; ------------
; Each digit is based from 1, i.e. '0' is represented by $01, '1' by $02, etc.

L64B9:  DEFB $00                                        ; Most significant digit.
        DEFB $00
        DEFB $00
L64BC:  DEFB $00                                        ; Least significant digit.

; -----------------
; Program Variables
; -----------------

L64BD:  DEFB $00                                        ; Holds a value used to initialise the most significant digit for the bonus. The value is incremented for each new level up to a maximum of value of $0B.
L64BE:  DEFB $00                                        ; Holds the number of lives + 1.
L64BF:  DEFW $0000                                      ; Holds the location of the displayed fuel can or leg of lamb.
L64C1:  DEFW $0000                                      ; Holds the address of the top line of the 1st diamond (line 38), or $0000 if it has been collected.
L64C3:  DEFW $0000                                      ; Holds the address of the top line of the 2nd diamond (line 66), or $0000 if it has been collected.
L64C5:  DEFW $0000                                      ; Holds the address of the top line of the 3rd diamond (line 94), or $0000 if it has been collected.
L64C7:  DEFW $0000                                      ; Holds the address of the top line of the 4th diamond (line 122), or $0000 if it has been collected.
L64C9:  DEFW $0000                                      ; Holds the address of the top line of the 5th diamond (line 150), or $0000 if it has been collected.

; -------------------------
; Decrement the Bonus Value
; -------------------------
; Each digit of the bonus is based from 1, i.e. 1 represent 0, 2 represents 1, ..., 10 represents 9.
; Entry: HL=Last digit of the bonus.
;        B=Number of digits of the bonus.
;        C=$00 (will be incremented for each digit of the bonus that must be redrawn).
; Exit : C=Number of digits that require redrawing.

L64CB:  INC     C                                       ; Increment the digit redraw count.

        LD      A,(HL)                                  ; Fetch the current digit.
        DEC     A                                       ; Decrement it.
        JR      NZ,L64D7                                ; Jump ahead if it has not overflowed.

; The digit has overflowed.

        LD      A,$0A                                   ; The value representing 9.
        LD      (HL),A                                  ; Store it for the current digit.

        DEC     HL                                      ; Point to the next least significant digit.
        DJNZ    L64CB                                   ; Repeat for all digits of the bonus.

; All digits were updated and so the bonus has expired.

        RET     

; The digit has not overflowed.

L64D7:  LD      (HL),A                                  ; Store the new digit value.
        RET     

; --------
; Not Used
; --------

L64D9:  RET                                             ; [Redundant]

; -----------------------------------
; Copy Block Into Hi-Res Display File
; -----------------------------------
; Entry: DE=Address of the block to copy.
;        HL=Address in the display file.
;        B=The number of lines to copy.
; Exit : DE=Address after the block.

L64DA:  PUSH    DE                                      ; Save the address of the block data.

        LD      A,(DE)                                  ; Fetch a byte of the block data.
        LD      DE,$0021                                ; There are 33 bytes per line of the display file.
        LD      (HL),A                                  ; Copy the block byte into the display file.
        ADD     HL,DE                                   ; Advance to the next line within the display file.

        POP     DE                                      ; Retrieve the address of the block data.
        INC     DE                                      ; Advance to the next byte of the block data.
        DJNZ    L64DA                                   ; Repeat for all lines.

        RET     

; -------------------
; Wait For Unique Key
; -------------------
; The key stores will initially have been set to $FFFF for no key and so this routine
; will interpret a no key press as a match with the existing key store entries and so
; will wait until a key has been pressed.
; Entry: HL=Key store address.
; Exit:  HL=Key store address.
;        DE=Unique key code.

L64E6:  PUSH    HL                                      ; Save the key store address.

L64E7:  LD      DE,(LAST_K)                             ; Fetch the key code from the last keyboard read.

        LD      HL,(L65C1)                              ; Point at the store the 'up' key.
        AND     A
        SBC     HL,DE                                   ; Has the new key previously been selected?
        JR      Z,L64E7                                 ; Jump back if so to await a different key press.

        LD      HL,(L65C3)                              ; Point at the store the 'down' key.
        AND     A
        SBC     HL,DE                                   ; Has the new key previously been selected?
        JR      Z,L64E7                                 ; Jump back if so to await a different key press.

        LD      HL,(L65C5)                              ; Point at the store the 'left' key.
        AND     A
        SBC     HL,DE                                   ; Has the new key previously been selected?
        JR      Z,L64E7                                 ; Jump back if so to await a different key press.

        LD      HL,(L65C7)                              ; Point at the store the 'right' key.
        AND     A
        SBC     HL,DE                                   ; Has the new key previously been selected?
        JR      Z,L64E7                                 ; Jump back if so to await a different key press.

        LD      HL,(L65C9)                              ; Point at the store the 'jump' key.
        AND     A
        SBC     HL,DE                                   ; Has the new key previously been selected?
        JR      Z,L64E7                                 ; Jump back if so to await a different key press.

        POP     HL                                      ; Retrieve the key store address.
        RET     

; -----------------------------------
; Render Value in Hi-Res Display File
; -----------------------------------
; This routine is used to render the score, high score and bonus values into the hi-res display file.
; Entry: BC=Address of the last digit of the value.
;        HL=Address within the hi-res display file for the last digit.
;        A=Number of digits.

L6515:  PUSH    BC                                      ; Save the address of the digit value to display.
        PUSH    AF                                      ; Save the number of digits.
        PUSH    HL                                      ; Save the location within the display file.

        LD      A,(BC)                                  ; Fetch the number digit to display.
        LD      B,A
        LD      HL,L6163-$0008                          ; Base address of the number graphics.
        LD      DE,$0008                                ; There are 8 lines per number.

L6520:  ADD     HL,DE                                   ; Advance to the next number.
        DJNZ    L6520                                   ; Loop back until the desired number graphic has been reached.

        EX      DE,HL                                   ; DE=Address of the number digit graphic.

        POP     HL                                      ; Retrieve the location within the display file.
        PUSH    HL                                      ; Save it again.

        LD      B,$08                                   ; There are 8 lines forming the number.

L6528:  LD      A,(DE)                                  ; Fetch a line of the number graphic.
        LD      (HL),A                                  ; Copy it into the display file.

        PUSH    DE                                      ; Save the address of the line within number digit graphic.
        LD      DE,$0021
        ADD     HL,DE                                   ; Advance to the next line in the display file.
        POP     DE                                      ; Retrieve the address of the line within number digit graphic.

        INC     DE                                      ; Advance to the next line of the number digit graphic.
        DJNZ    L6528                                   ; Loop back to copy all lines of the number digit into the display file.

        POP     HL                                      ; Retrieve the location within the display file.
        POP     AF                                      ; Retrieve the number of digits.
        POP     BC                                      ; Retrieve the address of the digit value to display.

        DEC     HL                                      ; Point at the previous column within the display file.
        DEC     BC                                      ; Point at the next significant digit of the value.

        DEC     A                                       ; Decrement the count of the number of digits.
        JR      NZ,L6515                                ; Loop back to render all digits of the value.

        RET     

; -----------
; Entry Point
; -----------

; Show the hi-res display file which has been pre-populated with the title screen image of the cosmic cockerel.

L653C:  LD      A,$0C                                   ; Select the pixel pattern to display the title screen with.
        LD      I,A
        LD      IX,L612F                                ; Select to display the hi-res display file.

; Wait for a new key press.

        LD      DE,$FFFF                                ; The code for 'no key press'.

L6547:  LD      HL,(LAST_K)                             ; Fetch the key code from the last keyboard read.
        AND     A
        SBC     HL,DE                                   ; Has a key been pressed?
        JR      Z,L6547                                 ; Loop back if not.

        CALL    L6487                                   ; Fill the hi-res display file black.
        CALL    L6499                                   ; Fill the standard display file black.

; Display the standard display file which is used for configuring the controls.

        LD      A,$1E                                   ; Select the pixel pattern for the standard Sinclair characters.
        LD      I,A
        LD      IX,$0281                                ; Select to display the standard display file.

; Display the current control keys.

        LD      HL,(D_FILE)                             ; Fetch the address of the standard display file.
        LD      DE,$00B2
        ADD     HL,DE                                   ; Advance to row 5 column 12.

        LD      DE,L642C                                ; Point to the control keys text.
        LD      C,$05                                   ; There are 5 rows (up, down, left, right and jump).

L6569:  LD      B,$07                                   ; There are 7 characters per row.

L656B:  LD      A,(DE)                                  ; Fetch the character to copy into the current row of the display file.
        LD      (HL),A                                  ; Insert it into the display file.
        INC     HL                                      ; Advance to the next character to copy.
        INC     DE                                      ; Advance to the next display file location.
        DJNZ    L656B                                   ; Repeat for all characters to copy to this row.

        PUSH    DE                                      ; Save the address of the next row of text to copy.
        LD      DE,$001A
        ADD     HL,DE                                   ; Advance to the row below in the display file.
        POP     DE                                      ; Retrieve the address of the next row of text to copy.
        DEC     C                                       ; Decrement the row count.
        JR      NZ,L6569                                ; Loop back if there are further rows of text to display.

; Display 'r to redefine keys'.

        LD      HL,(D_FILE)                             ; Fetch the address of the standard display file.
        LD      DE,$0173
        ADD     HL,DE                                   ; Advance to row 11 column 7.

        LD      DE,L644F                                ; Point at the text to copy into the display file.
        LD      B,$12                                   ; There are 18 characters to display.

L6586:  LD      A,(DE)                                  ; Fetch the character to copy into the current row of the display file.
        LD      (HL),A                                  ; Insert it into the display file.
        INC     HL                                      ; Advance to the next character to copy.
        INC     DE                                      ; Advance to the next display file location.
        DJNZ    L6586                                   ; Repeat for all characters of the text.

; Display 'any other key to start'.

        LD      BC,$002E
        ADD     HL,BC                                   ; Advance to row 13 column 5.

        LD      B,$16                                   ; There are 24 characters to display.

L6592:  LD      A,(DE)                                  ; Fetch the character to copy into the current row of the display file.
        LD      (HL),A                                  ; Insert it into the display file.
        INC     HL                                      ; Advance to the next character to copy.
        INC     DE                                      ; Advance to the next display file location.
        DJNZ    L6592                                   ; Repeat for all characters of the text.

        LD      DE,$FFFF                                ; The key code for 'no key pressed'.

; Wait until no keys are being pressed.

L659B:  LD      HL,(LAST_K)                             ; Fetch the key code from the last keyboard read.
        AND     A
        SBC     HL,DE                                   ; Was a key pressed?
        JR      NZ,L659B                                ; Loop back if so.

; Wait until a key is pressed.

L65A3:  LD      HL,(LAST_K)                             ; Fetch the key code from the last keyboard read.
        AND     A
        SBC     HL,DE                                   ; Was a key pressed?
        JR      Z,L65A3                                 ; Loop back if not.

; A key has been pressed.

        LD      HL,(LAST_K)                             ; Fetch the key code from the last keyboard read.
        LD      DE,$EFFB                                ; The key code for 'R'.
        AND     A
        SBC     HL,DE                                   ; Was 'R' pressed?
        JP      NZ,L6676                                ; Jump if not to start the game.

; 'R' was pressed.

L65B7:  LD      HL,(LAST_K)                             ; Fetch the key code from the last keyboard read.
        AND     A
        SBC     HL,DE                                   ; Is 'R' still being pressed?
        JR      Z,L65B7                                 ; Loop back if so.

; 'R' has been released.

        JR      L65D3                                   ; Jump ahead to redefine the keys.

; ---------------
; Key Code Stores
; ---------------
; These locations hold the key codes for the game controls. The combination key codes are configured but never used.

L65C1:  DEFW $FDFB                                      ; The key code for 'up'.
L65C3:  DEFW $FBFE                                      ; The key code for 'down'.
L65C5:  DEFW $DF7F                                      ; The key code for 'left'.
L65C7:  DEFW $F77F                                      ; The key code for 'right'.
L65C9:  DEFW $FDFD                                      ; The key code for 'jump'.
L65CB:  DEFW $FDF9                                      ; The key code for 'up' + 'jump'.
L65CD:  DEFW $F9FC                                      ; The key code for 'down' + 'jump'.
L65CF:  DEFW $DD7D                                      ; The key code for 'left' + 'jump'.
L65D1:  DEFW $F57D                                      ; The key code for 'right' + 'jump'.

; -------------
; Redefine Keys
; -------------

; Clear all key stores to $FFFF, i.e. no key.

L65D3:  LD      B,$12                                   ; There are 9 key store entries.
        LD      HL,L65C1                                ; Point to the start of the key stores.

L65D8:  LD      (HL),$FF                                ; Clear a byte within the store.
        INC     HL                                      ; Advance to the next location within the store.
        DJNZ    L65D8                                   ; Repeat for all store locations.

        CALL    L6499                                   ; Fill the standard display file black.

        LD      HL,(D_FILE)                             ; Fetch the location of the standard display file.
        LD      DE,$0111
        ADD     HL,DE                                   ; Advance to row 8 column 8.

        LD      DE,L6477                                ; Point to message 'press a key for:'.
        LD      B,$10                                   ; There are 16 characters in the message.

L65EC:  LD      A,(DE)                                  ; Fetch a character from the message.
        LD      (HL),A                                  ; Insert it into the display file.
        INC     HL                                      ; Advance to the next location within the display file.
        INC     DE                                      ; Advance to the next character of the message.
        DJNZ    L65EC                                   ; Loop back to insert all characters of the message.

        LD      DE,$0059
        ADD     HL,DE                                   ; Advance to row 11 column 14.
        EX      DE,HL                                   ; Transfer the display file location to DE.

        LD      BC,L642C+$0002                          ; Point to the description text for the 'up' key.
        LD      HL,L65C1                                ; The address of the key code stores.
        LD      A,$05                                   ; There are 5 redefinable keys.

; Enter a loop to fetch 5 unique key presses.

L65FF:  PUSH    AF                                      ; Save the key counter.

        PUSH    DE                                      ; Save the display file location.
        LD      A,$05                                   ; There are 5 characters in the text.

L6603:  PUSH    AF                                      ; Save the character count.

        LD      A,(BC)                                  ; Fetch a character of the text.
        LD      (DE),A                                  ; Insert it into the display file.
        INC     BC                                      ; Advance to the next text character.
        INC     DE                                      ; Advance to the next display file location.

        POP     AF                                      ; Retrieve the character count.
        DEC     A                                       ; Decrement the character count.
        JR      NZ,L6603                                ; Loop back if there are further characters to display.

        INC     BC                                      ; Skip over the text for the next key and equals sign to
        INC     BC                                      ; point at the description text for that key.

        CALL    L64E6                                   ; Wait for a unique key press.

        LD      (HL),E
        INC     HL
        LD      (HL),D                                  ; Store the key press in the key store.
        INC     HL

        POP     DE                                      ; Retrieve the display file location.
        POP     AF                                      ; Retrieve the key counter.
        DEC     A                                       ; Decrement the key counter.
        JR      NZ,L65FF                                ; Loop back for all 5 keys.

; 5 unique keys have been fetched. Codes are computed for the combination of each direction key with the jump key. However, these are then never used.

        EX      DE,HL                                   ; DE=Address of the direction-with-jump key entries within the key store.
        LD      A,$04                                   ; There are 4 direction keys.
        LD      BC,L65C1                                ; Point at the keys store.

L6620:  PUSH    AF                                      ; Save the key counter.

        LD      HL,L65C9                                ; Address of the key store entry for the 'jump' key.

        LD      A,(BC)                                  ; Fetch the low byte of the key code for the up key.
        AND     (HL)                                    ; Combine it with the low byte of the jump key.
        LD      (DE),A                                  ; Store it in the key store.

        INC     HL                                      ; Advance to the high byte of the jump entry in the key store.
        INC     DE                                      ; Advance to the high byte of the destination entry within the key store.
        INC     BC                                      ; Advance to the  byte of the direction key entry within the key store.

        LD      A,(BC)                                  ; Fetch the high byte of the key code for the up key.
        AND     (HL)                                    ; Combine it with the high byte of the jump key.
        LD      (DE),A                                  ; Store it in the key store.

        INC     DE                                      ; Advance to the low byte of the next destination entry within the key store.
        INC     BC                                      ; Advance to the low byte of the next direction key entry within the key store.

        POP     AF                                      ; Retrieve the key counter.
        DEC     A                                       ; Decrement the key counter.
        JR      NZ,L6620                                ; Loop back for all direction keys.

        JR      L6676                                   ; Jump ahead to start the game.

; ----------------------------------------------------
; Determine Random Location for Fuel Can / Leg of Lamb
; ----------------------------------------------------
; This routine selects a platform location to place the fuel can / leg of lamb.
; It selects a column at random using the value from the FRAMES system variable and searches
; downwards to find at platform at this column. When one is found, it searches for 3 columns
; to the left for a free valid location, i.e. above a platform and not clashing with the
; player. If a suitable location is not found then a new random column is chosen and the
; search process repeated until it succeeds. Platform levels are at lines 16, 44, 72, 100 and 128.
; Exit : HL=Display file location verified as suitable for the fuel can / leg of lamb.

L6635:  EXX                                             ; Preserve the main registers.

; Select a random column position between 15 and 30.

L6636:  LD      A,(FRAMES)                              ; Fetch the low byte of system variable FRAMES.
        AND     $0F                                     ; Reduce to a value of 0-15.
        INC     A                                       ; Increment the range to 1-16.

        LD      HL,L4292+$0020                          ; Line 16 column 32.

L663F:  DEC     HL                                      ; Point to the column to the left.
        DEC     A                                       ; Decrement the count.
        JR      NZ,L663F                                ; Repeat for all decrements.

        LD      DE,$039C                                ; Offset of 28 lines.

        LD      A,(FRAMES)                              ; Fetch the low byte of system variables FRAMES.
        SET     3,A                                     ; Make sure a maximum of 5 platform levels are tested.

L664B:  ADD     HL,DE                                   ; Advance down to the next platform level.
        RL      A                                       ; Is there another platform level to be tested?
        JR      NC,L664B                                ; Loop back if not to test the next platform level.

        LD      B,$03                                   ; A search will be made for 3 consecutive locations to the left.

L6652:  LD      A,(HL)                                  ; Fetch the character from the display file.

L6653:  CP      $0B                                     ; Is it the top of a platform (........)?
        JR      Z,L665C                                 ; Jump ahead if so.

        DEC     HL                                      ; Point to the column to the left.
        DJNZ    L6652                                   ; Repeat the test for the new column.

        JR      L6636                                   ; Jump back to select a new location to test.

; A platform has been found.

; *BUG* - It is possible that the Bubloid is in mid air at the same height as a platform and gets mistaken for a platform. This leads
; to the fuel can / leg of lamb being placed in mid air. A solution would be to check that the following lines contains the rest of
; the platform, but this would mean that the fuel can / leg of lamb could not be placed on a platform that is being overlapped by the Bubloid.

L665C:  LD      DE,$0108                                ; Offset of 8 lines.
        AND     A
        SBC     HL,DE                                   ; Point to 8 lines above the platform.

        JP      L781E                                   ; Jump to check whether the chosen location clashes with the player.

; --------
; Not Used
; --------

L6665:  RET                                             ; [Redundant]

; --------
; Not Used
; --------

L6666:  LD      D,D                                     ; [Redundant]
        POP     HL
        JR      NC,L6635-$0001

        JR      L6653

; --------
; Not Used
; --------

L666C:  LD      DE,$0108                                ; [Redundant]
        AND     A
        SBC     HL,DE
        PUSH    HL
        EXX     
        POP     HL
        RET     

; ----------
; Game Start
; ----------

L6676:  LD      HL,L688C                                ; Point at the store holding the fuel usage timer.
        LD      (HL),$01                                ; Initialise it to its lowest, i.e. no usage.

        CALL    L6499                                   ; Fill the standard display file black.

        LD      HL,L629C                                ; Point at the definition for screen level 1.
        LD      (L67B3+$0001),HL                        ; Save this as the first screen to play.

        LD      HL,L640D
        LD      (HL),$08                                ; Set the Bubloid acceleration rate, which will be reduced to $04 (slowest acceleration) when the level is started.
        INC     HL
        LD      (HL),$01                                ; Initialise the 'Previous 10000s' score digit, used to determine when to award an extra life.
        INC     HL
        LD      (HL),$01                                ; Never used [Redundant].

        LD      HL,L6425                                ; Point at the store holding the level number.
        LD      (HL),$00                                ; It will be incremented prior to starting the first level and so will have a value of $01.

        LD      HL,L64AB                                ; Point to the first digit of the score.
        LD      B,$07                                   ; The are 7 digits in the score.

L6699:  LD      (HL),$01                                ; Store the value for '0' (the digits are always stored incremented by 1 to make the code to display them simpler).
        INC     HL                                      ; Advance to the next digit.
        DJNZ    L6699                                   ; Loop back to initialise all digits.

        LD      HL,L64BD
        LD      (HL),$01                                ; Set the initial bonus amount.
        INC     HL
        LD      (HL),$06                                ; Set the number of lives + 1.

        LD      HL,L5DF9                                ; The base address for Bubloid graphics sets (when indexed from 0).
        LD      (L74FB+$0001),HL                        ; Change the program to set this as the active set. It will be advanced when the level is started and so point at set 1.

        LD      A,$0C                                   ; Select the pixel pattern to display the hi-res display file with.
        LD      I,A
        LD      IX,L612F                                ; Select to display the hi-res display file.

; Joins here when starting the next level.

L66B4:  CALL    L6487                                   ; Fill the hi-res display file black.

; Determine the Bubloid acceleration.

        LD      A,(L640D)                               ; Fetch the Bubloid acceleration rate.
        SRL     A                                       ; Divide it by 2 to make the Bubloid acceleration rate faster than the previous level.

        NOP                                             ; [Redundant]
        NOP     

        JR      NZ,L66C2                                ; Jump ahead if the Bubloid acceleration rate hasn't reached $00.

        LD      A,$04                                   ; Reset the Bubloid acceleration rate to the slowest level.

L66C2:  LD      (L640D),A                               ; Store the new Bubloid acceleration rate.

; There are 3 sets of Bubloid graphics. Each consists of 5 frames, with the first and
; last frames only a single column wide and the other frames of 2 columns wide.
; The sets are accessed using a 0 based index, and hence the base addresses for the sets are:
; - Before = 5DF9
; - Set 1  = 5E79
; - Set 2  = 5EF9
; - Set 3  = 5F79
; - End    = 5FF9

; Select the next Bubloid graphics set to use.

        LD      HL,(L74FB+$0001)                        ; Fetch the active Bubloid graphics set base address.
        LD      DE,$0080                                ; The size of each set.
        ADD     HL,DE                                   ; Advance to the next Bubloid graphics set.
        PUSH    HL                                      ; Save the current set address.

        LD      DE,L5FF9                                ; Address after Bubloid graphics set 3.
        AND     A
        SBC     HL,DE                                   ; Has set 3 been exceeded?

        POP     HL                                      ; Retrieve the current set address.
        JR      NZ,L66D9                                ; Jump ahead if not.

        LD      HL,L5E79                                ; Wrap back to Bubloid graphics set 1.

L66D9:  LD      (L74FB+$0001),HL                        ; Store the address of the Bubloid graphic set to use.

        NOP                                             ; [Redundant]
        NOP     
        NOP     

; Increment the level number.

        LD      HL,L6425                                ; Fetch the level number.
        INC     (HL)                                    ; Increment the level number.
        LD      A,$07
        CP      (HL)                                    ; Has the new level number reach 7?
        JR      NZ,L66F4                                ; Jump ahead if not.

; The level number has exceeded the maximum and so must be wrapped back to 1.

        LD      (HL),$01                                ; Reset to the first level.

        NOP                                             ; [Redundant]
        NOP     
        NOP     
        NOP     
        NOP     
        NOP     
        NOP     
        NOP     
        NOP     
        NOP     

; Initialise the wave timer.

L66F4:  LD      HL,L640A                                ; Point to the wave timer.
        LD      (HL),$FF                                ; Initialise it.

; Determine the value to initialise the bonus to.

        LD      HL,L64BD                                ; Point at the bonus amount.
        INC     (HL)                                    ; Increment it.
        LD      A,$0B
        CP      (HL)                                    ; Has the maximum value been exceeded?
        JR      NZ,L6703                                ; Jump ahead if not.

        DEC     (HL)                                    ; Decrement back to the maximum value.

L6703:  LD      A,(HL)                                  ; Store the new bonus amount.

; Initialise the bonus digits. The most significant digit is set from the bonus amount, with all other
; digits set to '0'.

        LD      B,$03                                   ; There are 3 digits of the bonus to reset.

L6706:  DEC     HL                                      ; Point at a digit of the bonus value.
        LD      (HL),$01                                ; Initialise it to '0'.
        DJNZ    L6706                                   ; Repeat for the 3 least significant digits.

        DEC     HL                                      ; Point at the most significant digit of the bonus.
        LD      (HL),A                                  ; Set it from the bonus amount.

; Store the display file addresses of the diamonds.

        LD      HL,L4568                                ; The display file address of line 38.
        LD      (L64C1),HL                              ; Set as the address within the display file of the top line of the 1st diamond.

        LD      HL,L4904                                ; The display file address of line 66.
        LD      (L64C3),HL                              ; Set as the address within the display file of the top line of the 2nd diamond.

        LD      HL,L4CA0                                ; The display file address of line 94.
        LD      (L64C5),HL                              ; Set as the address within the display file of the top line of the 3rd diamond.

        LD      HL,L503C                                ; The display file address of line 122.
        LD      (L64C7),HL                              ; Set as the address within the display file of the top line of the 4th diamond.

        LD      HL,L53D8                                ; The display file address of line 150.
        LD      (L64C9),HL                              ; Set as the address within the display file of the top line of the 5th diamond.

; Copy the score into the hi-res display file.

        LD      HL,L40C4+$0007                          ; Line 2 column 7.
        LD      BC,L64B1                                ; Point to the last digit of the score.
        LD      A,$07                                   ; There are 7 digits in the score.
        CALL    L6515                                   ; Display the score value in the display file.

; Copy the graphic "SCORE" into the hi-res display file.

        LD      HL,L40E5+$0008                          ; Line 3 column 8.
        LD      DE,L61B3                                ; The address of the graphic data.
        LD      C,$04                                   ; There are 4 columns.

L673E:  LD      B,$08                                   ; There are 8 lines to copy.
        PUSH    HL                                      ; Save the display file address.
        CALL    L64DA                                   ; Copy the graphic into the display file.
        POP     HL                                      ; Retrieve the display file address.
        INC     HL                                      ; Advance to the next column within the display file.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L673E                                ; Repeat for all columns.

; Copy the graphic "HI" into the hi-res display file.

        LD      HL,L40E5+$0014                          ; Line 3 column 20.
        LD      C,$02                                   ; There are 2 columns.

L674E:  LD      B,$08                                   ; There are 8 lines to copy.
        PUSH    HL                                      ; Save the display file address.
        CALL    L64DA                                   ; Copy the graphic into the display file.
        POP     HL                                      ; Retrieve the display file address.
        INC     HL                                      ; Advance to the next column within the display file.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L674E                                ; Repeat for all columns.

; Copy the graphic "BONUS" into the hi-res display file.

        LD      HL,L40E5+$001B                          ; Line 3 column 27.
        LD      C,$05                                   ; There are 5 columns.

L675E:  LD      B,$08                                   ; There are 8 lines to copy.
        PUSH    HL                                      ; Save the display file address.
        CALL    L64DA                                   ; Copy the graphic into the display file.
        POP     HL                                      ; Retrieve the display file address.
        INC     HL                                      ; Advance to the next column within the display file.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L675E                                ; Repeat for all columns.

; Copy the high score into the hi-res display file.

        LD      HL,L40E5+$0013                          ; Line 3 column 19.
        LD      BC,L64B8                                ; Point to the last digit of the high score.
        LD      A,$07                                   ; There are 7 digits in the high score.
        CALL    L6515                                   ; Display the high score value in the display file.

; Copy the bonus value into the hi-res display file.

        LD      HL,L40E5+$001A                          ; Line 3 column 26.
        LD      BC,L64BC                                ; Point to the last digit of the bonus.
        LD      A,$04                                   ; There are 4 digits in the bonus value.
        CALL    L6515                                   ; Display the bonus value in the display file.

; Insert the platforms for the diamonds into the hi-res display file.

        LD      A,$05                                   ; There are 5 platforms.
        LD      HL,L462E                                ; Line 44 column 0.

L6784:  PUSH    AF                                      ; Save the platform count.
        LD      C,$02                                   ; The platform is 2 columns wide.

L6787:  LD      DE,L615B                                ; Point to the graphic for the platforms for the diamonds.
        LD      B,$04                                   ; The are 4 lines to copy.
        PUSH    HL                                      ; Save the address within the display file.
        CALL    L64DA                                   ; Copy the graphic into the display file.
        POP     HL                                      ; Retrieve the address within the display file.

        INC     HL                                      ; Advance to the next column in the display file.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6787                                ; Loop back for all columns.

        LD      DE,$039A                                ; Offset of 33 lines and 31 columns.
        ADD     HL,DE                                   ; Advance to 34 lines below.

        POP     AF                                      ; Retrieve the platform count.
        DEC     A                                       ; Decrement the platform count.
        JR      NZ,L6784                                ; Loop back to draw the next platform.

; Insert the diamonds into the hi-res display file.

        LD      C,$05                                   ; There are 5 diamonds.
        LD      HL,L4568                                ; Line 38 column 0.

L67A2:  LD      DE,L6129                                ; Point to the graphic for a diamond.
        LD      B,$06                                   ; There are 6 lines to copy.
        PUSH    HL                                      ; Save the address within the display file.
        CALL    L64DA                                   ; Copy the graphic into the display file.
        POP     HL                                      ; Retrieve the address within the display file.

        LD      DE,$039C                                ; Offset of 28 lines and 0 columns.
        ADD     HL,DE                                   ; Advance to 28 lines below.

        DEC     C                                       ; Decrement the diamond count.
        JR      NZ,L67A2                                ; Loop back to draw the next diamond.

; Draw the screen level specific layout (platforms and ladders).

L67B3:  LD      HL,L629C                                ; Point to the screen level definition. The address is modified by the game as the player reaches each new level.
        LD      A,(HL)                                  ; Fetch the number of platforms to insert.

; Enter a loop to insert platforms into the display file.

L67B7:  PUSH    AF                                      ; Save the platform count.

        INC     HL
        LD      E,(HL)                                  ; Fetch the low byte of the address within the display file.
        INC     HL
        LD      D,(HL)                                  ; Fetch the high byte of the address within the display file.
        INC     HL
        LD      C,(HL)                                  ; Fetch the width of the platform, i.e. number of segments.

        PUSH    HL                                      ; Save the pointer into the screen level table.
        EX      DE,HL                                   ; HL=Display file address.

; Enter a loop to insert the segments of a platform into the display file.

L67C0:  LD      DE,L6157                                ; Address of the platform segment graphic.
        LD      B,$04                                   ; The segment consists of 4 lines.
        PUSH    HL                                      ; Save the display file address.
        CALL    L64DA                                   ; Copy the graphic into the display file.
        POP     HL                                      ; Retrieve the display file address.

        INC     HL                                      ; Advance to the next column within the display file.
        DEC     C                                       ; Decrement the platform width counter.
        JR      NZ,L67C0                                ; Loop back for each segment of the platform.

; Check whether there is another platform to insert.

        POP     HL                                      ; Retrieve the pointer into the screen level table.

        POP     AF                                      ; Retrieve the platform count.
        DEC     A                                       ; Decrement the platform count.
        JR      NZ,L67B7                                ; Loop back to draw the next platform.

; Insert all ladders into the display file.

        INC     HL
        LD      A,(HL)                                  ; Fetch the number of ladders to display.

; Enter a loop to insert ladder segments into the display file.

L67D5:  PUSH    AF                                      ; Save the ladder count.

        INC     HL
        LD      E,(HL)                                  ; Fetch the low byte of the address within the display file.
        INC     HL
        LD      D,(HL)                                  ; Fetch the high byte of the address within the display file.
        INC     HL
        LD      C,(HL)                                  ; Fetch the number of segments in the ladder.

        PUSH    HL                                      ; Save the pointer into the screen level table.
        EX      DE,HL                                   ; HL=Display file address.

L67DE:  LD      DE,L615F                                ; Address of the ladder segment graphic.
        LD      B,$04                                   ; The segment consists of 4 lines.
        CALL    L64DA                                   ; Copy the graphic into the display file.

        DEC     C                                       ; Decrement the segment counter. 
        JR      NZ,L67DE                                ; Loop back to insert the next segment.

        POP     HL                                      ; Retrieve the pointer into the screen level table.

        POP     AF                                      ; Retrieve the ladders count.
        DEC     A                                       ; Decrement the ladders count.
        JR      NZ,L67D5                                ; Loop back to draw the next ladder.

; Determine the address of the definition for the next screen level.

        INC     HL                                      ; Point at the next level definition.
        PUSH    HL                                      ; Save the screen level definition address.

        LD      DE,L6401                                ; The address after the definition for screen level 6.
        AND     A
        SBC     HL,DE                                   ; Has screen level 6 just been drawn?

        POP     HL                                      ; Retrieve the screen level definition address.
        JR      NZ,L67FC                                ; Jump ahead if not.

        LD      HL,L629C                                ; Wrap back to screen level 1.

L67FC:  LD      (L67B3+$0001),HL                        ; Save the address of the definition for the next level.

; Erase the area underneath the game screen.

        LD      HL,L57B6                                ; Line 180 column 0.
        LD      C,$20                                   ; There are 32 columns to draw.

L6804:  PUSH    HL                                      ; Save the location within the display file.
        LD      DE,$0021                                ; There are 33 bytes per line.
        LD      B,$0C                                   ; There are 8 lines to draw.

L680A:  LD      (HL),$0B                                ; Draw a blank character (........).
        ADD     HL,DE                                   ; Advance to the next line within the display file.
        DJNZ    L680A                                   ; Repeat for all lines.

        POP     HL                                      ; Retrieve the location within the display file.
        INC     HL                                      ; Advance to the next column within the display file.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6804                                ; Loop back to draw for all columns.

; Select a random location for the fuel can or leg of lamb.

        CALL    L6635                                   ; Select a random location for fuel can / leg of lamb.
        LD      (L64BF),HL                              ; Save as the location of the fuel can / leg of lamb.

; Draw the fuel can or leg of lamb depending on the level.

        LD      A,(L6425)                               ; Fetch the level number.
        BIT     2,A                                     ; Level 1, 2 or 3?
        JR      Z,L682C                                 ; Jump ahead if so.

; Level 4, 5 or 6 - draw leg of lamb.

        LD      DE,$0042                                ; The leg of lamb is lines shorter than the full can.
        ADD     HL,DE                                   ; Advance 2 lines lower in the display file.

        LD      B,$06                                   ; There are 6 lines to copy.
        LD      DE,L6296                                ; Point to the graphic for a leg of lamb.
        JR      L6831                                   ; Jump ahead to continue.

; Level 1, 2 or 3 - draw fuel can.

L682C:  LD      B,$08                                   ; There are 8 lines to copy.
        LD      DE,L628E                                ; Point to the graphic for a fuel can.

L6831:  CALL    L64DA                                   ; Copy the graphic into the display file.

        LD      HL,L688D                                ; Point to the location holding the fuel level.
        LD      (HL),$00                                ; Set the fuel level as empty.

        LD      C,(HL)                                  ; Fetch the fuel level.
        CALL    L68AD                                   ; Draw the fuel gauge.

; Joins here to restart the current level.

L683D:  LD      HL,L5564+$0009                          ; Line 162 column 9.
        LD      (L6411),HL                              ; Set the initial location of the Bubloid within the display file.

; *BUG* - If the player was killed while flying but still has fuel remaining then the rocket pack / vulture should be drawn. Failure to do so means
; it is possible for the player to collect the rocket pack / vulture even though it is not visible.

        JP      L785F                                   ; Jump to a patch that waits for a new key to be pressed before starting the screen level. The code returns below to L6846.

; A key has been pressed to start the screen level.

L6846:  LD      (FRAMES),A                              ; This sets the low byte of FRAMES to $00, which causes player and Bubloid movement timers to expire on the next cycle.

; *BUG* - The wave timer should be re-initialised here. Failure to do so can mean the wave graphic does not move for several seconds.

; Decrement the number of lives and update the lives indicators shown in the display file.

        LD      HL,L64BE                                ; Point to the number of lives remaining.
        DEC     (HL)                                    ; Decrement the number of lives.
        JP      Z,L7795                                 ; Jump if all lives have been lost.

        LD      C,(HL)                                  ; Fetch the number of lives remaining.
        CALL    L688E                                   ; Draw the lives indicators.

; Set the left column background for the player to all black.

        LD      HL,L5949                                ; Point to the buffer for the left column background.
        LD      B,$10                                   ; There are 16 lines.

L6859:  LD      (HL),$19                                ; The hi-res code for a 'black' character.
        INC     HL                                      ; Advance to the next line.
        DJNZ    L6859                                   ; Repeat for all lines.

; Set the right column background for the player to all black.

        LD      HL,L5A39                                ; Point to the buffer for the right column background.
        LD      B,$10                                   ; There are 16 lines.   

L6863:  LD      (HL),$19                                ; The hi-res code for a 'black' character.
        INC     HL                                      ; Advance to the next line.
        DJNZ    L6863                                   ; Repeat for all lines.

; Set up the player's starting location.

        LD      HL,L528E+$0011                          ; Line 140 column 17.
        LD      (L6401),HL                              ; Save as the address within the display file of the player.

; Initialise program variables.

        LD      HL,L6403
        LD      (HL),$00                                ; The player initially has no vertical speed.
        INC     HL
        LD      (HL),$00                                ; The player initially has no horizontal speed.
        INC     HL
        LD      (HL),$FF                                ; The player will be moved vertically on the next cycle.
        INC     HL
        LD      (HL),$FF                                ; The player will be moved horizontally on the next cycle.
        INC     HL
        LD      (HL),$00                                ; The initial player movement flags (facing left, rocket pack inactive, flying direction to the left).
        INC     HL
        LD      (HL),$03                                ; The initial player left facing frame number.
        INC     HL
        LD      (HL),$01                                ; Program flags (player is on a platform).

        NOP                                             ; [Redundant]
        NOP     
        NOP     
        NOP     

        JP      L68CF                                   ; Jump ahead to handle movement of the player.

; -----------------
; Program Variables
; -----------------

L688C:  DEFB $01                                        ; Holds the fuel usage timer, which controls the rate at which the fuel is used ($01 to $0A).
L688D:  DEFB $00                                        ; Holds the amount of fuel remaining ($00 to $0B).

; -----------------------
; Display Lives Remaining
; -----------------------
; This routine displays the number of lives remaining excluding the 'active' life,
; i.e. if 5 lives remain then 4 are shown below the game area.
; Entry: C=Number of lives remaining.

L688E:  LD      HL,L57D7+$0001                          ; Line 181 column 1 in the hi-res display file.
        DEC     C                                       ; Decrement the life counter.
        JR      Z,L68A2                                 ; Jump ahead if there are no lives left.

; There is at least 1 life remaining.

L6894:  PUSH    HL                                      ; Save the display file address.

        LD      DE,L620B
        LD      B,$0B                                   ; There are 11 lines.
        CALL    L64DA                                   ; Copy a block into the display file.

        POP     HL                                      ; Retrieve the display file address.
        INC     HL                                      ; advance to the next column in the display file.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6894                                ; Loop back for all columns.

; Blank out the life just lost.

L68A2:  LD      B,$0B                                   ; There are 11 lines to blank.
        LD      DE,$0021                                ; Each line is 33 bytes wide.

L68A7:  LD      (HL),$0B                                ; Insert the hi-res code for a 'white' character.
        ADD     HL,DE                                   ; Advance to the next line.
        DJNZ    L68A7                                   ; Repeat for all lines.

        RET     

; ------------------
; Display Fuel Gauge
; ------------------
; Entry: C=Fuel level.

L68AD:  LD      HL,L57D7+$001D                          ; Line 181 column 29.
        LD      DE,$0021                                ; There are 33 bytes per line.
        LD      A,$0B                                   ; The maximum fuel level.
        SUB     C                                       ; Determine the amount of fuel gone.
        JR      Z,L68C1                                 ; Jump ahead if the fuel level is full.

; Draw the amount of fuel gone.

        LD      B,A                                     ; The amount of full gone.

L68B9:  LD      (HL),$0F                                ; Insert an 'empty' graphic (##....##).
        ADD     HL,DE                                   ; Advance to the next line in the display file.
        DJNZ    L68B9                                   ; Repeat for all lines.

        CP      $0B                                     ; Is there any fuel to be drawn?
        RET     Z                                       ; Return if not.

; Draw the amount of fuel remaining.

L68C1:  LD      B,C                                     ; Fetch the amount of fuel remaining.

L68C2:  LD      (HL),$19                                ; Insert a 'full' graphic (########).
        ADD     HL,DE                                   ; Advance to the next line in the display file.
        DJNZ    L68C2                                   ; Repeat for all lines.

        RET     

; -------------------
; End of BASIC Line 3
; -------------------

L68C8:  DEFB _NEWLINE

; ------------
; BASIC Line 4
; ------------

L68C9:  DEFB 4 >> 8
        DEFB 4 & $FF
        DEFW L6D7F - 1 - $
        DEFB _REM

; ------------------
; Keys Pressed Flags
; ------------------
; This byte is used to hold a set of flags indicating the combination of keys being pressed.
; A pressed key is denoted by a bit being set.
;
; Bit 76543210
;     000JRLDU
;
; where U=up, D=down, L=left, R=right, J=jump.

L68CE:  DEFB $00                                        ; Holds a set of flags indicating which keys are being pressed.

; -----------------------------
; Handle Movement of the Player
; -----------------------------
; A set of flags is generated based upon which of the 5 keys is being pressed. However, because the keyboard is read
; using the LAST_K system variable it is not always possible to resolve key presses uniquely. The following example
; (reported by Steve McDonald) demonstrates this. If A, I and P are selected for jump, left and right respectively
; then pressing I and P to fly left will also make the game think that key R is being pressed.
;
; Key(s)  LAST_K (hex)      LAST_K (binary)
; I     = $F7DF           = %1111 0111 1101 1111
; P     = $FDDF           = %1111 1101 1101 1111
; A     = $FDFD           = %1111 1101 1111 1101
;
; I + A = $F5DD AND $FDFD = %1111 0101 1101 1101
;                                 I A    I    A
;
; P     = $FDDF           = %1111 1101 1101 1111
; D     = $F7FD           = %1111 0111 1111 1101
;
; P + D = $FDDF AND $F7FD = %1111 0101 1101 1101
;                                 D P    P    D
;
; When a key is pressed, LAST_K reports it by resetting a bit in the high byte to indicate the keyboard column and
; resetting a bit in the low byte to indicate the keyboard half row. When multiple keys are pressed, LAST_K reports
; an OR of the separate key values but all context of which reset bits relate to which keys is lost. In the example
; above, it can be seen that the code for P overlaps with the high byte reset bit for A and the low byte reset bit for I;
; the other reset bits overlap with the key code for D. Hence pressing P and D is indistinguishable to pressing I and A.

L68CF:  XOR     A                                       ; Holds a set of flags to indicate which keys are being pressed.
        LD      C,$01                                   ; C holds the bit value for the key being tested. It cycles through $01 >> $02 >> $04 >> $08 >> $10.
        LD      B,$05                                   ; There are 5 keys to test (up, down, left, right and jump).
        LD      DE,L65C1                                ; Point to the store holding the code for the 'up' key.

L68D7:  PUSH    AF                                      ; Save the keys flags.

        LD      HL,LAST_K                               ; Point to the system variable holding the code of the latest keyboard read.
        LD      A,(DE)                                  ; Fetch the low byte of the key code.
        CPL                                             ; Invert so that the AND can be used to test for equality.
        AND     (HL)                                    ; Does the key code byte match?
        INC     DE                                      ; Point to the high byte of the key code.
        JR      NZ,L68E7                                ; Jump if not.

        INC     HL                                      ; Advance to the high byte of the LAST_K system variable.
        LD      A,(DE)                                  ; Fetch the high byte of the key code.
        CPL                                             ; Invert so that AND can be used to test for equality.
        AND     (HL)                                    ; Does the key code byte match?
        JR      Z,L68EA                                 ; Jump if so.

; The key code did not match.

L68E7:  POP     AF                                      ; Retrieve the key flags.
        JR      L68EC                                   ; Jump ahead to continue with the next key.

; The key code matched.

L68EA:  POP     AF                                      ; Retrieve the key flags.
        OR      C                                       ; Set the bit corresponding to the current key.

L68EC:  RLC     C                                       ; Determine the flag for the next key.

        INC     DE                                      ; Point at the low byte of the next key store.
        DJNZ    L68D7                                   ; Repeat for all 5 keys.

        LD      (L68CE),A                               ; Store the keys pressed flags.

; The set of flags indicating which keys have been pressed has been generated.

        LD      A,(L6409)                               ; Fetch the player flags.
        BIT     6,A                                     ; Flying via a rocket pack?
        JP      NZ,L6D85                                ; Jump if so.

; Not flying via a rocket pack.

        LD      HL,L6405                                ; Point at the store holding the player movement timer.
        LD      A,(FRAMES)
        CP      (HL)                                    ; Is it time to move the player?
        JP      P,L7323                                 ; Jump if not to handle movement of the Bubloid.

; It is time to move the player.

L6906:  SUB     $02                                     ; Determine the new value for the player movement timer.
        LD      (HL),A                                  ; Store the new player movement timer.

; Test whether the player has fallen into the sea.

        LD      A,(L6409)                               ; Fetch the player flags.
        BIT     7,A                                     ; Has the player fallen into the sea?
        JP      NZ,L784F                                ; Jump if so to display the splash graphic.

; The player has not fallen into the sea.

        BIT     4,A                                     ; Has the player been caught?
        JP      NZ,L75E3                                ; Jump if so to display the Player Caught graphic.

; The player has not been caught.

        BIT     5,A                                     ; Is the player flying via a vulture?
        JP      NZ,L70E3                                ; Jump if so.

; Player is not flying via a vulture.

        LD      A,(L68CE)                               ; Fetch the keys pressed flags.
        BIT     2,A                                     ; Was the left key pressed?
        JR      NZ,L6972                                ; Jump if so.

; The left key was not pressed.

        BIT     3,A                                     ; Was the right key pressed?
        JP      Z,L69BE                                 ; Jump if not.

; The right key was pressed
; -------------------------

        LD      HL,L6408                                ; Point to the store holding the player frame number.

        LD      A,(L6409)                               ; Fetch the player flags.
        BIT     3,A                                     ; Is the player on a ladder?
        JR      Z,L6933                                 ; Jump ahead if not.

        LD      (HL),$03                                ; Set the displayed frame number when facing right.

L6933:  AND     $F7                                     ; Signal the player is not on a ladder.
        LD      (L6409),A                               ; Store the updated flags.

        DEC     (HL)                                    ; Change to the next frame.
        JR      NZ,L696D                                ; Jump ahead if this is not the last right-facing frame.

        LD      (HL),$04                                ; Wrap back to the first right-facing frame.

; The player has moved to a new column so it is necessary to restore the background graphics at the player's old position and
; save the background graphics at the player's new position.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        INC     HL
        INC     HL                                      ; Point to the right most column of the player.

        LD      BC,$1001                                ; There are 16 lines and 1 columns to check.
        CALL    L6B7E                                   ; Check if the player can move (i.e. not off screen, not caught by Bubloid, not picked up fuel can/lag-of-lamb).
        CP      $01                                     ; Did the player move successfully?
        JP      Z,L6B7B                                 ; Jump if not.

; The player moved right successfully.

        PUSH    HL                                      ; Save the new right position of the player within the display file.
        DEC     HL                                      ; Point at the new left position of the player within the display file.
        LD      (L6401),HL                              ; Save as the address within the display file of the player.

; Restore the background where the left of the player used to be.

        DEC     HL                                      ; Point back to the original left position.
        LD      B,$10                                   ; There are 16 lines to restore into the display file.
        LD      DE,L5949                                ; Point to the buffer for the left column background.
        CALL    L64DA                                   ; Copy the block into the display file.

; Save the new background graphics.

        LD      BC,$0010                                ; There are 16 lines to copy.
        LD      DE,L5949                                ; Point to the buffer for the left column background.
        LD      HL,L5A39                                ; Point to the buffer for the right column background.
        LDIR                                            ; The background for the original right column becomes the background for the new left column.

        POP     HL                                      ; Retrieve the new right position of the player within the display file.
        LD      DE,L5A39                                ; Point to the buffer for the right column background.
        CALL    L6D50                                   ; Save the background at the new right position into the buffer.

; Joins here when the player has moved right but not into a new column.

L696D:  LD      A,$01                                   ; Signal that the player is facing right.
        JP      L69BB                                   ; Jump ahead to continue.

; The left key was pressed
; ------------------------

L6972:  LD      HL,L6408                                ; Point to the store holding the player frame number.

        LD      A,(L6409)                               ; Fetch the player flags.
        BIT     3,A                                     ; Is the player on a ladder?
        JR      Z,L697E                                 ; Jump ahead if not.

        LD      (HL),$02                                ; Set the displayed frame number when facing left.

L697E:  AND     $F7                                     ; Signal the player is not on a ladder.
        LD      (L6409),A                               ; Store the updated flags.

        INC     (HL)                                    ; Change to the next frame.
        LD      A,$05
        CP      (HL)                                    ; Has the last frame been exceeded?
        JR      NZ,L69BA                                ; Jump ahead if this is not the last left-facing frame.

        LD      (HL),$01                                ; Wrap back to the first left-facing frame.

; The player has moved to a new column so it is necessary to restore the background graphics at the player's old position and
; save the background graphics at the player's new position.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        DEC     HL                                      ; Point to the position to the left.

        LD      BC,$1001                                ; There are 16 lines and 1 columns to check.
        CALL    L6B7E                                   ; Check if the player can move (i.e. not off screen, not caught by Bubloid, not picked up fuel can/lag-of-lamb).
        CP      $01                                     ; Did the player move successfully?
        JP      Z,L6B7B                                 ; Jump if not.

; The player moved left successfully.

        LD      (L6401),HL                              ; Save as the new address within the display file of the player.

; Restore the background where the right of the player used to be.

        PUSH    HL                                      ; Save the address of the new left position within the display file.
        INC     HL
        INC     HL                                      ; Point to the original right position within the display file.
        LD      B,$10                                   ; There are 16 lines to restore into the display file.
        LD      DE,L5A39                                ; Point to the buffer for the right column background.
        CALL    L64DA                                   ; Copy the block into the display file.

; Save the new background graphics.

        LD      BC,$0010                                ; There are 16 lines to copy.
        LD      DE,L5A39                                ; Point to the buffer for the right column background.
        LD      HL,L5949                                ; Point to the buffer for the left column background.
        LDIR                                            ; The background for the original left column becomes the background for the new right column.

        POP     HL                                      ; Retrieve the address of the new left position within the display file.
        LD      DE,L5949                                ; Point to the buffer for the left column background.
        CALL    L6D50                                   ; Save the background at the new left position into the buffer.

; Joins here when the player has moved left but not into a new column.

L69BA:  XOR     A                                       ; Signal that the player is facing left.

; Joins here when moving right.

L69BB:  LD      (L6407),A                               ; Store the flag indicating whether the player is facing left ($00) or right ($01).

; The left or right keys have been processed.
; Joins here when the left and right keys were not being pressed.

L69BE:  LD      HL,L6403                                ; Fetch the player's vertical speed.
        XOR     A                                       ; A=$00, i.e. no speed.
        CP      (HL)                                    ; Is the player in the process of jumping?
        JR      Z,L69C9                                 ; Jump ahead if not.

; The player is in the process of jumping.

        DEC     (HL)                                    ; Decrement the player's vertical speed.
        JP      L6A68                                   ; Jump to move the player up.

; The player is not in the process of jumping.

L69C9:  LD      HL,L6409                                ; Point to the flags.
        LD      A,(HL)                                  ; Fetch the player flags.
        AND     $06                                     ; Is the left or right of the player on a ladder?
        JR      Z,L69DD                                 ; Jump if neither.

; The left or right of the player is on a ladder.

        LD      A,(L68CE)                               ; Fetch the keys pressed flags.
        BIT     0,A                                     ; Was the up key pressed?
        JP      NZ,L6A57                                ; Jump if so.

; The up key was not pressed.

        BIT     1,A                                     ; Was the down key pressed?
        JR      NZ,L69E5                                ; Jump if so.

; The player is on a ladder and neither the up or down keys were pressed.
; Joins here when the player is not on a ladder.

L69DD:  LD      A,(HL)                                  ; Fetch the player flags.
        AND     $09                                     ; Is the player on a ladder or platform?
        JP      NZ,L6ABC                                ; Jump if so to test the jump key.

        JR      L69E7                                   ; Jump ahead to continue.

; The player is on a ladder and the down key was pressed

L69E5:  SET     3,(HL)                                  ; Signal the player is on a ladder.

; Attempt to move the player down.
; Joins here when the player is not on a ladder and not on a platform, i.e. falling.

L69E7:  LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        PUSH    HL                                      ; Save the original address of the player.
        LD      DE,$0210                                ; The player is 16 lines tall.
        ADD     HL,DE                                   ; Point to below the player.
        LD      BC,$0202                                ; There are 2 lines and 2 columns to check.
        CALL    L6B7E                                   ; Check if the player can move (i.e. not off screen, not caught by Bubloid, not picked up fuel can/lag-of-lamb).
        CP      $01                                     ; Did the player move successfully?
        JR      NZ,L69FD                                ; Jump if so.

; The player did not move down successfully.

        POP     HL                                      ; Retrieve the original display file address of the player.
        JP      L6B7B                                   ; Jump ahead to continue.

; The player moved down successfully.

L69FD:  LD      DE,$01CE                                ; 14 lines.
        AND     A
        SBC     HL,DE                                   ; Point back up to 2 lines below the original position.
        LD      (L6401),HL                              ; Save as the new address within the display file of the player.

; Check whether the player has moved too low.

        LD      DE,L5564+$001F                          ; Line 162 column 31.
        AND     A
        SBC     HL,DE                                   ; Has the player fallen into the sea?
        JR      C,L6A16                                 ; Jump ahead if not.

; The player has fallen into the sea.

        LD      HL,L6408                                ; Point at the store holding the player frame number.
        LD      (HL),$04                                ; Set such that the initial splash graphic frame will be shown.
        INC     HL
        LD      (HL),$80                                ; Signal fallen into the sea, i.e. the splash graphic needs to be drawn

L6A16:  POP     HL                                      ; Retrieve the original display file address of the player.

; Restore the background where the top of the player used to be.

        LD      C,$02                                   ; There are 2 columns to restore.
        LD      DE,L5949                                ; Point to the buffer for the left column background of the player.

L6A1C:  PUSH    HL                                      ; Save the original display file location.
        LD      B,$02                                   ; There are 2 lines to restore in the current column.
        CALL    L64DA                                   ; Copy the block into the display file.
        POP     HL                                      ; Retrieve the original display file location.
        INC     HL                                      ; Advance to the right column.
        LD      DE,L5A39                                ; Point to the buffer for the right column background of the player.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6A1C                                ; Repeat the restoration for the right column.

; Shift the existing background graphics up 2 lines.

        LD      DE,L5949                                ; Point to the top line of the buffer for the left column background of the player.
        LD      HL,L5949+$0002                          ; Point 2 lines into the buffer.
        LD      BC,$000E                                ; There are 14 lines to shift 'up'.
        LDIR                                            ; Shift the background graphics up 2 lines in the buffer.

        LD      DE,L5A39                                ; Point to the top line of the buffer for the right column background of the player.
        LD      HL,L5A39+$0002                          ; Point 2 lines into the buffer.
        LD      BC,$000E                                ; There are 14 lines to shift 'up'.
        LDIR                                            ; Shift the background graphics up 2 lines in the buffer.

; Save the new background graphics where the player has moved to.

        LD      HL,(L6401)                              ; Fetch the new address within the display file of the player.
        LD      C,$02                                   ; There are 2 columns to store.
        LD      DE,$01CE                                ; Offset of 14 lines.
        ADD     HL,DE                                   ; Advance to the bottom 2 lines of the buffer.

        LD      DE,L5949+$000E                          ; Point to the bottom 2 lines of the buffer for the left column background of the player.

L6A4C:  CALL    L6D5E                                   ; Copy 2 lines of the display file at the column position into the background buffer.
        LD      DE,L5A39+$000E                          ; Point to the bottom 2 lines of the buffer for the right column background of the player.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6A4C                                ; Repeat to store the background for the right column.

        JR      L6ABC                                   ; Jump ahead to test the jump key.

; The player is on a ladder and the up key was pressed
; ----------------------------------------------------

L6A57:  LD      HL,L6409                                ; Point to the flags.
        LD      A,(HL)                                  ; Fetch the player flags.
        AND     $06                                     ; Is the left or right of the player on a ladder?
        JR      Z,L6A68                                 ; Jump if neither.

; The player is on a ladder.

        DEC     HL                                      ; Point to the store holding the player frame number.
        DEC     (HL)                                    ; Decrement frame number.
        JR      NZ,L6A65                                ; Jump if the frame number has not reached 0.

        LD      (HL),$04                                ; Wrap the frame number back to 4.

L6A65:  INC     HL                                      ; Point at the flags.
        SET     3,(HL)                                  ; Signal the player is on a ladder.

; Attempt to move the player up.
; Joins here when the player is in the process of jumping.

L6A68:  LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      DE,$0042                                ; Moving up by 2 lines.
        AND     A
        SBC     HL,DE                                   ; Point to 2 lines up.
        LD      BC,$0202                                ; There are 2 lines and 2 columns to check.
        CALL    L6B7E                                   ; Check if the player can move (i.e. not off screen, not caught by Bubloid, not picked up fuel can/lag-of-lamb).
        CP      $01                                     ; Did the player move successfully?
        JP      Z,L6B7B                                 ; Jump if not.

; The player moved up successfully.

        PUSH    HL                                      ; Save the new display file address of the player.
        LD      (L6401),HL                              ; Save as the new address within the display file of the player.

; Restore the background where the bottom of the player used to be.

        LD      DE,$0210                                ; The player is 16 lines tall.
        ADD     HL,DE                                   ; Point to below the new player's location.
        LD      C,$02                                   ; There are 2 columns to restore.
        LD      DE,L5949+$000E                          ; Point to the bottom 2 lines in the buffer for the left column background.

L6A89:  PUSH    HL                                      ; Save the display file location below the player.
        LD      B,$02                                   ; There are 2 lines to restore in the current column.
        CALL    L64DA                                   ; Copy the block into the display file.
        POP     HL                                      ; Retrieve the display file location below the player.
        INC     HL                                      ; Advance to the right column.
        LD      DE,L5A39+$000E                          ; Point to the bottom 2 lines in the buffer for the right column background.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6A89                                ; Repeat the restoration for the right column.

; Shift the existing background graphics down 2 lines.

        LD      DE,L5949+$000F                          ; Point to the bottom line of the buffer for the left column background.
        LD      HL,L5949+$000D                          ; Point 2 lines above the bottom line in the buffer.
        LD      BC,$000E                                ; There are 14 lines to shift 'down'.
        LDDR                                            ; Shift the background graphics down 2 lines in the buffer.

        LD      DE,L5A39+$000F                          ; Point to the bottom line of the buffer for the right column background.
        LD      HL,L5A39+$000D                          ; Point 2 lines above the bottom line in the buffer.
        LD      BC,$000E                                ; There are 14 lines to shift 'down'.
        LDDR                                            ; Shift the background graphics down 2 lines in the buffer.

; Save the new background graphics where the player has moved to.

        POP     HL                                      ; Retrieve the display file location below the player.
        LD      C,$02                                   ; There are 2 columns to store.
        LD      DE,L5949                                ; Point to the top line of the buffer for the left column background.

L6AB3:  CALL    L6D5E                                   ; Copy 2 lines of the display file at the column position into the background buffer.
        LD      DE,L5A39                                ; Point to the top line of the buffer for the right column background.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6AB3                                ; Repeat to store the background for the right column.

; Joins here after the player has moved up or down
; ------------------------------------------------

L6ABC:  LD      A,(L68CE)                               ; Fetch the keys pressed flags.
        BIT     4,A                                     ; Was the jump key pressed?
        JR      Z,L6AD3                                 ; Jump if not.

; The jump key was pressed
; ------------------------

        LD      HL,L6403                                ; Point to the store holding the player's vertical speed.
        XOR     A                                       ; A=$00, i.e. no speed.
        CP      (HL)                                    ; Is the player is the process jumping?
        JR      NZ,L6AD3                                ; Jump ahead if so.

; The player is not currently jumping.

        LD      A,(L6409)                               ; Fetch the player flags.
        AND     $09                                     ; Is the player on a ladder or platform?
        JR      Z,L6AD3                                 ; Jump if not.

        LD      (HL),$08                                ; Reset the number of vertical offsets to perform at the next jump.

; Determine the player graphic to draw
; ------------------------------------
; Joins here when the player is already in the process of jumping or the jump key was not pressed.

L6AD3:  LD      C,$02                                   ; The player spans 2 columns.

        LD      HL,L6409                                ; Point to the flags.
        LD      A,(HL)                                  ; [Redundant]

; Check whether the player is on a ladder.

        LD      DE,L5949                                ; Point to the buffer for the left column background of the player.

; Enter a loop to check both columns covered by the player.

L6ADC:  LD      B,$10                                   ; There are 16 lines.

L6ADE:  LD      A,(DE)                                  ; Fetch a background character.
        CP      $8F                                     ; Is it a ladder? (..####..).
        JR      Z,L6AED                                 ; Jump if so.

        INC     DE                                      ; Point to the next background character.
        DJNZ    L6ADE                                   ; Repeat for all background buffer characters.

; The player is not on a ladder.

        LD      A,C                                     ; A=$02 if testing the left of the player, $01 if testing the right of the player.
        SLA     A                                       ; A=$04 for left of player, $02 for right of player.
        CPL                                             ; A=$FB for left of player, $FD for right of player.
        AND     (HL)                                    ; Clear bit 2 if the left of the player is not on a ladder and clear bit 1 if the right of the player is not on a ladder.
        JR      L6AF1                                   ; Jump ahead to continue.

; The player is on a ladder.

L6AED:  LD      A,C                                     ; A=$02 if testing the left of the player, $01 if testing the right of the player.
        SLA     A                                       ; A=$04 for left of player, $02 for right of player.
        OR      (HL)                                    ; Set bit 2 if the left of the player is on a ladder and set bit 1 if the right of the player is on a ladder.

L6AF1:  LD      (HL),A
        DEC     C                                       ; Decrement the column count.
        LD      DE,L5A39                                ; Point to the buffer for the right column background.
        JR      NZ,L6ADC                                ; Repeat for the right column.

; Was the player on a ladder?

        AND     $06                                     ; Is either the left or right of the player on a ladder?
        JR      NZ,L6B05                                ; Jump if so.

; The player is not on a ladder.

        LD      A,(L68CE)                               ; Fetch the keys pressed flags. [Redundant]
        BIT     0,A                                     ; Was the up key pressed? [Redundant]

        NOP                                             ; [Redundant]
        NOP

        RES     3,(HL)                                  ; Signal the player is not on a ladder.

; Joins here when the player is on a ladder.

L6B05:  LD      A,(HL)                                  ; Fetch the player flags.
        BIT     3,A                                     ; Is the player on a ladder?
        JR      Z,L6B37                                 ; Jump ahead if not.

; The player is on a ladder.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        BIT     2,A                                     ; Is the left of the player on the ladder?
        JR      NZ,L6B19                                ; Jump if so.

; The right of the player is on the ladder.

        INC     HL                                      ; Point to the right hand column of the player.
        PUSH    HL                                      ; Save the address of the location on the ladder.
        DEC     HL                                      ; Point back to the left hand column of the player.
        LD      DE,L5949                                ; The address of the first column of the single column right facing graphic.
        JR      L6B1E                                   ; Jump ahead to continue.

; The left of the player is on the ladder.

L6B19:  PUSH    HL                                      ; Save the address of the location on the ladder.
        INC     HL                                      ; Point to the right hand column occupied by the player.
        LD      DE,L5A39                                ; The address of the second column of the single column left facing graphic.

L6B1E:  LD      B,$10                                   ; There are 16 lines.
        CALL    L64DA                                   ; Copy the graphic into the display file.

; Draw the player climbing a ladder.

        LD      A,(L6408)                               ; Fetch the player frame.
        BIT     0,A                                     ; Is the left arm/leg up?
        JR      NZ,L6B2F                                ; Jump if not.

        LD      DE,L5A49                                ; Point to the player climbing left arm/leg up.
        JR      L6B32                                   ; Jump ahead to continue.

L6B2F:  LD      DE,L5A59                                ; Point to the player climbing right arm/leg up.

L6B32:  LD      C,$01                                   ; The graphic is 1 column wide.
        POP     HL                                      ; Retrieve the display file address.
        JR      L6B70                                   ; Jump ahead to draw the player in the display file.

; The player is not on a ladder.

L6B37:  LD      A,(L6407)                               ; Fetch the movement flags.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      BC,$0231                                ; Offset of 17 lines, i.e. below the player.

        BIT     0,A                                     ; Is the player facing left?
        JR      Z,L6B4A                                 ; Jump ahead if so.

; The player is facing right.

        LD      DE,L5949-$0020                          ; Base address for the player facing right graphics.
        INC     HL                                      ; Point to the right column of the player.
        JR      L6B4D                                   ; Jump ahead to continue.

; The player is facing left.

L6B4A:  LD      DE,L59A9                                ; Base address for the player facing left graphics.

; Joins here to check whether the player is on a platform.

L6B4D:  ADD     HL,BC                                   ; Point below the player.
        LD      A,(HL)                                  ; Fetch the character below the player.
        CP      $0B                                     ; On the player on a platform (........)?
        LD      A,(L6409)                               ; Fetch the player flags.
        JR      Z,L6B5A                                 ; Jump if so.

; The player is not on a platform.

        RES     0,A                                     ; Signal not on a platform.
        JR      L6B5C                                   ; Jump ahead to continue.

; The player is on a platform.

L6B5A:  SET     0,A                                     ; Signal on a platform.

L6B5C:  LD      (L6409),A                               ; Store the new flags.

; Locate the graphic for the player to display.

        LD      HL,L6408                                ; Point to the store holding the player frame number.
        LD      B,(HL)                                  ; Fetch the player frame number.
        EX      DE,HL                                   ; HL=Base address of the graphics for the facing direction.
        LD      DE,$0020                                ; The size of each graphic.

L6B67:  ADD     HL,DE                                   ; Advance to the next graphic.
        DJNZ    L6B67                                   ; Repeat until the required graphic has been reached.

        EX      DE,HL                                   ; DE=Address of the player graphic to display.

; Prepare to draw the player in the display file.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      C,$02                                   ; There are 2 columns.

; Draw the player into the display file
; -------------------------------------
; Joins here when drawing the player climbing or descending a ladder.

L6B70:  PUSH    HL                                      ; Save the display file address.
        LD      B,$10                                   ; There are 16 lines.
        CALL    L64DA                                   ; Copy a column of the graphic into the display file.
        POP     HL                                      ; Retrieve the display file address.

        INC     HL                                      ; Advance to the next column within the display file.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6B70                                ; Repeat for all columns.

L6B7B:  JP      L77F8                                   ; Jump ahead to draw the fuel can / leg of lamb.

; ------------------------------------------
; Check Whether Player Can Move Successfully
; ------------------------------------------
; This routine checks whether a move would cause the player to go off screen, or to be caught by Bubloid, or to pick up a fuel can/leg of lamb.
; Entry: HL=New location within display file.
;        B=Number of lines.
;        C=Number of columns.
; Exit : A=$01 if off screen / picked up fuel can/leg of lamb / caught by Bubloid, else $00.

L6B7E:  LD      A,(HL)                                  ; Fetch the contents of the new position of the player.
        CP      $C9                                     ; Is the position off the screen, i.e. the end of line terminator has been reached?
        JP      Z,L6CA5                                 ; Jump if so.

; The new position is not off the screen.

        PUSH    HL                                      ; Save the new location within the display file of the player.

        CALL    L6CA8                                   ; Check whether the player has been caught by the Bubloid.
        JR      NC,L6B90                                ; Jump ahead if the player has not been caught.

; The player has been caught by the Bubloid.

        CALL    L6CFD                                   ; Configure caught by Bubloid.
        JP      L6CA4                                   ; Jump to make a return signalling the player has been caught.

; The player has not been caught by the Bubloid.

L6B90:  LD      A,C                                     ; Fetch the width of the player graphic, i.e. number of columns to test.

; Enter a loop here to test each column occupied by the player to see whether a fuel can / leg of lamb has been collected.

L6B91:  PUSH    HL                                      ; Save the address of the player within the display file.
        PUSH    AF                                      ; Save the column count.
        PUSH    BC                                      ; Save the original width and height of the player.

; Enter a loop here to test every line of the player within the current column to check whether the player has
; collided with the fuel can / leg of lamb.

L6B94:  EX      DE,HL                                   ; DE=Address of the player within the display file.

        LD      HL,(L64BF)                              ; Fetch the location of the fuel can / leg of lamb.
        PUSH    HL                                      ; Save the location of the fuel can / leg of lamb within the display file.
        PUSH    DE                                      ; Save the address of the player within the display file.

        LD      DE,$00E7                                ; Offset of 7 lines.
        ADD     HL,DE

        POP     DE                                      ; Retrieve the address of the player within the display file.
        AND     A
        SBC     HL,DE                                   ; Is the player's waist at the same position as the fuel can / leg of lamb (occurs when the player is standing on the platform)?
        POP     HL                                      ; Retrieve the location of the fuel can / leg of lamb within the display file.
        JR      Z,L6BAB                                 ; Jump if so

        AND     A
        SBC     HL,DE                                   ; Is the player's head at the same position as the fuel can / leg of lamb?
        JP      NZ,L6C2B                                ; Jump if not.

; The player is at the correct position to collect the fuel can / leg of lamb so erase it from the display file.

; *BUG* - If the Bubloid is partially overlapping the fuel can / leg of lamb, e.g. its top, as the player collects it from the other end, e.g. bottom,
; then an artefact of the fuel can / leg of lamb will be left on the screen. This occurs because the background buffer for the Bubloid contains the
; graphics for the fuel can / leg of lamb and will be restored when the Bubloid next moves. To overcome this, the background buffer of the Bubloid
; should be restored on the screen, the fuel can / leg of lamb erased, and then the Bubloid redrawn.

L6BAB:  LD      HL,(L64BF)                              ; Fetch the location of the fuel can / leg of lamb.
        LD      DE,$0021                                ; There are 33 bytes per line.
        LD      B,$08                                   ; There are 8 lines to erase.

L6BB3:  LD      (HL),$19                                ; Erase a line of the fuel can / leg of lamb (########).
        ADD     HL,DE                                   ; Advance to the next line within the display file.
        DJNZ    L6BB3                                   ; Repeat for all lines.

; Increment the score by 100.

        LD      A,$01                                   ; The increment to apply to the selected score digit.
        LD      B,$05                                   ; There are potentially 5 digits that need updating.
        LD      HL,L64B1-$0002                          ; Point to the 100s digit of the score.
        CALL    L6D6F                                   ; Add to the score.

; Copy the score into the hi-res display file.

        LD      HL,L40C4+$0007                          ; Line 2 column 7.
        LD      BC,L64B1                                ; Point to the last digit of the score.
        LD      A,$07                                   ; There are 7 digits in the score.
        CALL    L6515                                   ; Display the score value in the display file.

; Select a random location for the fuel can / leg of lamb.

        CALL    L6635                                   ; Select a random location for fuel can / leg of lamb.
        LD      (L64BF),HL                              ; Save as the location of the fuel can / leg of lamb.

; Draw the fuel can or leg of lamb depending on the level.

        LD      A,(L6425)                               ; Fetch the level number.
        BIT     2,A                                     ; Level 1, 2 or 3?
        JR      Z,L6BE5                                 ; Jump ahead if so.

; Level 4, 5 or 6.

        LD      DE,$0042                                ; The leg of lamb is lines shorter than the full can.
        ADD     HL,DE                                   ; Advance 2 lines lower in the display file.

        LD      B,$06                                   ; There are 6 lines to copy.
        LD      DE,L6296                                ; Point to the graphic for a leg of lamb.
        JR      L6BEA                                   ; Jump ahead to continue.

; Level 1, 2 or 3.

L6BE5:  LD      B,$08                                   ; There are 8 lines to copy.
        LD      DE,L628E                                ; Point to the graphic for a fuel can.

L6BEA:  CALL    L64DA                                   ; Copy the graphic into the display file.

; Update the fuel gauge.

        LD      HL,L688D                                ; Point to the location holding the fuel level.
        LD      A,$0B                                   ; The maximum fuel level.
        CP      (HL)                                    ; Is the fuel gauge fuel?
        JR      Z,L6BFA                                 ; Jump ahead if so.

; The fuel gauge is not full.

        INC     (HL)                                    ; Increment the fuel level.

        LD      C,(HL)                                  ; Fetch the amount of fuel remaining.
        CALL    L68AD                                   ; Draw the fuel gauge.

; Draw the rocket pack or vulture depending upon the level.

L6BFA:  LD      A,(L6425)                               ; Fetch the level number.
        LD      HL,L443F+$000F                          ; Line 29 column 15.
        BIT     2,A                                     ; Level 1, 2 or 3?
        JR      Z,L6C0C                                 ; Jump ahead if so.

; Level 4, 5 or 6.

        DEC     HL                                      ; The vulture appears at line 29 column 14.

        LD      DE,L5D89                                ; Point to the graphic for the vulture.
        LD      C,$02                                   ; The vulture is 2 columns wide.
        JR      L6C11                                   ; Jump ahead to draw the vulture.

; Level 1, 2 or 3.

L6C0C:  LD      DE,L621E                                ; Point to the graphic for the rocket pack.
        LD      C,$01                                   ; The rocket pack is 1 column wide.

L6C11:  LD      B,$10                                   ; The vulture / rocket pack is 16 lines tall.
        PUSH    HL                                      ; Save the location within the display file.
        CALL    L64DA                                   ; Copy a block into the display file.
        POP     HL                                      ; Retrieve the location within the display file.

        INC     HL                                      ; Advance to the next column within the display file.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6C11                                ; Loop back to draw all columns.

        NOP                                             ; [Redundant]  
        NOP     
        NOP     
        NOP     
        NOP     
        NOP     
        NOP     
        NOP     
        NOP     
        NOP     

        POP     HL                                      ; Discard the original width and height of the player.
        POP     HL                                      ; Discard the column count.
        POP     HL                                      ; Discard the address of the player within the display file.
        JR      L6CA4                                   ; Jump ahead to make a return.

; The column of the player just tested did not collide with the fuel can / leg of lamb.

L6C2B:  LD      A,(L688D)                               ; Fetch the fuel level.
        CP      $00                                     ; Out of fuel?
        JR      Z,L6C91                                 ; Jump if so.

; The player has some fuel.

        LD      HL,L443F+$000F                          ; Line 29 column 15.
        AND     A
        SBC     HL,DE                                   ; At the location of the rocket pack or vulture?
        JR      NZ,L6C91                                ; Jump if not.

; The player is at the location of the rocket pack or vulture so switch to flying mode.

; Remove the current rocket pack / vulture graphic from the display file.

        LD      HL,L443F+$000F                          ; Line 29 column 15.
        LD      C,$02                                   ; There are 2 columns.
        LD      DE,$0021                                ; The length of a line in the display file.

L6C42:  PUSH    HL                                      ; Save the display file position.
        LD      B,$10                                   ; There are 16 lines.

L6C45:  LD      (HL),$19                                ; Erase a line of the rocket pack / vulture (#######).
        ADD     HL,DE                                   ; Advance to the next line.
        DJNZ    L6C45                                   ; Repeat for all lines in this column.

        POP     HL                                      ; Retrieve the display file position.
        DEC     HL                                      ; Point to the column to the left.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6C42                                ; Repeat for both columns.

; Clear all background buffers.

        LD      DE,L5AF5                                ; Right hand column of the background buffer for the rocket pack.
        PUSH    DE                                      ; Stack the buffer address.
        LD      DE,L5B09                                ; Left hand column of the background buffer for the rocket pack.
        PUSH    DE                                      ; Stack the buffer address.
        LD      DE,L5E79                                ; Left hand column of the background buffer for the flying vulture.
        PUSH    DE                                      ; Stack the buffer address.
        LD      DE,L5E89                                ; Middle column of the background buffer for the flying vulture.
        PUSH    DE                                      ; Stack the buffer address.
        LD      DE,L5E99                                ; Right hand column of the background buffer for the flying vulture.
        PUSH    DE                                      ; Stack the buffer address.

        LD      C,$05                                   ; There are 5 background buffers to clear.

L6C65:  LD      B,$10                                   ; There are 16 lines to clear.
        POP     HL                                      ; Retrieve the next stacked background buffer address.

L6C68:  LD      (HL),$19                                ; Erase a line of the background buffer (########).
        INC     HL                                      ; Advance to the next location within the buffer.
        DJNZ    L6C68                                   ; Repeat for all buffer content locations.

        DEC     C                                       ; Decrement the buffer count.
        JR      NZ,L6C65                                ; Repeat for all buffers.

; Determine the flying mode: rocket pack or vulture.

        LD      A,(L6425)                               ; Fetch the level number.
        BIT     2,A                                     ; Level 1, 2 or 3?
        JR      Z,L6C7B                                 ; Jump ahead if so.

; Level 4, 5 or 6.

        LD      A,$20                                   ; Signal flying is via a vulture.
        JR      L6C7D                                   ; Jump ahead to continue.

; Level 1, 2 or 3.

L6C7B:  LD      A,$40                                   ; Signal flying is via a rocket pack.

L6C7D:  LD      (L6409),A                               ; Store the new flags.

; Initialise the player's flying movement program variables.

        LD      HL,L6403
        LD      (HL),$02                                ; Set the player's initial vertical flying speed.
        INC     HL
        LD      (HL),$02                                ; Set the player's initial horizontal flying speed.
        INC     HL
        LD      A,(HL)                                  ; Fetch the player's vertical movement timer.
        INC     HL
        LD      (HL),A                                  ; Set the player's horizontal movement timer so be the same as the vertical movement timer.

        POP     HL                                      ; Discard the original width and height of the player.
        POP     HL                                      ; Discard the column count.
        POP     HL                                      ; Discard the address of the player within the display file.
        JR      L6CA4                                   ; Jump ahead to make a return.

; Out of fuel or got fuel but not at the location of the rocket pack / vulture.
; Repeat the check for each line and column of the player.

L6C91:  LD      HL,$0021                                ; The length of a line in the display file.
        ADD     HL,DE                                   ; Point to the next line.
        DEC     B                                       ; Decrement the line count.
        JP      NZ,L6B94                                ; Repeat for all lines of the player.

; Every line of the player in the current column has been tested for an overlap with the fuel can / leg of lamb
; but none was found.

        POP     BC                                      ; Retrieve the original width and height of the player.
        POP     AF                                      ; Retrieve the column count.
        POP     HL                                      ; Retrieve the address of the player within the display file.

        INC     HL                                      ; Advance to the next column occupied by the player.
        DEC     A                                       ; Decrement the column count.
        JP      NZ,L6B91                                ; Jump back to test the next column of the player.

; Every position occupied by the player has been tested for a collision with the fuel can / leg of lamb but none was found.

        POP     HL                                      ; Retrieve the new location within the display file of the player
        XOR     A                                       ; Signal not off screen / not picked up fuel can/leg of lamb / not caught by Bubloid.
        RET

L6CA4:  POP     HL                                      ; Retrieve the location within the display file.

; Joins here when the new position is off the screen.

L6CA5:  LD      A,$01                                   ; Signal off screen / picked up fuel can/leg of lamb / caught by Bubloid.
        RET     

; ---------------------------------
; Check if Player Caught by Bubloid
; ---------------------------------
; This routine tests for a collision between every display file position occupied by the player against that
; of the Bubloid. Each position comprising the player is checked against the four corners of the Bubloid.
; Entry: HL=New location within display file.
;        B=Number of lines.
;        C=Number of columns.
; Exit : Carry flag set if the player has been caught.

L6CA8:  PUSH    HL                                      ; Save the location within the display file.
        PUSH    BC                                      ; Save the line and column counts.

; Enter a loop to test all columns occupied by the player to see whether a collision with the Bubloid has occurred.

L6CAA:  PUSH    HL                                      ; Save the location within the display file.
        PUSH    BC                                      ; Save the column counts.

; Enter a loop to test all positions within a column occupied by the player for a collision.

L6CAC:  PUSH    BC                                      ; Save the line count.
        EX      DE,HL                                   ; DE=Location within the display file.

        LD      HL,(L6411)                              ; Fetch the location of the Bubloid within the display file.
        PUSH    HL                                      ; Save the top-left address of the Bubloid.
        INC     HL
        PUSH    HL                                      ; Save the top-right address of the Bubloid.
        LD      BC,$01EE                                ; Offset of 14 lines 32 columns
        ADD     HL,BC
        PUSH    HL                                      ; Save the bottom-left address of the Bubloid.
        INC     HL
        PUSH    HL                                      ; Save the bottom-right address of the Bubloid.

        LD      B,$04                                   ; There are 4 corners of the Bubloid to test against.

L6CBD:  POP     HL                                      ; Retrieve a corner address of the Bubloid.
        AND     A
        SBC     HL,DE                                   ; Has a collision occurred?
        JR      Z,L6CD6                                 ; Jump if so.

        DJNZ    L6CBD                                   ; Repeat for the other corner addresses.

; A collision has not occurred.

        POP     BC                                      ; Retrieve the line count.
        LD      HL,$0021
        ADD     HL,DE                                   ; Advance to the next line in the display file.
        DJNZ    L6CAC                                   ; Repeat the collision test for all lines within the current column.

        POP     BC                                      ; Retrieve the column count.
        POP     HL                                      ; Retrieve the original location within the display file.

        INC     HL
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6CAA                                ; Repeat the collision test for all columns.

        AND     A                                       ; Clear the carry flag to signal a collision has not occurred.

        POP     BC                                      ; Retrieve the original line and column counts.                 
        POP     HL                                      ; Retrieve the original location within the display file.
        RET     

; A collision has occurred.

L6CD6:  DEC     B                                       ; Was this the last corner?
        JR      Z,L6CDC                                 ; Jump ahead if so.

L6CD9:  POP     HL                                      ; Discard the stack corner address.
        DJNZ    L6CD9                                   ; Repeat for all other stacked corner addresses.

L6CDC:  POP     BC                                      ; Discard the line count.
        POP     BC                                      ; Discard the column counts.
        POP     HL                                      ; Discard the location within the display file.

        POP     BC                                      ; Retrieve the original line and column counts.
        POP     HL                                      ; Retrieve the original location within the display file.

        SCF                                             ; Signal a collision has occurred.
        RET     

; --------
; Not Used
; --------

L6CE3:  RST     00H                                     ; [Redundant]

        AND     A
        JR      L6CF4

        DEC     B
        JR      Z,L6CED

L6CEA:  POP     HL
        DJNZ    L6CEA

L6CED:  POP     HL
        POP     HL
        POP     HL
        POP     BC
        POP     HL
        POP     AF

        SCF     

L6CF4:  LD      A,$11
        LD      (L6CAC+$0008),A
        RET     

L6CFA:  POP     BC
        SCF     
        RET     

; -------------------------------
; Configure Caught by the Bubloid
; -------------------------------

; Restore the background graphics where the Bubloid is.

L6CFD:  LD      DE,L6029                                ; Point to the background buffer for the Bubloid.
        LD      HL,(L6411)                              ; Fetch the address within the display file of the Bubloid.
        LD      C,$02                                   ; The width of the Bubloid.

L6D05:  LD      B,$10                                   ; There are 16 lines to restore.
        PUSH    HL                                      ; Save the display file address.
        CALL    L64DA                                   ; Copy the column of the background buffer into the display file.
        POP     HL                                      ; Retrieve the display file address.

        INC     HL                                      ; Advance to the next column in the display file.
        DEC     C                                       ; Decrement the width count.
        JR      NZ,L6D05                                ; Loop back to draw all lines.

; Fill the Bubloid background graphics buffer with black.

        LD      HL,L6029                                ; Point to the background buffer for the Bubloid.
        LD      B,$20                                   ; There are 2 columns of 16 lines.

L6D15:  LD      (HL),$19                                ; Insert the graphic for black (########).
        INC     HL                                      ; Advance to the next buffer location.
        DJNZ    L6D15                                   ; Repeat for all buffer locations.

; Update the flags.

        LD      A,$10                                   ; Signal that the player has been caught.
        LD      (L6409),A                               ; Save the new flags setting.

; Copy the background graphics for the player as the background graphics for the Caught by the Bubloid.

        LD      HL,L5949                                ; Point to the buffer for the left column background of the player.
        LD      DE,L6089                                ; Point to the left column for the player caught graphic.
        LD      BC,$0010                                ; There are 16 lines to copy.
        LDIR                                            ; Copy the graphic.    

        LD      HL,L5A39                                ; Point to the buffer for the right column background of the player.
        LD      DE,L6119                                ; Point to the right column for the player caught graphic.
        LD      BC,$0010                                ; There are 16 lines to copy.
        LDIR                                            ; Copy the graphic.

; Reset Bubloid movement variables.

        LD      HL,L641E
        LD      (HL),$03                                ; Bubloid movement flags - the Bubloid is moving up and to the right.
        INC     HL
        LD      (HL),$04                                ; Bubloid vertical acceleration = slowest acceleration.
        INC     HL
        LD      (HL),$0A                                ; Bubloid vertical speed = 2/3 maximum speed.
        INC     HL
        LD      (HL),$FF                                ; Bubloid vertical movement timer to expire on the next cycle.
        INC     HL
        LD      (HL),$04                                ; Bubloid horizontal acceleration = slowest acceleration.
        INC     HL
        LD      (HL),$0A                                ; Bubloid horizontal speed = 2/3 maximum speed.
        INC     HL
        LD      (HL),$FF                                ; Bubloid horizontal movement timer to expire on the next cycle.
        RET     

; --------
; Not Used
; --------

L6D4D:  LDIR                                            ; [Redundant]
        RET     

; ----------------------------------------------------
; Copy Column from Display File into Background Buffer
; ----------------------------------------------------
; This routine is used when moving left or right.
; Entry: HL=Address within display file.
;        DE=Address within background buffer.

L6D50:  LD      B,$10                                   ; There are 16 lines to copy.

L6D52:  LD      A,(HL)                                  ; Fetch a character code from the display file.
        LD      (DE),A                                  ; Write it into the buffer.
        PUSH    DE                                      ; Save the buffer address.
        LD      DE,$0021                                ; There are 33 bytes per line.
        ADD     HL,DE                                   ; Advance to the next line.
        POP     DE                                      ; Retrieve the buffer address.
        INC     DE                                      ; Advance to the next buffer location.
        DJNZ    L6D52                                   ; Repeat for all lines.

        RET     

; ---------------------------------------------------
; Copy Lines from Display File into Background Buffer
; ---------------------------------------------------
; This routine is used when moving up or down. It copies 2 lines from the display file into the background buffer.
; Entry: HL=Address within display file.
;        DE=Address within background buffer.
; Exit : HL=Address of column to the right within display file.

L6D5E:  PUSH    HL                                      ; Save the display file location.
        LD      B,$02                                   ; There are 2 lines to copy.

L6D61:  LD      A,(HL)                                  ; Fetch a character code from the display file.
        LD      (DE),A                                  ; Write it into the buffer.
        PUSH    DE                                      ; Save the buffer address.
        LD      DE,$0021                                ; There are 33 bytes per line.
        ADD     HL,DE                                   ; Advance to the next line.
        POP     DE                                      ; Retrieve the buffer address.
        INC     DE                                      ; Advance to the next buffer location.
        DJNZ    L6D61                                   ; Repeat for all lines.

        POP     HL                                      ; Retrieve the display file location.
        INC     HL                                      ; Advance to the column to the right.
        RET     

; ---------------
; Increment Score
; ---------------
; Entry: HL=Address of the last digit.
;        A=Increment.

L6D6F:  ADD     A,(HL)                                  ; Add the increment to the least significant digit.

L6D70:  CP      $0B                                     ; Has the new digit value overflowed?
        JR      C,L6D7D                                 ; Jump ahead if not.

; The new digit value has overflowed.

        SUB     $0A                                     ; Determine the overflow amount.
        LD      (HL),A                                  ; Store this as the new value for the digit.

        DEC     HL                                      ; Point to the next significant digit.
        LD      A,(HL)                                  ; Fetch the current value.
        INC     A                                       ; Increment it.
        DJNZ    L6D70                                   ; Loop back to test whether this digit has overflowed.

        RET     

; The new digit value has not overflowed.

L6D7D:  LD      (HL),A                                  ; Store the new value for the digit.
        RET     

; -------------------
; End of BASIC Line 4
; -------------------

L6D7F:  DEFB _NEWLINE

; ------------
; BASIC Line 5
; ------------

L6D80:  DEFB 5 >> 8
        DEFB 5 & $FF
        DEFW L731D - 1 - $
        DEFB _REM

; --------------------------------------
; The Player is Flying via a Rocket Pack
; --------------------------------------

L6D85:  LD      HL,L6405                                ; Point at the store holding the player vertical movement timer.
        LD      A,(FRAMES)
        CP      (HL)                                    ; Is it time to move the player vertically?
        JP      P,L6EB1                                 ; Jump if not to check the player horizontal movement timer.

; It is time to move the player vertically.

        DEC     HL                                      ; Point at the store holding the player's vertical speed.
        DEC     HL
        SUB     (HL)                                    ; Determine the FRAMES number of the next vertical movement.
        INC     HL
        INC     HL
        LD      (HL),A                                  ; Store the new vertical movement timer.

; Is the jump key being pressed?

        LD      HL,L6407                                ; Point at the movement flags.
        LD      A,(L68CE)                               ; Fetch the keys pressed flags.
        BIT     1,(HL)                                  ; Is the rocket pack active?
        JR      NZ,L6DA5                                ; Jump ahead if so.

; The rocket pack is inactive.

        BIT     4,A                                     ; Was the jump key pressed?
        JR      Z,L6DAD                                 ; Jump if not.

        JR      L6DB1                                   ; Jump ahead to continue.

; The rocket pack is active.

L6DA5:  BIT     4,A                                     ; Was the jump key pressed?
        JR      Z,L6DB5                                 ; Jump if not.

; The rocket pack is active. The jump key was pressed.

        LD      C,$01                                   ; Low climbing acceleration.
        JR      L6DCA                                   ; Jump ahead to continue.

; The rocket pack is inactive. The jump key was not pressed.

L6DAD:  LD      C,$02                                   ; High falling acceleration.
        JR      L6DCA                                   ; Jump ahead to continue.

; The rocket pack is inactive. The jump key was pressed.

L6DB1:  LD      C,$01                                   ; Low climbing acceleration. 
        JR      L6DB7                                   ; Jump ahead to continue.

; The rocket pack is active. The jump key was not pressed.

L6DB5:  LD      C,$02                                   ; High climbing acceleration.

; Accelerate when:
; - The rocket pack is active and the jump key was not pressed.
; - The rocket pack is inactive and the jump key was pressed.

L6DB7:  LD      A,(L6403)                               ; Fetch the player's vertical speed.
        CP      $07                                     ; Is the player at the top vertical flying speed?
        JR      NC,L6DC4                                ; Jump ahead if so.

; The player has not reached top vertical flying speed.

        ADD     A,C                                     ; Increase the player's vertical speed ($01 if jumping and $02 if flying).
        LD      (L6403),A                               ; Store the update vertical speed.

        JR      L6DD9                                   ; Jump ahead to continue.

; The player has exceeded top vertical flying speed.

L6DC4:  LD      A,(HL)                                  ; Fetch the movement flags.
        XOR     $02                                     ; Switch the rocket pack off to reverse vertical flying direction to slow down the player.
        LD      (HL),A                                  ; Store the updated movement flags.

        JR      L6DD9                                   ; Jump ahead to continue.

; Deceleration when:
; - The rocket pack is inactive and the jump key was not pressed.
; - The rocket pack is active and the jump key was pressed.

L6DCA:  LD      HL,L6403                                ; Point at the store holdiing the player's vertical speed.
        LD      A,(HL)                                  ; Fetch the vertical speed.
        SUB     C                                       ; Decrease the player's vertical speed.
        CP      $02                                     ; Is the player above minimum vertical flying speed?
        JR      NC,L6DD5                                ; Jump ahead if not.

        LD      A,$02                                   ; Cap the at the minimum vertical speed.

L6DD5:  LD      (HL),A                                  ; Store the new vertical speed.

        NOP                                             ; [Redundant]
        NOP     
        NOP     

; Joins here after decreasing the player's vertical speed.

L6DD9:  LD      HL,L6407                                ; Point at the store holding the movement flags.
        BIT     1,(HL)                                  ; Is the rocket pack active?
        JR      NZ,L6E51                                ; Jump if so.

; The rocket pack is inactive so attempt to move the player down 2 lines
; ----------------------------------------------------------------------

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        PUSH    HL                                      ; Save the address within the display file of the player.

        LD      DE,$0210
        ADD     HL,DE                                   ; Advance 16 lines, i.e. below the player.

        LD      BC,$0202                                ; There are 2 lines and 2 columns to test.
        CALL    L700B                                   ; Can the player move? A return is not made if the move results in all diamonds being collected.

        NOP                                             ; [Redundant]
        NOP     

        JR      NC,L6DF6                                ; Jump ahead if the player can move to the new location.

; The player cannot move 2 lines down.

        POP     HL                                      ; Discard the address within the display file of the player.
        JP      L6FB2                                   ; Jump to draw the rocket pack in the display file.

; The player can move 2 lines down
; --------------------------------

L6DF6:  LD      DE,$01CE                                ; Offset of 14 lines.
        AND     A
        SBC     HL,DE                                   ; Point to 14 lines above the bottom of the player, i.e. 2 lines down from the original location.
        LD      (L6401),HL                              ; Save as the new address of the player within the display file.

; Determine whether the player has fallen into the sea.

        LD      DE,L5564+$001F                          ; Line 162 column 31.
        AND     A
        SBC     HL,DE                                   ; Has the player fallen into the sea?
        JR      C,L6E0F                                 ; Jump ahead if so.

; The player has fallen into the sea.

        LD      HL,L6408                                ; Point at the store holding the player frame number.
        LD      (HL),$04                                ; Set such that the initial splash graphic frame will be shown.
        INC     HL
        LD      (HL),$80                                ; Signal fallen into the sea, i.e. the splash graphic needs to be drawn

L6E0F:  POP     HL                                      ; Retrieve the display file address of the player.

; Restore the background where the top of the rocket pack used to be.

        LD      C,$02                                   ; There are 2 columns to restore.
        LD      DE,L5B09                                ; Point to the buffer for the left column background of the rocket pack.

L6E15:  PUSH    HL                                      ; Save the original display file location.
        LD      B,$02                                   ; There are 2 lines to restore in the current column.
        CALL    L64DA                                   ; Copy the block into the display file.
        POP     HL                                      ; Retrieve the original display file location.
        INC     HL                                      ; Advance to the right column.
        LD      DE,L5AF5                                ; Point to the buffer for the right column background of the rocket pack.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6E15                                ; Repeat the restoration for the right column.

; Shift the existing background graphics up 2 lines.

        LD      DE,L5AF5                                ; Point to the top line of the buffer for the right column background of the rocket pack.
        LD      HL,L5AF5+$0002                          ; Point 2 lines into the buffer.
        LD      BC,$000E                                ; There are 14 lines to shift 'up'.
        LDIR                                            ; Shift the background graphics up 2 lines in the buffer.

        LD      DE,L5B09                                ; Point to the top line of the buffer for the left column background of the rocket pack.
        LD      HL,L5B09+$0002                          ; Point 2 lines into the buffer.
        LD      BC,$000E                                ; There are 14 lines to shift 'up'.
        LDIR                                            ; Shift the background graphics up 2 lines in the buffer.

; Save the background graphics for the bottom 2 lines that the rocket pack has moved into.

        LD      HL,(L6401)                              ; Fetch the new address within the display file of the player.
        LD      DE,$01CE                                ; Offset of 14 lines.
        ADD     HL,DE                                   ; Advance to the bottom 2 lines of the buffer.
        LD      C,$02                                   ; There are 2 columns to store.
        LD      DE,L5B15+$0002                          ; Point to the bottom 2 lines of the base 1 buffer for the left column background of the rocket pack.

L6E45:  CALL    L6D5E                                   ; Copy 2 lines of the display file at the column position into the background buffer.
        LD      DE,L5B01+$0002                          ; Point to the bottom 2 lines of the base 1 buffer for the right column background of the rocket pack.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6E45                                ; Repeat to store the background for the right column.

        JP      L6FB2                                   ; Jump to draw the rocket pack in the display file.

; The rocket pack is active so attempt to move the player up 2 lines
; ------------------------------------------------------------------

L6E51:  LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      DE,$0042
        AND     A
        SBC     HL,DE                                   ; Point to 2 lines above the current player's location.
        LD      BC,$0202                                ; There are 2 lines and 2 columns to test.
        CALL    L700B                                   ; Can the player move? A return is not made if the move results in all diamonds being collected.

        NOP                                             ; [Redundant]
        NOP     

        JR      NC,L6E6E                                ; Jump if the player can move to the new location.

; The player cannot move up 2 lines.

        LD      HL,L6407                                ; Point at the store holding the movement flags.
        LD      A,(HL)                                  ; Fetch the movement flags.
        XOR     $02                                     ; Toggle the vertical direction bit to 'bounce' the player downwards.
        LD      (HL),A                                  ; Store the updated movement flags.

        JP      L6FB2                                   ; Jump to draw the rocket pack in the display file.

; The player can move up 2 lines
; ------------------------------

L6E6E:  PUSH    HL                                      ; Save the player's new display file location.
        LD      (L6401),HL                              ; Save as the new address within the display file of the player.

; Restore the background for the bottom 2 lines that the player has moved up from.

        LD      DE,$0210                                ; Offset of 16 lines.
        ADD     HL,DE                                   ; Point to below the player.
        LD      C,$02                                   ; There are 2 columns.
        LD      DE,L5B15+$0002                          ; Background buffer for the left hand column rocket pack base 1.

L6E7B:  PUSH    HL                                      ; Save the display file address.
        LD      B,$02                                   ; There are 2 lines to copy.
        CALL    L64DA                                   ; Copy a block into the display file.
        POP     HL                                      ; Retrieve the display file address.
        INC     HL                                      ; Advance to the right hand column.
        LD      DE,L5B01+$0002                          ; Background buffer for the right hand column rocket pack base 1.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6E7B                                ; Repeat for both columns.

; Shift the graphics in the rocket pack background buffers down 2 lines.

        LD      DE,L5AF5+$000F                          ; Point to the last line of the background buffer for the right column of the rocket pack.
        LD      HL,L5AF5+$000D                          ; Point 2 lines into the buffer.
        LD      BC,$000E                                ; There are 14 lines to shift 'down'.
        LDDR                                            ; Shift the graphics down 2 lines in the background buffer.

        LD      DE,L5B09+$000F                          ; Point to the last line of the background buffer for the left column of the rocket pack.
        LD      HL,L5B09+$000D                          ; Point 2 lines into the buffer.
        LD      BC,$000E                                ; There are 14 lines to shift 'down'.
        LDDR                                            ; Shift the graphics down 2 lines in the background buffer.

; Save the background graphics for the top 2 lines that the rocket pack has moved into.

        POP     HL                                      ; Retrieve the player's new display file location.
        LD      C,$02                                   ; There are 2 columns.
        LD      DE,L5B09                                ; The background buffer for the left column of the rocket pack.

L6EA5:  CALL    L6D5E                                   ; Copy 2 lines of the display file at the column position into the background buffer.
        LD      DE,L5AF5                                ; The background buffer for the right column of the rocket pack.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6EA5                                ; Repeat for both columns.

        JP      L6FB2                                   ; Jump to draw the rocket pack in the display file.

; Check whether it is time to move the player horizontally.

L6EB1:  LD      HL,L6406                                ; Point at the store holding the player horizontal movement timer.
        LD      A,(FRAMES)
        CP      (HL)                                    ; Is it time to move the player horizontally?
        JP      P,L6FB2                                 ; Jump if not to draw the rocket pack in the display file.

; It is time to move the player horizontally.

        DEC     HL                                      ; Point at the store holding the player's horizontal speed.
        DEC     HL
        SUB     (HL)                                    ; Determine the FRAMES number of the next horizontal movement.
        INC     HL
        INC     HL
        LD      (HL),A                                  ; Store the new horizontal movement timer.

; Which direction is the player facing?

        LD      HL,L6407                                ; Point at the movement flags.
        LD      A,(L68CE)                               ; Fetch the keys pressed flags.
        BIT     0,(HL)                                  ; Is the player facing right?
        JR      NZ,L6EE3                                ; Jump ahead if so.

; The player is facing left.

        BIT     3,A                                     ; Was the right key pressed?
        JR      Z,L6ED7                                 ; Jump if not.

; The player is facing left. The right key was pressed.

        SET     2,(HL)                                  ; Signal the player is flying right.

        BIT     4,A                                     ; Was the jump key pressed?
        JR      Z,L6EFD                                 ; Jump if not.

; The player is facing left. The right key was pressed. The jump key was pressed.

        JR      L6F01                                   ; Jump ahead to continue.

; The player is facing left. The right key was not pressed.

L6ED7:  BIT     2,A                                     ; Was the left key pressed?
        JR      Z,L6EFD                                 ; Jump if not.

; The player is facing left. The left key was pressed.

        RES     2,(HL)                                  ; Signal the player is flying left.

        BIT     4,A                                     ; Was the jump key pressed?
        JR      Z,L6EFD                                 ; Jump if not.

; The player is facing left. The left key was pressed. The jump key was pressed.

        JR      L6EF9                                   ; Jump ahead to continue.

; The player is facing right.

L6EE3:  BIT     2,A                                     ; Was the left key pressed?
        JR      Z,L6EEF                                 ; Jump if not.

; The player is facing right. The left key was pressed.

        RES     2,(HL)                                  ; Signal the player is flying left.

        BIT     4,A                                     ; Was the jump key pressed?
        JR      Z,L6EFD                                 ; Jump if not.

; The player is facing right. The left key was pressed. The jump key was pressed.

        JR      L6F01                                   ; Jump ahead to continue.

; The player is facing right. The left key was not pressed.

L6EEF:  BIT     3,A                                     ; Was the right key pressed?
        JR      Z,L6EFD                                 ; Jump if not.

; The player is facing right. The right key was pressed.

        SET     2,(HL)                                  ; Signal the player is flying right.

        BIT     4,A                                     ; Was the jump key pressed?
        JR      Z,L6EFD                                 ; Jump if not.

; Player continues to fly in the current direction when:
; The player is facing left. The left key was pressed. The jump key was pressed.
; The player is facing right. The right key was pressed. The jump key was pressed.

L6EF9:  LD      C,$02                                   ; High horizontal acceleration.
        JR      L6F16                                   ; Jump ahead to continue.

; The player is no longer flying when:
; The player is facing left. The right key was pressed. The jump key was not pressed.
; The player is facing left. The right key was not pressed. The left key was not pressed.
; The player is facing left. The left key was pressed. The jump key was not pressed.
; The player is facing right. The left key was pressed. The jump key was not pressed.
; The player is facing right. The left key was not pressed. The jump key was not pressed.
; The player is facing right. The right key was not pressed. The jump key was not pressed.

L6EFD:  LD      C,$01                                   ; Low falling acceleration.
        JR      L6F03                                   ; Jump ahead to continue.

; Player reverses direction of flying when:
; The player is facing left. The right key was pressed. The jump key was pressed.
; The player is facing right. The left key was pressed. The jump key was pressed.

L6F01:  LD      C,$02                                   ; High horizontal acceleration.

; The player is no longer flying or begins reversing horizontal direction.

L6F03:  LD      A,(L6404)                               ; Fetch the player's horizontal speed.
        CP      $07                                     ; Is the player exceeding the top horizontal flying speed?
        JR      NC,L6F10                                ; Jump ahead if so.

; The player has not reached top horizontal flying speed.

        ADD     A,C                                     ; Increase the player's horizontal speed ($01 if falling and $02 if flying).
        LD      (L6404),A                               ; Store the updated horizontal speed.

        JR      L6F25                                   ; Jump ahead to continue.

; The player has exceeded top horizontal flying speed.

L6F10:  LD      A,(HL)                                  ; Fetch the movement flags.
        XOR     $01                                     ; Reverse the horizontal flying direction to slow down the player.
        LD      (HL),A                                  ; Store the updated movement flags.

        JR      L6F25                                   ; Jump ahead to continue.

; The player continues to fly in the current direction.

L6F16:  LD      HL,L6404                                ; Point at the store holding the player's horizontal speed.
        LD      A,(HL)                                  ; Fetch the horizontal speed.
        SUB     C                                       ; Decrease the player's horizontal speed.
        CP      $02                                     ; Is the player above minimum horizontal flying speed?
        JR      NC,L6F21                                ; Jump ahead if not.

        LD      A,$02                                   ; Cap the at the minimum horizontal speed.

L6F21:  LD      (HL),A                                  ; Store the new horizontal speed.

        NOP                                             ; [Redundant]
        NOP     
        NOP

; Joins here after decreasing the player's horizontal speed.

L6F25:  LD      HL,L6407                                ; Point at the store holding the movement flags.
        LD      A,(L6404)                               ; Fetch the player's horizontal speed.
        BIT     0,(HL)                                  ; Is the player facing right?
        JR      NZ,L6F6E                                ; Jump if so.

; The player is facing to the left.

        CP      $07                                     ; Is the player exceeding the top vertical flying speed?
        JR      NC,L6FB2                                ; Jump if so to draw the rocket pack in the display file.

; The player has not reached top vertical flying speed.

        LD      HL,L6408                                ; Point at the store holding the player frame number.
        INC     (HL)                                    ; Next frame.
        LD      A,$05
        CP      (HL)                                    ; Has the last frame been exceeded?
        JR      NZ,L6FB2                                ; Jump if not to draw the rocket pack in the display file.

        LD      (HL),$01                                ; Wrap back to the first frame.

; Attempt to move the player one column to the left
; -------------------------------------------------

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        DEC     HL                                      ; Point to the display file location to the left.
        LD      BC,$1001                                ; There are 16 lines and 1 column to test.
        CALL    L700B                                   ; Can the player move? A return is not made if the move results in all diamonds being collected.

        NOP                                             ; [Redundant]
        NOP     

        JR      C,L6F89                                 ; Jump if the player cannot move to the new location.

; The player can move to the left.

        LD      (L6401),HL                              ; Save the new address within the display file of the player.

; Restore the background graphics in the column moved out of by the player.

        PUSH    HL                                      ; Save the new display file location of the player.
        INC     HL
        INC     HL                                      ; Point to the old display file right hand column location of the player.
        LD      B,$10                                   ; There are 16 lines to copy.
        LD      DE,L5AF5                                ; The background buffer for the right column of the rocket pack.
        CALL    L64DA                                   ; Copy the background buffer contents into the display file.

; Copy the background graphics from the left column buffer to the right column buffer.

        LD      BC,$0010                                ; There are 16 lines to copy.
        LD      DE,L5AF5                                ; The background buffer for the right column of the rocket pack.
        LD      HL,L5B09                                ; The background buffer for the left column of the rocket pack.
        LDIR                                            ; Copy the background graphics from the left column buffer to the right column buffer.

; Save the background graphics from the new column the player has moved into.

        POP     HL                                      ; Retrieve the new display file location of the player.
        LD      DE,L5B09                                ; The background buffer for the left column of the rocket pack.
        CALL    L6D50                                   ; Save the background at the new position into the buffer.

        JR      L6FB2                                   ; Jump ahead to draw the rocket pack in the display file.

; The player is facing to the right.

L6F6E:  CP      $07
        JR      NC,L6FB2                                ; Jump if not to draw the rocket pack in the display file.

        LD      HL,L6408                                ; Point at the store holding the player frame number.
        DEC     (HL)                                    ; Decrement the frame number.
        JR      NZ,L6FB2                                ; Jump if the last frame has not been exceeded.

        LD      (HL),$04                                ; Wrap back to the top frame number.

; Attempt to move the player one column to the right
; --------------------------------------------------

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        INC     HL
        INC     HL                                      ; Point 2 positions to the right within the display file.
        LD      BC,$1001                                ; There are 16 lines and 1 column to test.
        CALL    L700B                                   ; Can the player move? A return is not made if the move results in all diamonds being collected.

        NOP                                             ; [Redundant]
        NOP     

        JR      NC,L6F92                                ; Jump if the player can move to the new location.

; Joins here when the player cannot move to the left or right. The player's horizontal direction is reversed.

L6F89:  LD      HL,L6407                                ; Point at the store holding the movement flags.
        LD      A,(HL)                                  ; Fetch the movement flags.
        XOR     $01                                     ; Reverse horizontal direction the player is facing.
        LD      (HL),A                                  ; Store the updated flags.

        JR      L6FB2                                   ; Jump ahead to continue.

; The player can move to the right.

L6F92:  PUSH    HL                                      ; Save the new address of the right column of the player in the display file.
        DEC     HL                                      ; Point at the new address of the left column of the player in the display file.
        LD      (L6401),HL                              ; Save the new address within the display file of the player.

; Restore the graphics to the column the player has moved out from.

        DEC     HL                                      ; Point at the column the player has moved out from.
        LD      B,$10                                   ; There are 16 lines to copy.
        LD      DE,L5B09                                ; The background buffer for the left column of the rocket pack.
        CALL    L64DA                                   ; Copy the background buffer contents into the display file.

; Copy the background graphics from the right hand column buffer into the left hand column buffer.

        LD      BC,$0010                                ; There are 16 lines to copy.
        LD      DE,L5B09                                ; The background buffer for the left column of the rocket pack.
        LD      HL,L5AF5                                ; The background buffer for the right column of the rocket pack.
        LDIR                                            ; Copy the background graphics from the right hand column buffer into the left hand column buffer.

; Save the background graphics at the new column the player has moved into.

        POP     HL                                      ; Retrieve the new address of the right column of the player in the display file.
        LD      DE,L5AF5                                ; The background buffer for the right column of the rocket pack.
        CALL    L6D50                                   ; Save the background at the new position into the buffer.

; Draw the rocket pack graphic into the display file
; --------------------------------------------------
; All paths join here, whether the player could move or not.

L6FB2:  LD      DE,L5B05                                ; The destination is the rocket pack right column base 2 background buffer.
        LD      HL,L5B01                                ; The source is the rocket pack right column base 1 background buffer.
        LD      BC,$0004                                ; There are 4 characters in the buffer.
        LDIR                                            ; Copy the buffer contents.

        LD      DE,L5B19                                ; The destination is the rocket pack left column base 2 background buffer.
        LD      HL,L5B15                                ; The source is the rocket pack left column base 1 background buffer.
        LD      BC,$0004                                ; There are 4 characters in the buffer.
        LDIR                                            ; Copy the buffer contents.

; Determine the rocket pack graphic to display.

        LD      HL,L6408                                ; Point to the store holding the player frame number.
        LD      B,(HL)                                  ; Fetch the frame number.
        SLA     B                                       ; Multiple the frame number by 2 => 2, 4, 6 or 8.
        LD      A,(L6407)                               ; Fetch the movement flags.
        BIT     2,A                                     ; Is the player flying to the right?
        JR      NZ,L6FDA                                ; Jump ahead if so.

; Flying left.

        LD      HL,L5B09-$0028                          ; The base address for the left rocket pack graphics (index is 0 based) [1=>5B09, 2=>5B31, 3=>5B59, 4=>5B81].
        JR      L6FDD                                   ; Jump ahead to continue.

; Flying right.

L6FDA:  LD      HL,L5A69-$0028                          ; The base address for the right rocket pack graphics (index is 0 based) [1=>5A69, 2=>5A7D, 3=>5AA5, 4=>5ACD].

L6FDD:  LD      DE,$0014                                ; The size of each rocket pack graphic.

L6FE0:  ADD     HL,DE                                   ; Advance to the next graphic.
        DJNZ    L6FE0                                   ; Repeat until the required graphic has been reached.

; Draw the rocket pack graphic without the base in the display file.

        PUSH    HL                                      ; Save the address of the graphic for the left column.
        ADD     HL,DE                                   ; Advance to the graphic for the right column.
        PUSH    HL                                      ; Save the address of the graphic for the right column.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        INC     HL                                      ; Advance one position to the right since the player has moved right.

        LD      C,$02                                   ; There are 2 columns to draw.

L6FEC:  POP     DE                                      ; Retrieve the address of the rocket pack graphic.
        PUSH    HL                                      ; Save the address within the display file of the player.
        LD      B,$0C                                   ; There are 12 lines to rocket pack graphic.
        CALL    L64DA                                   ; Copy the graphic into the display file.

; Check whether the jump was pressed, which determines the graphic to use for the base of the rocket pack.

        LD      A,(L68CE)                               ; Fetch the keys pressed flags.
        BIT     4,A                                     ; Was the jump key pressed?
        JR      Z,L6FFE                                 ; Jump ahead if not.

; The jump key was pressed.

        INC     DE                                      ; Advance to the rocket active base graphic.
        INC     DE
        INC     DE
        INC     DE

; Draw the rocket pack base graphic into the display file.

L6FFE:  LD      B,$04                                   ; There are 4 lines to the rocket pack base.
        CALL    L64DA                                   ; Copy a block into the display file.

        POP     HL                                      ; Retrieve the address within the display file of the player.
        DEC     HL                                      ; Point to the column to the left.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L6FEC                                ; Loop back to draw all columns.

        JP      L7323                                   ; Jump to handle movement of the Bubloid.

; ---------------------------------------------------------
; Check Whether Player Can Move When Flying Via Rocket Pack
; ---------------------------------------------------------
; This routine tests whether the player has been caught by Bubloid or whether the player has collected a diamond.
; If all diamonds have been collected then this routine does not return but exits to $7748.
; Entry: HL=New location within the display file.
;        B=Number of lines to test.
;        C=Number of columns to test.
; Exit : Carry flag set if the player cannot move to the new location.
;        HL=The new player's location within the display file (if the move is possible).

L700B:  LD      A,(HL)                                  ; Fetch the contents at the new position.
        CP      $C9                                     ; Is the new location off the edge of the screen?
        JP      Z,L70E0                                 ; Jump if so.

; The new location is not off the edge of the screen.

        PUSH    HL                                      ; Save the player's location within the display file.

        CALL    L6CA8                                   ; Check whether the player has been caught by the Bubloid.
        JR      NC,L7039                                ; Jump ahead if the player has not been caught.

; The player has been caught by the Bubloid.

L7017:  CALL    L6CFD                                   ; Configure caught by the Bubloid.

        LD      HL,L5AF5                                ; The background buffer for the right column of the rocket pack.
        LD      DE,L6119                                ; The background buffer for the right column of the Caught by Bubloid.
        LD      BC,$0010                                ; There are 16 lines to copy.
        LDIR                                            ; Copy the graphics from the rocket pack background buffer to the Caught by Bubloid background buffer.

        LD      HL,L5B09                                ; The background buffer for the left column of the rocket pack.
        LD      DE,L6089                                ; The background buffer for the left column of the Caught by Bubloid.
        LD      BC,$0010                                ; There are 16 lines to copy.
        LDIR                                            ; Copy the graphics from the rocket pack background buffer to the Caught by Bubloid background buffer.

        LD      A,(FRAMES)
        LD      (L6405),A                               ; Set the player's vertical movement timer so that the player will move on the next cycle.

        JP      L70D7                                   ; Jump ahead to continue.

; The player has not been caught by the Bubloid.

L7039:  PUSH    HL                                      ; Save the player's location within the display file.

        LD      DE,L418A                                ; Line 8 column 0.
        AND     A
        SBC     HL,DE                                   ; Is the player between line 0 and 7, where the score is displayed?

        POP     HL                                      ; Retrieve the player's location within the display file.
        JP      C,L70D7                                 ; Jump if the player is between line 0 and 7.

; The player is not at lines 0 to 7. Test whether the player's new position collides with a diamond.

        LD      A,C                                     ; Fetch the width of the player graphic.

L7045:  PUSH    HL                                      ; Save the player's location within the display file.
        PUSH    BC                                      ; Save the width and column counts.
        PUSH    AF                                      ; Save the column count.

; Enter a loop to test each line of the player against the 5 diamonds and diamond platforms.

L7048:  PUSH    BC                                      ; The line count.

        EX      DE,HL                                   ; DE=Player's location within the display file.
        LD      A,(DE)                                  ; Fetch the character at the player's new location.
        CP      $1C                                     ; Is it the top of a diamond platform?
        JP      Z,L70D3                                 ; Jump if so, i.e. the new player position collides with a diamond platform.

; The new player position does not collide with a diamond platform.

        LD      B,$05                                   ; There are 5 diamonds to test for.

; Enter a loop to test for each diamond.

L7052:  LD      HL,(L64C1)                              ; Fetch the address of the current diamond being tested.
        AND     A
        SBC     HL,DE                                   ; Does the location contain the top of the diamond?
        JR      Z,L7079                                 ; Jump if so.

; The location does not contain the top of the diamond.

        LD      HL,L7052+$0001                          ; Point to the diamond location store accessed by this routine.
        LD      A,$02                                   ; Offset to the next diamond store.
        ADD     A,(HL)                                  ; Point to the next diamond location store.
        LD      (HL),A                                  ; Modify the program so that it tests this diamond upon the next loop iteration.

        DJNZ    L7052                                   ; Repeat for all diamonds.

; This line of the column of the player has not collided with a diamond or diamond platform.

        LD      A,L64C1 & $FF
        LD      (L7052+$0001),A                         ; Restore the program to point back at the top diamond.

        LD      HL,$0021
        ADD     HL,DE                                   ; Advance to the next line.

        POP     BC                                      ; The line count.
        DJNZ    L7048                                   ; Repeat for all lines in the current column.

; This column of the player has not collided with a diamond or diamond platform.

        POP     AF                                      ; Retrieve the column count.
        POP     BC                                      ; Retrieve the width and column counts.

        POP     HL                                      ; Retrieve the player's location within the display file.
        INC     HL                                      ; Advance to the next column within the display file.
        DEC     A                                       ; Decrement the column count.
        JR      NZ,L7045                                ; Loop back to test all columns.

; The player has not collided with a diamond or diamond platform.

        XOR     A                                       ; Reset the carry flag to signal that the player can move to the new location.
        POP     HL                                      ; Retrieve the player's location within the display file.
        RET     

; The location contains the top of the diamond.
; B=Diamond collided with ($01 for the top diamond to $05 for the bottom diamond).

L7079:  POP     HL                                      ; Discard the line count.
        POP     HL                                      ; Discard the column count.
        POP     HL                                      ; Discard the width and column counts.
        POP     HL                                      ; Retrieve the player's location within the display file.

; Determine the store holding the address of the top line of the diamond that has been collided with. The program code that
; follows this routine is modified so that it accesses one of the diamond stores: $64C1, $64C3, $64C5, $64C7 or $64C9.

        LD      A,(L64C9+$0002) & $FF                   ; Low byte of base address corresponding to B=0.

L707F:  SUB     $02                                     ; Decrement the low address byte by 2.
        DJNZ    L707F                                   ; Repeat until the count reaches 0.

; A now holds $C1, $C3, $C5, $C7 or $C9.

        LD      (L7089+$0001),A                         ; Modify the program below that fetches the address of the diamond platform.
        LD      (L7099+$0001),A                         ; Modify the program below that stores the address of the diamond platform.

; Erase the diamond from the screen.

L7089:  LD      HL,(L64C1-$00C1)                        ; Fetch the address of the top line of the diamond within the display file.
        LD      DE,$0021                                ; The number of bytes in a line.
        LD      B,$06                                   ; The height of the diamond graphic.

L7091:  LD      (HL),$19                                ; The pseudo hi-res code for all black pixels.
        ADD     HL,DE                                   ; Advance to the next line.
        DJNZ    L7091                                   ; Repeat for all lines forming the diamond.

        LD      HL,$0000                                ; Mark the diamond as removed.
L7099:  LD      (L64C1-$00C1),HL

; Increment the score by 1000.

        LD      A,$01                                   ; The increment to apply to the selected score digit.
        LD      B,$04                                   ; There are potentially 4 digits that need updating.
        LD      HL,L64B1-$0003                          ; Point to the 1000s digit of the score.
        CALL    L6D6F                                   ; Add to the score.

; Copy the score into the hi-res display file.

        LD      HL,L40C4+$0007                          ; Line 2 column 7.
        LD      BC,L64B1                                ; Point to the last digit of the score.
        LD      A,$07                                   ; There are 7 digits in the score.
        CALL    L6515                                   ; Display the score value in the display file.

; Test whether all diamonds have been collected.

        LD      B,$05                                   ; There are 5 diamonds to check.
        LD      DE,$0000                                ; A collected diamond with have its address set as $0000.

L70B6:  LD      HL,(L64C1)                              ; Fetch the address of the current diamond being tested.
        AND     A
        SBC     HL,DE                                   ; Has this diamond been collected?
        JR      NZ,L70D7                                ; Jump if not.

; This diamond has been collected.

        LD      HL,L70B6+$0001                          ; Point to the diamond location store accessed by this routine.
        LD      A,$02                                   ; The offset to the next diamond location store.
        ADD     A,(HL)                                  ; Point to the next diamond location store.
        LD      (HL),A                                  ; Modify the program so that it tests this diamond upon the next loop iteration.

        DJNZ    L70B6                                   ; Repeat for all diamonds.

; All diamonds have been collected.

        LD      HL,L64BE                                ; Point to the number of lives remaining.
        INC     (HL)                                    ; Increment the number of lives remaining.

        POP     HL                                      ; Discard the player's location within the display file.
        POP     HL                                      ; Discard the return address.

        LD      HL,L7748                                ; Stack a new return address that will add the bonus to the score and
        PUSH    HL                                      ; then start the next level.

        JR      L70D8                                   ; Jump ahead to continue.

; The new player position collides with a diamond platform.

L70D3:  POP     HL                                      ; Discard the line count.
        POP     HL                                      ; Discard the column count.
        POP     HL                                      ; Discard the width and column counts.
        POP     HL                                      ; Discard the player's location within the display file.

; Joins here when:
; - The player is above the top diamond.
; - A diamond has been collected but others remain.
; - The player has been caught by the Bubloid.

L70D7:  POP     HL                                      ; Retrieve the player's location within the display file.

; Joins here when all diamonds have been collected.

L70D8:  LD      A,L64C1 & $FF                           ; The low byte of the store holding the address of the top diamond.
        LD      (L70B6+$0001),A                         ; Restore the program to point back at the top diamond.
        LD      (L7052+$0001),A                         ; Restore the program to point back at the top diamond.

; Joins here when the new location is off the edge of the screen.

L70E0:  XOR     A                                       ; [Redundant]
        SCF                                             ; Set the carry flag to signal that the player cannot move to the new location.
        RET     

; ----------------------------------
; The Player is Flying via a Vulture
; ----------------------------------

L70E3:  LD      A,(L68CE)                               ; Fetch the keys pressed flags.
        BIT     4,A                                     ; Was the jump key pressed?
        LD      HL,L6404                                ; Point at the store holding the Player's vertical flying speed when flying via a vulture.
        JR      Z,L70F5                                 ; Jump if not.

; The jump key was pressed.

        BIT     7,(HL)                                  ; Was the jump key previously being pressed?
        JR      Z,L70F7                                 ; Jump ahead if it was.

; The jump key was not previously but is now being pressed, i.e. flap the vulture's wings.

        LD      (HL),$05                                ; Set to maximum vertical flying speed.
        JR      L70F7                                   ; Jump ahead to continue.

; The jump key was not pressed, or the jump key is being pressed but was previously being pressed.

L70F5:  SET     7,(HL)                                  ; Signal the jump key is not being pressed for the next check of the vulture movement.

; Adjust the player's vertical speed.

L70F7:  LD      A,(HL)                                  ; Fetch the Player's vertical flying speed when flying via a vulture.
        AND     $7F                                     ; Discard the status bit indicating whether the jump key was previously being pressed.
        JR      Z,L7165                                 ; Jump ahead if the vertical flying speed is zero.

        DEC     (HL)                                    ; Decrement the vertical flying speed.

; Attempt to move the player up.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      DE,$0042                                ; Offset of 2 lines.
        AND     A
        SBC     HL,DE                                   ; Point 2 lines up.
        LD      BC,$0203                                ; There are 2 lines and 3 columns to test.
        CALL    L72E2                                   ; Attempt to move the player.
        JR      NC,L7116                                ; Jump if the player could move up.

; The player could not move up.

        CP      $00                                     ; Could the player not move because at the edge of the screen?
        JP      Z,L71E2                                 ; Jump ahead if so.

; The player was caught by the Bubloid.

        JP      L72DF                                   ; Jump to handle movement of the Bubloid.

; The player could move up.

L7116:  PUSH    HL                                      ; Save the new address within the display file of the player.
        LD      (L6401),HL                              ; Store the new address within the display file of the player.

; Restore the background graphics into the 2 lines the player has moved out of.

        LD      DE,$0210                                ; Offset of 16 lines.
        ADD     HL,DE                                   ; Point to the position below the player.
        LD      C,$03                                   ; There are 3 columns to restore.
        LD      DE,L5E79+$000E                          ; Point at the last 2 lines of the left column of the background buffer for the vulture.

L7123:  PUSH    HL                                      ; Save the address in the display file.
        LD      B,$02                                   ; There are 2 lines to copy.
        CALL    L64DA                                   ; Copy the background graphics into the display file.
        POP     HL                                      ; Retrieve the address in the display file.
        INC     HL                                      ; Advance to the column to the right.
        LD      A,$0E
        ADD     A,E                                     ; Offset to the bottom 2 lines in the next column background buffer.
        LD      E,A
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L7123                                ; Repeat for all columns.

; Shift the graphics in the vulture background buffer up 2 lines.

        LD      DE,L5E79+$000F                          ; Point at the last line of the left column of the vulture background buffer.
        LD      HL,L5E79+$000D                          ; Point 2 lines higher up in the background buffer.
        LD      BC,$000E                                ; There are 14 lines to shift.
        LDDR                                            ; Shift the lines in the column background buffer.

        LD      DE,L5E89+$000F                          ; Point at the last line of the middle column of the vulture background buffer.
        LD      HL,L5E89+$000D                          ; Point 2 lines higher up in the background buffer.
        LD      BC,$000E                                ; There are 14 lines to shift.
        LDDR                                            ; Shift the lines in the column background buffer.

        LD      DE,L5E99+$000F                          ; Point at the last line of the right column of the vulture background buffer.
        LD      HL,L5E99+$000D                          ; Point 2 lines higher up in the background buffer.
        LD      BC,$000E                                ; There are 14 lines to shift.
        LDDR                                            ; Shift the lines in the column background buffer.

; Save the background graphics for the bottom 2 lines that the vulture has moved into.

        POP     HL                                      ; Retrieve the new address within the display file of the player.
        LD      C,$03                                   ; There are 3 columns to draw.
        LD      DE,L5E79                                ; Point at the left column of the vulture background buffer.

L7159:  CALL    L6D5E                                   ; Copy 2 lines of the display file at the column position into the background buffer.
        LD      A,$0E
        ADD     A,E                                     ; Offset to the start of the next column in the background buffer.
        LD      E,A
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L7159                                ; Repeat for all columns.

        JR      L71E2                                   ; Jump ahead to continue.

; The player's vertical speed is zero so attempt to move the player down.

L7165:  LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        PUSH    HL                                      ; Save the address within the display file of the player.
        LD      DE,$0210                                ; Offset of 16 lines.
        ADD     HL,DE                                   ; Point to below the player position.
        LD      BC,$0203                                ; There are 2 lines and 3 columns to test.
        CALL    L72E2                                   ; Attempt to move the player.
        JR      NC,L717D                                ; Jump if the player could move down.

; The player could not move down.

        POP     HL                                      ; Retrieve the address within the display file of the player.
        CP      $00                                     ; Could the player not move because at the edge of the screen?
        JR      Z,L71E2                                 ; Jump ahead if so.

; The player was caught by the Bubloid.

        JP      L72DF                                   ; Jump to handle movement of the Bubloid.

; The player could move down.

L717D:  LD      DE,$01CE                                ; Offset of 14 lines.
        AND     A
        SBC     HL,DE                                   ; Point 2 lines down from the current player position within the display file.
        LD      (L6401),HL                              ; Save the new address within the display file of the player.

        LD      DE,L5564+$001F                          ; Line 162 column 31.
        AND     A
        SBC     HL,DE                                   ; Has the player fallen into the sea?
        JR      C,L7196                                 ; Jump if not.

; The player has fallen into the sea.

        LD      HL,L6408                                ; Point at the store holding the player frame number.
        LD      (HL),$04                                ; Set such that the initial splash graphic frame will be shown.
        INC     HL
        SET     7,(HL)                                  ; Signal fallen into the sea, i.e. the splash graphic needs to be drawn

; Restore the background graphics into the 2 lines the player has moved out of.

L7196:  POP     HL                                      ; Retrieve the address within the display file of the player.
        LD      C,$03                                   ; There are 3 columns to draw.
        LD      DE,L5E79                                ; Point at the left column of the background buffer for the vulture.

L719C:  PUSH    HL                                      ; Save the address within the display file of the player.
        LD      B,$02                                   ; There are 2 lines to restore.
        CALL    L64DA                                   ; Copy the background graphics into the display file.
        POP     HL                                      ; Retrieve the address within the display file of the player.
        INC     HL
        LD      A,$0E
        ADD     A,E                                     ; Offset to the next column of the background buffer.
        LD      E,A
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L719C                                ; Repeat for all columns.

; Shift the graphics in the vulture background buffer down 2 lines.

        LD      DE,L5E79                                ; Point at the left column of the vulture background buffer.
        LD      HL,L5E79+$0002                          ; Point 2 lines lower down in the background buffer.
        LD      BC,$000E                                ; There are 14 lines to shift.
        LDIR                                            ; Shift the lines in the column background buffer.

        LD      DE,L5E89                                ; Point at the middle column of the vulture background buffer.
        LD      HL,L5E89+$0002                          ; Point 2 lines lower down in the background buffer.
        LD      BC,$000E                                ; There are 14 lines to shift.
        LDIR                                            ; Shift the lines in the column background buffer.

        LD      DE,L5E99                                ; Point at the right column of the vulture background buffer.
        LD      HL,L5E99+$0002                          ; Point 2 lines lower down in the background buffer.
        LD      BC,$000E                                ; There are 14 lines to shift.
        LDIR                                            ; Shift the lines in the column background buffer.

; Save the background graphics for the top 2 lines that the vulture has moved into.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      DE,$01CE                                ; Offset of 14 lines.
        ADD     HL,DE                                   ; Point at the bottom 2 lines that the player will now occupy.
        LD      C,$03                                   ; There are 3 columns to draw.
        LD      DE,L5E79+$000E                          ; Point at the last 2 lines of the left column of the background buffer.

L71D8:  CALL    L6D5E                                   ; Copy 2 lines of the display file at the column position into the background buffer.
        LD      A,$0E
        ADD     A,E                                     ; Offset to the start of the next column in the background buffer.
        LD      E,A
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L71D8                                ; Repeat for all columns.

; All paths join here, when either the player tried to move off screen and couldn't, or the player moved up, or the player moved down.

L71E2:  LD      A,(L68CE)                               ; Fetch the keys pressed flags.
        BIT     2,A                                     ; Was the left key pressed?
        JR      Z,L723C                                 ; Jump if not.

; The left key was pressed.

        LD      HL,L6408                                ; Point at the store holding the player frame number.
        INC     (HL)                                    ; Move to the next frame.
        LD      A,$05
        CP      (HL)                                    ; Has the last frame bee exceeded?
        JR      NZ,L7239                                ; Jump if not.

        LD      (HL),$01                                ; Wrap back to the first frame.

; Attempt to move the player left.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        DEC     HL                                      ; Point to the location to the left.
        LD      BC,$1001                                ; There is 1 line and 1 column to test.
        CALL    L72E2                                   ; Attempt to move the player.
        JR      NC,L720D                                ; Jump if the player could move left.

; The player could not move left.

        LD      HL,L6408                                ; Point at the store holding the player frame number.
        LD      (HL),$04
        CP      $00                                     ; Could the player not move because at the edge of the screen?
        JP      Z,L728F                                 ; Jump ahead if so.

        JP      L72DF                                   ; Jump to handle movement of the Bubloid.

; The player could move left.

L720D:  LD      (L6401),HL                              ; Store the address within the display file of the player.

; Restore the background graphics into the column the player has moved out of.

        PUSH    HL                                      ; Save the address within the display file of the player.
        INC     HL
        INC     HL
        INC     HL                                      ; Point to the old right hand column position.
        LD      B,$10                                   ; There are 16 lines to copy.
        LD      DE,L5E99                                ; The right column of the background buffer for the vulture.
        CALL    L64DA                                   ; Copy the background graphics into the display file.

; Shift the graphics in the vulture background one column to the left.

        LD      BC,$0010                                ; There are 16 lines to shift.
        LD      DE,L5E99                                ; The right column of the background buffer for the vulture.
        LD      HL,L5E89                                ; The middle column of the background buffer for the vulture.
        LDIR                                            ; Shift the lines in the column background buffer.

        LD      DE,L5E89                                ; The middle column of the background buffer for the vulture.
        LD      HL,L5E79                                ; The left column of the background buffer for the vulture.
        LD      BC,$0010                                ; There are 16 lines to shift.
        LDIR                                            ; Shift the lines in the column background buffer.

; Save the new background graphics where the vulture has moved to.

        POP     HL                                      ; Retrieve the address within the display file of the player.
        LD      DE,L5E79                                ; The left column of the background buffer for the vulture.
        CALL    L6D50                                   ; Copy the column of the display file into the background buffer.

; Joins here when the last frame moving left has not been reached.

L7239:  XOR     A                                       ; Signal the player is facing left.
        JR      L728B                                   ; Jump ahead to continue.

; The left key was not pressed.

L723C:  BIT     3,A                                     ; Was the right key pressed?
        JR      Z,L728E                                 ; Jump if not.

; The right key was pressed.

        LD      HL,L6408                                ; Point at the store holding the player frame number.
        DEC     (HL)                                    ; Move to the next frame.
        JR      NZ,L7289                                ; Jump if the last frame has been exceeded.

        LD      (HL),$04                                ; Wrap back to the first frame.

; Attempt to move the player right.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        INC     HL
        INC     HL
        INC     HL                                      ; Point to the location to the right that player wants to move into.
        LD      BC,$1001                                ; There is 1 line and 1 column to test.
        CALL    L72E2                                   ; Attempt to move the player.
        JR      NC,L725D                                ; Jump if the player could move right.

; The player could not move right.

        CP      $00                                     ; Could the player not move because at the edge of the screen?
        JR      Z,L728E                                 ; Jump ahead if so.

        JP      L72DF                                   ; Jump to handle movement of the Bubloid.

; The player could move right.

L725D:  PUSH    HL                                      ; Save the address of the new right hand column of the player's position in the display file.
        DEC     HL
        DEC     HL                                      ; Point to the address of the new left hand column of the player's position in the display file.
        LD      (L6401),HL                              ; Save as the new address within the display file of the player.

; Restore the background graphics into the display file column moved out of by the player.

        DEC     HL                                      ; Point at the old left hand column occupied by the player in the display file.
        LD      B,$10                                   ; There are 16 lines to restore.
        LD      DE,L5E79                                ; The left hand column of the background buffer for the vulture.
        CALL    L64DA                                   ; Copy the background buffer graphics into the display file.

; Shift the graphics in the background buffer one column to the right.

        LD      BC,$0010                                ; There are 16 lines to shift.
        LD      DE,L5E79                                ; The left hand column of the background buffer for the vulture.
        LD      HL,L5E89                                ; The middle column of the background buffer for the vulture.
        LDIR                                            ; Shift the graphics.

        LD      BC,$0010                                ; There are 16 lines to shift.
        LD      DE,L5E89                                ; The middle column of the background buffer for the vulture.
        LD      HL,L5E99                                ; The right hand column of the background buffer for the vulture.
        LDIR                                            ; Shift the graphics.

; Save the new background graphics where the vulture has moved to.

        POP     HL                                      ; Retrieve the address of the new right hand column of the player's position in the display file.
        LD      DE,L5E99                                ; The right hand column of the background buffer for the vulture.
        CALL    L6D50                                   ; Copy the column of the display file into the background buffer.

; Joins here when the last frame moving right has not been reached.

L7289:  LD      A,$01                                   ; Signal the player is facing right.

; Joins here when moving left.

L728B:  LD      (L6407),A                               ; Store the flag indicating the direction the player is facing.

; Multiple paths converge here:
; - Neither left or right key pressed.
; - Attempted to move right but could not move off screen.
; - Could move right.
; - Could move left.
; - Right pressed but last frame not reached.
; - Left pressed but last frame not reached.

L728E:  NOP                                             ; [Redundant]

; Joins here when attempted to move left but could not move off screen.

L728F:  LD      HL,L6408                                ; Point at the store holding the player frame number.
        LD      B,(HL)                                  ; Fetch the frame number.
        DEC     HL                                      ; Point at the movement flags.
        LD      A,(L68CE)                               ; Fetch the keys pressed flags.
        LD      DE,$00B0                                ; Offset to select the flying vulture graphics.
        BIT     0,(HL)                                  ; Facing right?
        JR      NZ,L72A3                                ; Jump if so.

; Facing left.

        LD      HL,L5CC9                                ; Base address for the right column for the walking vulture left graphics (index based from 0).
        JR      L72A6                                   ; Jump ahead to continue.

; Facing right.

L72A3:  LD      HL,L5B69                                ; Base address for the left column for the walking vulture right graphics (index based from 0).

L72A6:  BIT     4,A                                     ; Was the jump key pressed?
        JR      Z,L72AB                                 ; Jump ahead if not.

; The jump key was pressed.

        ADD     HL,DE                                   ; Point at the flying vulture graphics.

L72AB:  LD      DE,$0030                                ; The size of each vulture graphic (3 columns of 16 lines).

L72AE:  ADD     HL,DE                                   ; Advance to the next graphic.
        DJNZ    L72AE                                   ; Repeat until the required graphic has been reached.

        EX      DE,HL                                   ; DE=Address of the vulture graphic.
        LD      A,(L6408)                               ; Fetch the player frame number.
        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        CP      $01                                     ; Is this the first frame, which is only 2 columns wide?
        JR      NZ,L72D2                                ; Jump ahead if not.

; This is the first frame, which is only 2 columns wide.

        PUSH    HL                                      ; Save the address within the display file of the player.
        PUSH    DE                                      ; Save the address of the vulture graphic.

        LD      DE,L5E79                                ; The address of the left column of the background buffer for the vulture.
        LD      B,$10                                   ; There are 16 lines to copy.
        CALL    L64DA                                   ; Copy the column from the background buffer into the display file.

        LD      HL,$0010                                ; Offset to the first column of the vulture graphic.
        POP     DE                                      ; Retrieve the address of the vulture graphic.
        ADD     HL,DE                                   ; Point at the start of the 2 column wide vulture graphic.
        EX      DE,HL                                   ; DE=Address of the vulture graphic.

        POP     HL                                      ; Retrieve the address within the display file of the player.
        INC     HL                                      ; Advance to the column to the right since the graphic is only 2 columns wide.
        LD      C,$02                                   ; Set the column count to 2.
        JR      L72D4                                   ; Jump ahead to continue.

; Frame 2 or above.

L72D2:  LD      C,$03                                   ; Set the column count to 3.

; Both paths converge here to draw the vulture in the display file.

L72D4:  LD      B,$10                                   ; There are 16 lines to draw.
        PUSH    HL                                      ; Save the address within the display file.
        CALL    L64DA                                   ; Copy a block into the display file.
        POP     HL                                      ; Retrieve the address within the display file.
        INC     HL                                      ; Advance to the next column to the right.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L72D4                                ; Repeat for all columns.

L72DF:  JP      L7323                                   ; Jump to handle movement of the Bubloid.

; -----------------------------------------------------
; Check Whether Player Can Move When Flying Via Vulture
; -----------------------------------------------------
; This routine tests whether the player has been caught by Bubloid or whether the player has collected a diamond.
; If all diamonds have been collected then this routine does not return but exits to $7748.
; Entry: HL=New location within the display file.
;        B=Number of lines to test.
;        C=Number of columns to test.
; Exit:  Carry flag set if the player cannot move to the new location.
;        HL=The new player's location within the display file (if the move is possible).
;        A=$00 if move would be off screen, $01=caught by bubloid?

L72E2:  LD      A,(HL)                                  ; Fetch the contents at the new position.
        CP      $C9                                     ; Is the new location off the edge of the screen?
        JR      NZ,L72EA                                ; Jump if not.

; The new location is off the edge of the screen.

        XOR     A                                       ; Signal could not move off screen.
        JR      L731B                                   ; Jump to make a return with the carry flag set.

; The new location is not off the edge of the screen.

L72EA:  CALL    L6CA8                                   ; Check whether the player has been caught by the Bubloid.
        JR      C,L72F3                                 ; Jump ahead if the player has been caught.

; The player has not been caught by the Bubloid.

        PUSH    HL                                      ; Save the player's location within the display file.
        JP      L7039                                   ; Check whether the player has collided with a diamond.

; The player has been caught by the Bubloid so transfer the graphics from the Vulture background buffer into the Caught by the Bubloid background buffer.

L72F3:  CALL    L6CFD                                   ; Configure caught by the Bubloid.

        LD      HL,L5E79                                ; The source is the left column of the Vulture background buffer.
        LD      DE,L6089                                ; The destination is the left column of the Caught by the Bubloid background buffer.
        LD      BC,$0010                                ; There are 16 lines to copy.
        LDIR                                            ; Copy the background graphics.

        LD      HL,L5E89                                ; The source is the middle column of the Vulture background buffer.
        LD      DE,L6119                                ; The destination is the right column of the Caught by the Bubloid background buffer.
        LD      BC,$0010                                ; There are 16 lines to copy.
        LDIR                                            ; Copy the background graphics.    

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        INC     HL
        INC     HL                                      ; Point to the column where the right of the vulture was displayed.
        LD      DE,L5E99                                ; The source is the right column of the Vulture background buffer.
        LD      B,$10                                   ; There are 16 lines to copy.
        CALL    L64DA                                   ; Restore the background for the column into the display file.

        LD      A,$01                                   ; Signal caught by the Bubloid.

L731B:  SCF                                             ; Signal the player cannot move.
        RET     

; -------------------
; End of BASIC Line 5
; -------------------

L731D:  DEFB _NEWLINE

; ------------
; BASIC Line 7
; ------------

L731E:  DEFB 7 >> 8
        DEFB 7 & $FF
        DEFW L75DD - 1 - $
        DEFB _REM

; ------------------------------
; Handle Movement of the Bubloid
; ------------------------------

L7323:  LD      A,(L6409)                               ; Fetch the player flags.
        AND     $90                                     ; Has the player been caught by the Bubloid or fallen into the sea?
        JP      NZ,L68CF                                ; Jump if so to handle movement of the player.

; Check whether it is time to process the Bubloid vertical movement
; -----------------------------------------------------------------

        LD      HL,L6421                                ; Point at the store holding the Bubloid vertical movement timer.
        LD      A,(FRAMES)
        CP      (HL)                                    ; Is it time to move the Bubloid vertically?
        JP      P,L742F                                 ; Jump if not to check whether it is time to process the Bubloid horizontal movement.

; It is time to process the Bubloid vertical movement.

        DEC     HL                                      ; Point at the store holding the Bubloid vertical speed.
        SUB     (HL)                                    ; Determine the FRAMES number of the next vertical movement
        INC     HL                                      ; Point back at the Bubloid vertical movement timer.
        LD      (HL),A                                  ; Store the updated timer value.

; Determine the Bubloid's vertical speed
; --------------------------------------

        CALL    L75B8                                   ; Determine the horizontal and vertical directions of the player from the Bubloid.

        LD      HL,L641E                                ; Point at the store holding the Bubloid movement flags.
        BIT     1,(HL)                                  ; Is the Bubloid moving up?
        INC     HL                                      ; Point at the store holding the Bubloid vertical acceleration.
        JR      NZ,L734A                                ; Jump if so.

; The Bubloid is moving down.

        BIT     7,A                                     ; Is the player higher than the Bubloid?
        JR      Z,L734E                                 ; Jump ahead if not.

; The Bubloid is moving down and the player is higher than the Bubloid.

        JR      L7353                                   ; Jump ahead to continue.

; The Bubloid is moving up.

L734A:  BIT     7,A                                     ; Is the player higher than the Bubloid?
        JR      Z,L7353                                 ; Jump ahead if not.

; The Bubloid is moving up and the player is higher than the Bubloid.
; The Bubloid is moving down and the player is lower than the Bubloid.

L734E:  CALL    L7594                                   ; Increase the speed of the Bubloid towards the player.
        JR      L7358                                   ; Jump ahead to continue.

; The Bubloid is moving down and the player is higher than the Bubloid.
; The Bubloid is moving up and the player is lower than the Bubloid.

L7353:  LD      C,$02                                   ; Bit value for the vertical flag in the Bubloid movement flags.
        CALL    L75A2                                   ; Decrease the speed of the Bubloid away from the player until able to reverse direction.

; The Bubloid's vertical speed has been updated.

L7358:  LD      HL,L641E                                ; Point at the store holding the Bubloid movement flags.
        BIT     1,(HL)                                  ; Is the Bubloid moving up?
        JR      NZ,L73BF                                ; Jump if so.

; Move the Bubloid down
; ---------------------

        LD      HL,(L6411)                              ; Fetch the location of the Bubloid in the display file.
        PUSH    HL                                      ; Save the original location of the Bubloid in the display file.
        LD      DE,$0042                                ; Offset of 2 lines.
        ADD     HL,DE                                   ; Move the Bubloid down 2 lines.
        LD      (L6411),HL                              ; Store the new location of the Bubloid in the display file.

; Attempt to move the Bubloid down.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      BC,$0202                                ; There are 2 lines and 2 columns to test.

        LD      A,(L6409)                               ; Fetch the player flags.
        BIT     5,A                                     ; Flying via a vulture?
        JR      Z,L7378                                 ; Jump ahead if not.

        INC     C                                       ; Increment the column count to 3.

L7378:  POP     DE                                      ; Retrieve the original location of the Bubloid in the display file.
        CALL    L7542                                   ; Could the Bubloid be moved down?
        JR      C,L73E4                                 ; Jump if not.

; The Bubloid could be moved down.
; Restore the background graphics into the lines that the Bubloid has moved out of.

        LD      C,$02                                   ; The Bubloid is 2 columns wide.
        LD      DE,L6029                                ; The left hand column of the background buffer for the Bubloid.

L7383:  PUSH    HL                                      ; Save the new address within the display file of the Bubloid.
        LD      B,$02                                   ; There are 2 lines to restore.
        CALL    L64DA                                   ; Copy the background graphics into the display file.
        POP     HL                                      ; Retrieve the new address within the display file of the Bubloid.
        INC     HL                                      ; Advance to the right hand column.
        LD      DE,L6039                                ; The right hand column of the background buffer for the Bubloid.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L7383                                ; Repeat for both columns.

; Shift the background buffer contents up 2 lines.

        LD      DE,L6029                                ; Point at the start of the left hand column of the background buffer for the Bubloid.
        LD      HL,L6029+$0002                          ; Point 2 lines into the left hand column of the background buffer for the Bubloid.
        LD      BC,$000E                                ; There are 14 lines to shift.
        LDIR                                            ; Shift the left hand background buffer up 2 lines.

        LD      DE,L6039                                ; Point at the start of the right hand column of the background buffer for the Bubloid.
        LD      HL,L6039+$0002                          ; Point 2 lines into the right hand column of the background buffer for the Bubloid.
        LD      BC,$000E                                ; There are 14 lines to shift.
        LDIR                                            ; Shift the right hand background buffer up 2 lines.

; Save the new background graphics where the Bubloid has moved to.

        LD      HL,(L6411)                              ; Fetch the display file address of the Bubloid.
        LD      DE,$01CE                                ; Offset of 14 lines.
        ADD     HL,DE                                   ; Point to the last 2 lines of the new Bubloid location.
        LD      C,$02                                   ; The Bubloid is 2 columns wide.
        LD      DE,L6029+$000E                          ; Point to the bottom 2 lines of the left hand column of the background buffer for the Bubloid.

L73B3:  CALL    L6D5E                                   ; Copy 2 lines of the background at the column position into the buffer.
        LD      DE,L6039+$000E                          ; Point to the bottom 2 lines of the right hand column of the background buffer for the Bubloid.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L73B3                                ; Repeat for both columns.

        JP      L74F6                                   ; Jump to draw the Bubloid.

; Move the Bubloid up
; -------------------

L73BF:  LD      HL,(L6411)                              ; Fetch the location of the Bubloid in the display file.
        PUSH    HL                                      ; Save the location of the Bubloid in the display file.
        LD      DE,$0042                                ; Offset of 2 lines.
        AND     A
        SBC     HL,DE                                   ; Move the Bubloid up 2 lines.
        LD      (L6411),HL                              ; Store the new location of the Bubloid in the display file.

; Attempt to move the Bubloid up.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      DE,$01CE                                ; Offset of 14 lines.
        ADD     HL,DE                                   ; Point to the last 2 lines of the new Bubloid location.
        LD      BC,$0202                                ; There are 2 lines and 2 columns to test.

        LD      A,(L6409)                               ; Fetch the player flags.
        BIT     5,A                                     ; Flying via a vulture?
        JR      Z,L73DE                                 ; Jump ahead if not.

        INC     C                                       ; Increment to the column count to 3.

L73DE:  POP     DE                                      ; Retrieve the location of the Bubloid in the display file.
        CALL    L7542                                   ; Could the Bubloid be moved up?
        JR      NC,L73EE                                ; Jump if so.

; The Bubloid could not be moved up.

L73E4:  LD      HL,L641E                                ; Point at the store holding the Bubloid movement flags.
        LD      A,(HL)                                  ; Fetch the movement flags.
        XOR     $02                                     ; Reverse the Bubloid's vertical direction.
        LD      (HL),A                                  ; Store the updated movement flags.

        JP      L753F                                   ; Jump to draw the wave.

; The Bubloid could be moved down.

L73EE:  LD      DE,$01CE                                ; Offset of 14 lines.
        ADD     HL,DE                                   ; Point to the last 2 lines of the new Bubloid location.
        LD      C,$02                                   ; The Bubloid is 2 columns wide.
        LD      DE,L6029+$000E                          ; Point to the bottom 2 lines of the left hand column of the background buffer for the Bubloid.

L73F7:  PUSH    HL                                      ; Save the new address within the display file of the Bubloid.
        LD      B,$02                                   ; There are 2 lines to 
        CALL    L64DA                                   ; Copy the background graphics into the display file.
        POP     HL                                      ; Retrieve the new address within the display file of the Bubloid.
        INC     HL                                      ; Advance to the right hand column.
        LD      DE,L6039+$000E                          ; Point to the bottom 2 lines of the right hand column of the background buffer for the Bubloid.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L73F7                                ; Repeat for both columns.

; Shift the background buffer contents down 2 lines.

        LD      DE,L6029+$000F                          ; Point at the last line in the left hand column of the background buffer for the Bubloid.
        LD      HL,L6029+$000D                          ; Point at the 3 lines from the bottom of the left hand column of the background buffer for the Bubloid.
        LD      BC,$000E                                ; There are 14 lines to copy.
        LDDR                                            ; Copy the graphic.    

        LD      DE,L6039+$000F                          ; Point at the last line in the right hand column of the background buffer for the Bubloid.
        LD      HL,L6039+$000D                          ; Point at the 3 lines from the bottom of the right hand column of the background buffer for the Bubloid.
        LD      BC,$000E                                ; There are 14 lines to copy.
        LDDR                                            ; Copy the graphic.    

; Restore the background graphics into the lines that the Bubloid has moved out of.

        LD      HL,(L6411)                              ; Fetch the display file address of the Bubloid.
        LD      C,$02                                   ; The width of the Bubloid.
        LD      DE,L6029                                ; Point to the left column in the background buffer for the Bubloid.

L7423:  CALL    L6D5E                                   ; Copy 2 lines of the background at the column position into the buffer.
        LD      DE,L6039                                ; Point to the right column in the background buffer for the Bubloid.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L7423                                ; Repeat for both columns.

        JP      L74F6                                   ; Jump to draw the Bubloid.

; Check whether it is time to process the Bubloid horizontal movement
; -------------------------------------------------------------------

L742F:  LD      HL,L6424                                ; Point at the store holding the Bubloid horizontal movement timer.
        LD      A,(FRAMES)
        CP      (HL)                                    ; Is it time to move the Bubloid horizontally?
        JP      P,L76B1                                 ; Jump if not to draw the wave.

; It is time to process the Bubloid horizontal movement.

        DEC     HL                                      ; Point at the store holding the Bubloid horizontal speed.
        SUB     (HL)                                    ; Determine the FRAMES number of the next horizontal movement.
        INC     HL                                      ; Point back at the Bubloid horizontal movement timer.
        LD      (HL),A                                  ; Store the updated timer value.

; Determine Bubloid's horizontal speed
; ------------------------------------

        CALL    L75B8                                   ; Determine the horizontal and vertical directions of the player from the Bubloid.

; A=%vh000000 where h=1 if the player is to the right of the Bubloid and v=1 if the player is higher up than the Bubloid.

        LD      HL,L641E                                ; Point at the store holding the Bubloid movement flags.
        BIT     0,(HL)                                  ; Is the Bubloid moving right?
        LD      HL,L6422                                ; Point at the store holding the Bubloid horizontal acceleration.
        JR      NZ,L7450                                ; Jump if so.

; The Bubloid is moving left.

        BIT     6,A                                     ; Is the player to the right of the Bubloid?
        JR      Z,L7454                                 ; Jump if not.

; The Bubloid is moving left. The player is to the right of the Bubloid.

        JR      L7459                                   ; Jump ahead to continue.

; The Bubloid is moving right.

L7450:  BIT     6,A                                     ; Is the player to the right of the Bubloid?
        JR      Z,L7459                                 ; Jump if not.

; The Bubloid is moving left. The player is to the left of the Bubloid.
; The Bubloid is moving right. The player is to the right of the Bubloid.

L7454:  CALL    L7594                                   ; Increase the speed of the Bubloid towards the player.
        JR      L745E                                   ; Jump ahead to continue.

; The Bubloid is moving left. The player is to the right of the Bubloid.
; The Bubloid is moving right. The player is to the left of the Bubloid.

L7459:  LD      C,$01                                   ; Bit value for the horizontal flag in the Bubloid movement flags.
        CALL    L75A2                                   ; Decrease the speed of the Bubloid away from the player until able to reverse direction.

; The Bubloid's horizontal speed has been updated.

L745E:  LD      HL,L641E                                ; Point at the store holding the Bubloid movement flags.
        BIT     0,(HL)                                  ; Is the Bubloid moving right?
        JR      NZ,L74AC                                ; Jump if so.

; The Bubloid is facing left. Is it time to move?

        LD      HL,L6410                                ; Point to the store holding the Bubloid frame number.
        INC     (HL)                                    ; Increment the frame number.
        LD      A,$06
        CP      (HL)                                    ; Has the last frame been exceeded?
        JP      NZ,L74F6                                ; Jump if not to draw the Bubloid.

        LD      (HL),$01                                ; Wrap back to the first frame.

; Move the Bubloid left
; ---------------------
; The Bubloid is only moved after cycle through all left facing frames.

        LD      HL,(L6411)                              ; Fetch the display file address of the Bubloid.
        PUSH    HL                                      ; Save the original display file address of the Bubloid.
        DEC     HL                                      ; Move the Bubloid to the left.
        LD      (L6411),HL                              ; Store the new display file address of the bubloid.

; Attempt to move the Bubloid left.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      BC,$1002                                ; There are 16 lines and 2 columns to test.

        LD      A,(L6409)                               ; Fetch the player flags.
        BIT     5,A                                     ; Flying via a vulture?
        JR      Z,L7487                                 ; Jump ahead if not.

        INC     C                                       ; Increment the column count to 3.

L7487:  POP     DE                                      ; Retrieve the original location of the Bubloid in the display file.
        CALL    L7542                                   ; Could the Bubloid be moved left?
        JR      C,L74D0                                 ; Jump if not.

; The Bubloid could be moved left.
; Restore the background graphics into the column that the Bubloid has moved out of.

        INC     HL                                      ; Point at the original right hand column of the Bubloid in the display file.
        LD      B,$10                                   ; There are 16 lines to restore.
        LD      DE,L6039                                ; The right hand column of the background buffer for the Bubloid.
        CALL    L64DA                                   ; Copy the background graphics into the display file.

; Shift the background buffer contents right 1 column.

        LD      BC,$0010                                ; There are 16 lines to shift.
        LD      DE,L6039                                ; The right hand column of the background buffer for the Bubloid.
        LD      HL,L6029                                ; The left hand column of the background buffer for the Bubloid.
        LDIR                                            ; Shift the background buffer contents.

; Save the new background graphics where the Bubloid has moved to.

        LD      HL,(L6411)                              ; Fetch the display file address of the Bubloid.
        LD      DE,L6029                                ; The left hand column of the background buffer for the Bubloid.
        CALL    L6D50                                   ; Save the background at the new position into the buffer.

        JR      L74F6                                   ; Jump to draw the Bubloid.

; The Bubloid is facing right. Is it time to move?

L74AC:  LD      HL,L6410                                ; Point to the store holding the Bubloid frame number.
        DEC     (HL)                                    ; Decrement the frame number.
        JR      NZ,L74F6                                ; Jump if the last frame has not been exceeded to draw the Bubloid.

        LD      (HL),$05                                ; Wrap back to the first frame.

; Move the Bubloid right
; ----------------------

        LD      HL,(L6411)                              ; Fetch the display file address of the Bubloid.
        PUSH    HL                                      ; Save the original display file address of the Bubloid.
        INC     HL                                      ; Move the Bubloid to the right.
        LD      (L6411),HL                              ; Store the new display file address of the bubloid.

; Attempt to move the Bubloid right.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      BC,$1002                                ; There are 16 lines and 2 columns to test.

        LD      A,(L6409)                               ; Fetch the player flags.
        BIT     5,A                                     ; Flying via a vulture?
        JR      Z,L74CA                                 ; Jump ahead if not.

        INC     C                                       ; Increment the column count to 3.

L74CA:  POP     DE                                      ; Retrieve the original location of the Bubloid in the display file.
        CALL    L7542                                   ; Could the Bubloid be moved right?
        JR      NC,L74D9                                ; Jump if so.

; The Bubloid could not be moved left or right.

L74D0:  LD      HL,L641E                                ; Point at the store holding the Bubloid movement flags.
        LD      A,(HL)                                  ; Fetch the movement flags.
        XOR     $01                                     ; Reverse the horizontal direction.
        LD      (HL),A                                  ; Store the updated movement flags.

        JR      L753F                                   ; Jump to draw the wave.

; The Bubloid could be moved right.
; Restore the background graphics into the column that the Bubloid has moved out of.

L74D9:  LD      B,$10                                   ; There are 16 lines to restore.
        LD      DE,L6029                                ; The left hand column of the background buffer for the Bubloid.
        CALL    L64DA                                   ; Copy the background graphics into the display file.

; Shift the background buffer contents left 1 column.

        LD      BC,$0010                                ; There are 16 lines to shift.
        LD      DE,L6029                                ; The left hand column of the background buffer for the Bubloid.
        LD      HL,L6039                                ; The right hand column of the background buffer for the Bubloid.
        LDIR                                            ; Shift the background buffer contents.

; Save the new background graphics where the Bubloid has moved to.

        LD      HL,(L6411)                              ; Fetch the new display file address of the Bubloid.
        INC     HL                                      ; Point at the right hand column of the Bubloid.
        LD      DE,L6039                                ; The right hand column of the background buffer for the Bubloid.
        CALL    L6D50                                   ; Save the background at the new position into the buffer.

; Draw the Bubloid
; ----------------
; The Bubloid has been moved up, down, left or right.

L74F6:  LD      HL,L6410                                ; Point to the store holding the Bubloid frame number.
        LD      B,(HL)                                  ; Fetch the frame number.
        LD      A,B                                     ; Note the frame number.

L74FB:  LD      HL,L5E79                                ; The base address of the active Bubloid graphics set (index is 0 based). This address is dynamically changed by the program.
        LD      DE,$0020                                ; The size of each graphic.

L7501:  ADD     HL,DE                                   ; Advance to the next graphic.
        DJNZ    L7501                                   ; Repeat until the required graphic has been reached.

        EX      DE,HL                                   ; DE=Address of the Bubloid graphic to draw.

; Draw the Bubloid in the display file.

        LD      HL,(L6411)                              ; Fetch the display file address of the Bubloid.
        CP      $01                                     ; Displaying the single column Bubloid graphic?
        JR      NZ,L751D                                ; Jump ahead if not.

; There is a single column forming the Bubloid graphic.
; The left column is restored from the left hand background buffer; the Bubloid is drawn in the right column.

        LD      HL,$0010                                ; The size of a column of the Bubloid graphic.
        ADD     HL,DE                                   ; Advance by a column.
        EX      DE,HL                                   ; DE=Address of the Bubloid graphic.

        LD      HL,(L6411)                              ; Fetch the display file address of the Bubloid.
        INC     HL                                      ; Advance to the column to the right.
        PUSH    HL                                      ; Save the address of the column to the right.
        DEC     HL                                      ; Point back to the display file address of the Bubloid.
        PUSH    DE                                      ; Save the address of the Bubloid graphic.
        LD      DE,L6029                                ; Point to the left hand background buffer for the Bubloid.                             

        JR      L7527                                   ; Jump ahead to continue.

; Not displaying the single column Bubloid graphic.

L751D:  CP      $05                                     ; Displaying the final single column Bubloid graphic?
        JR      NZ,L7532                                ; Jump ahead if not.

; There is a single column forming the Bubloid graphic.
; The left column is restored from the right hand background buffer; the Bubloid is drawn in the left column.

        PUSH    HL                                      ; Save the display file address of the Bubloid.
        INC     HL                                      ; Advance to the column to the right.
        PUSH    DE                                      ; Save the address of the Bubloid graphic.
        LD      DE,L6039                                ; Point to the right hand background buffer for the Bubloid.                            

L7527:  LD      B,$10                                   ; There are 16 lines to copy.
        CALL    L64DA                                   ; Restore the background for the right column into the display file.

        POP     DE                                      ; Retrieve the address of the Bubloid graphic.
        POP     HL                                      ; Retrieve the display address of the Bubloid.
        LD      C,$01                                   ; There is a single column to draw.
        JR      L7534                                   ; Jump ahead to continue.

; There are 2 columns forming the Bubloid graphic.

L7532:  LD      C,$02                                   ; There are 2 columns to draw.

; Joins here to draw a 1 column or a 2 column wide Bubloid.

L7534:  PUSH    HL                                      ; Save the display file address of the Bubloid.
        LD      B,$10                                   ; There are 16 lines to draw.
        CALL    L64DA                                   ; Copy a block into the display file.
        POP     HL                                      ; Retrieve the display file address of the Bubloid.
        INC     HL                                      ; Advance to the right.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L7534                                ; Repeat for all columns.

L753F:  JP      L76B1                                   ; Jump to draw the wave.

; ----------------------------------------
; Determine Whether Bubloid Could Be Moved
; ----------------------------------------
; Entry: HL=Display file address of the player.
;        DE=Original display file address of the Bubloid.
;        B=Number of lines to test.
;        C=Number of columns to test.
; Exit : Carry flag set if the Bubloid could not be moved.
;        HL=New display file address of the Bubloid.

L7542:  PUSH    DE                                      ; Save the original display file address of the Bubloid.

        CALL    L6CA8                                   ; Check whether the player has been caught by the Bubloid.
        JR      NC,L756A                                ; Jump ahead if the player has not been caught.

; The player has been caught by the Bubloid.

        POP     HL                                      ; Retrieve the original display file address of the Bubloid.
        LD      (L6411),HL                              ; Store as the display file location of the Bubloid.

        LD      A,(L6409)                               ; Fetch the player flags.
        BIT     6,A                                     ; Flying via a rocket pack?
        JR      Z,L755C                                 ; Jump ahead if not.

; The player has been caught by the Bubloid while flying via a rocket pack.

        CALL    L7558                                   ; Call below to allow the display file address of the Bubloid to be stacked.
        JR      L7568                                   ; Jump ahead to continue.

L7558:  PUSH    HL                                      ; Save the original display file address of the Bubloid.
        JP      L7017                                   ; Transfer the graphics from the rocket pack background buffer to the background buffer for the Caught by Bubloid.

; The player has been caught by the Bubloid and is not flying via a rocket pack.

L755C:  BIT     5,A                                     ; Flying via a vulture?
        JR      Z,L7565                                 ; Jump ahead if not.

; The player has been caught by the Bubloid while flying via a vulture.

        CALL    L72F3                                   ; Transfer the graphics from the vulture background buffer to the background buffer for the Caught by Bubloid.
        JR      L7568                                   ; Jump ahead to continue.

; The player has been caught by the Bubloid and is not flying via a rocket pack or a vulture.

L7565:  CALL    L6CFD                                   ; Configure caught by the Bubloid.

; The player has been caught by the Bubloid - all paths join here.

L7568:  SCF                                             ; Signal the Bubloid could not be moved.
        RET     

; The player has not been caught by the Bubloid.

L756A:  LD      HL,(L6411)                              ; Fetch the new display file location of the Bubloid.

        LD      A,(L641E)                               ; Fetch the Bubloid movement flags.
        BIT     0,A                                     ; Is the Bubloid moving right?
        JR      Z,L7575                                 ; Jump ahead if not.

        INC     HL                                      ; Point at the right hand column of the Bubloid.

L7575:  LD      A,(HL)
        CP      $C9                                     ; Has the Bubloid moved off the edge of the screen?
        JR      Z,L758E                                 ; Jump if so.

; The player has not been caught by the Bubloid. The Bubloid has not moved off the edge of the screen.

        EX      DE,HL                                   ; DE=Address of the Bubloid.
        LD      HL,L418A                                ; Line 8 column 0.
        AND     A
        SBC     HL,DE                                   ; Is the Bubloid between line 0 and 7, where the score is displayed?
        JR      NC,L758E                                ; Jump if the Bubloid is between line 0 and 7.

; The Bubloid has not moved into the lines occupied by the score details.

        LD      HL,L5564+$001F                          ; Line 162 column 31.
        AND     A
        SBC     HL,DE                                   ; Has the Bubloid moved into the sea?
        JR      C,L758E                                 ; Jump if not.

; The Bubloid has not fallen into the sea.

        POP     HL                                      ; Retrieve the original display file address of the Bubloid.

        AND     A                                       ; Signal the Bubloid could be moved.
        RET     

; The player has not been caught by the Bubloid. The Bubloid has moved off the edge of the screen, into the score details or into the sea.

L758E:  POP     HL                                      ; Retrieve the original display file address of the Bubloid.
        LD      (L6411),HL                              ; Restore it as the display file location of the Bubloid.

        SCF                                             ; Signal the Bubloid could not be moved.
        RET     

; ----------------------
; Increase Bubloid Speed
; ----------------------
; The Bubloid is moving in the direction of the player so increase its speed.
; The Bubloid takes accelerates faster with each new level, resetting back to the slowest rate on level 4 and then on level 1.
; Entry: HL=Points at the store holding the Bubloid's active horizontal or vertical acceleration timer.

L7594:  DEC     (HL)                                    ; Decrement the Bubloid's acceleration timer.
        RET     NZ                                      ; Return if it hasn't reached zero.

; The Bubloid acceleration timer has expired so to increase the Bubloid's speed.

        LD      A,(L640D)                               ; Fetch the Bubloid acceleration rate for this level.
        LD      (HL),A                                  ; Reset the acceleration timer.

; Increase the Bubloid's speed.

        INC     HL                                      ; Point at the store holding the Bubloid's speed.
        DEC     (HL)                                    ; Increase the speed.

        LD      A,$01
        CP      (HL)                                    ; Has the maximum been exceeded?
        RET     NZ                                      ; Return if not.

        INC     (HL)                                    ; Cap to the maximum speed.
        RET     

; -------------------------------------------------
; Decrease Bubloid Speed Before Reversing Direction
; -------------------------------------------------
; The Bubloid is moving away from the player so decrease its speed before changing direction.
; Entry: HL=Points at the store holding the Bubloid's active horizontal or vertical acceleration timer.
;        C=$01 if the Bubloid is to reverse horizontal direction, $02 if the Bubloid is to reverse vertical direction.

L75A2:  DEC     (HL)                                    ; Decrement the acceleration timer.
        RET     NZ                                      ; Return if it hasn't reached zero.

; The Bubloid acceleration timer has expired so to decrease the Bubloid's speed.

        LD      A,(L640D)                               ; Fetch the Bubloid acceleration rate for this level.
        LD      (HL),A                                  ; Reset the acceleration timer.

; Decrease the Bubloid's speed.

        INC     HL                                      ; Point at the store holding the Bubloid's speed.
        INC     (HL)
        INC     (HL)                                    ; Decrease the speed.

        LD      A,$0F
        CP      (HL)                                    ; Has the minimum been exceeded?
        RET     NC                                      ; Return if not.

        DEC     (HL)                                    ; Cap to the minimum speed.
        DEC     (HL)

; Reverse the Bubloid's direction.

        LD      HL,L641E                                ; Point at the store holding the Bubloid movement flags.
        LD      A,(HL)                                  ; Fetch the movement flags.
        XOR     C                                       ; Reverse the specified direction.
        LD      (HL),A                                  ; Store the updated movement flags.
        RET     

; ---------------------------------------------------------------------------
; Determine Horizontal and Vertical Directions of the Player from the Bubloid
; ---------------------------------------------------------------------------
; Exit: A=%vh000000 where h=1 if the player is to the right of the Bubloid and v=1 if the player is higher up than the Bubloid.

L75B8:  LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      DE,(L6411)                              ; Fetch the address within the display file of the Bubloid.
        LD      BC,$0000                                ; B holds the player column count, C holds the Bubloid column count.

; Enter a loop to determine how close the Bubloid and the player are to the right edge of the screen.

L75C2:  INC     HL                                      ; Point to the location right of the player.
        INC     DE                                      ; Point to the location right of the Bubloid.

        LD      A,(HL)
        CP      $C9                                     ; Is the player at the edge of the display file?
        JR      Z,L75CA                                 ; Jump ahead if so.

        INC     B                                       ; Increment the player column count.

L75CA:  INC     C                                       ; Increment the Bubloid column count.

        LD      A,(DE)
        CP      $C9                                     ; Is the Bubloid at the edge of the display file?
        JR      NZ,L75C2                                ; Jump back if not.

; B=Number of columns the player was shifted right.
; C=Number of columns the Bubloid was shifted right.

; If the player were at the same column or to the left of the Bubloid then the two counts will be the same.
; If the player were to the right of the Bubloid then the player count will be one less than the Bubloid count.

        LD      A,B                                     ; Fetch the player column count.
        LD      B,$00                                   ; Used to holds a flag indicating whether the player is to the left or right of the Bubloid.
        CP      C                                       ; Carry flag will be set if the player is to the right of the Bubloid.
        RR      B                                       ; Shift the status into the B register.

        AND     A
        SBC     HL,DE                                   ; Carry flag will be set if the player is on a line above the Bubloid.
        RR      B                                       ; Shift the status into the B register.

        LD      A,B                                     ; Return the status flags in the A register.
        RET     

; -------------------
; End of BASIC Line 7
; -------------------

L75DD:  DEFB _NEWLINE

; ------------
; BASIC Line 8
; ------------

L75DE:  DEFB 8 >> 8
        DEFB 8 & $FF
        DEFW L77F2 - 1 - $
        DEFB _REM

; --------------------------
; Draw Player Caught Graphic
; --------------------------
; The player has been caught by the Bubloid and is being dragged down to the sea.

L75E3:  LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      DE,$0042                                ; Offset of 2 lines.
        PUSH    HL                                      ; Save the original display file location.
        ADD     HL,DE                                   ; Advance the player 2 lines down.

        NOP                                             ; [Redundant]   
        NOP     

        LD      (L6401),HL                              ; Save the new address of the player within the display file.

        LD      DE,L5564+$001F                          ; Line 162 column 31.
        AND     A
        SBC     HL,DE                                   ; Has the player fallen into the sea?
        JR      C,L7600                                 ; Jump if not.

; The player has fallen into the sea.

        LD      HL,L6408                                ; Point to the store holding the player frame number.
        LD      (HL),$05                                ; Set such that the initial splash graphic frame will be shown.
        INC     HL                                      ; Point to the flags.
        SET     7,(HL)                                  ; Signal fallen into the sea, i.e. the splash graphic needs to be drawn

; Joins here when the player has not fallen into the sea.

L7600:  POP     HL                                      ; Retrieve the original display file location of the player.

; Restore the background for the 2 lines the player no longer occupies.

        LD      C,$02                                   ; There are 2 columns to draw.
        LD      DE,L6089                                ; Point to the graphic store holding the background for the left hand column.

L7606:  PUSH    HL                                      ; Save the display file location.

        LD      B,$02                                   ; There are 2 lines.
        CALL    L64DA                                   ; Copy the graphic into the display file.

        POP     HL                                      ; Retrieve the display file location.
        INC     HL                                      ; Advance to the next column within the display file.

        LD      DE,L6119                                ; Point to the graphic store holding the background for the right hand column.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L7606                                ; Repeat for both columns.

; Shift the background buffers for the Bubloid up 2 lines.

        LD      DE,L6089                                ; Shift the left hand Bubloid background column up 2 lines.
        LD      HL,L6089+$0002
        LD      BC,$000E                                ; There are 14 lines to shift.
        LDIR    

        LD      DE,L6119                                ; Shift the right hand Bubloid background column up 2 lines.
        LD      HL,L6119+$0002
        LD      BC,$000E                                ; There are 14 lines to shift.
        LDIR    

; Fill the bottom 2 lines of the Bubloid background buffers with the graphics from the display file that the player will now occupy.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      DE,$01CE                                ; Offset of 14 lines.
        ADD     HL,DE                                   ; Point 14 lines down.

        LD      C,$02                                   ; There are 2 columns to save.
        LD      DE,L6089+$000E                          ; Address within the left hand column background buffer.

L7636:  CALL    L6D5E                                   ; Copy 2 lines of the display file at the column position into the background buffer.

        LD      DE,L6119+$000E                          ; Address within the right hand column background buffer.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L7636                                ; Repeat for both columns.

; Decrement the frame number.

        LD      HL,L6408                                ; Point at the player frame number.
        DEC     (HL)                                    ; Decrement the frame number.
        JR      NZ,L7647                                ; Jump ahead if it has not reached 0.

        LD      (HL),$05                                ; Wrap back to the first caught frame.

L7647:  LD      B,(HL)                                  ; Fetch the frame number.

; Locate the new Player Caught graphic to display.

        LD      HL,L6089-$0020                          ; Address of the Player Caught graphics (index is 0 based).
        LD      DE,$0020                                ; The size of each graphic.

L764E:  ADD     HL,DE                                   ; Offset to the next graphic.
        DJNZ    L764E                                   ; Repeat until the required graphic has been reached.

        EX      DE,HL                                   ; DE=Address of the Player Caught graphic.

; Draw the Player Caught graphic into the display file.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      C,$02                                   ; There are 2 columns to display.

L7657:  LD      B,$10                                   ; There are 16 lines to display.
        PUSH    HL                                      ; Save the display file address.
        CALL    L64DA                                   ; Copy a block into the display file.
        POP     HL                                      ; Retrieve the display file address.
        INC     HL                                      ; Advance to the right hand column.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L7657                                ; Repeat for both columns.

        JP      L76B1                                   ; Jump to draw the wave.

; ----------------------
; Display Splash Graphic
; ----------------------

L7665:  LD      HL,L6408                                ; Point at the player frame number.
        DEC     (HL)                                    ; Decrement the frame number.
        JR      Z,L7689                                 ; Jump ahead if all splash frames have been shown.

; There are further splash frames to show.

        LD      B,(HL)                                  ; Fetch the frame number.

        LD      HL,L622E-$0020                          ; Address of the splash graphics (index is 0 based).
        LD      DE,$0020                                ; Size of each graphic.

L7672:  ADD     HL,DE                                   ; Advance to the next splash graphic.
        DJNZ    L7672                                   ; Repeat until the required splash graphic has been found.

        EX      DE,HL                                   ; DE=Address of the splash graphic.

; Draw the splash graphic.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        LD      C,$02                                   ; The splash graphic is 2 columns wide.

L767B:  LD      B,$10                                   ; The splash graphic is 16 lines tall.
        PUSH    HL                                      ; Save the address within the display file.
        CALL    L64DA                                   ; Copy a block of the graphic into the display file.
        POP     HL                                      ; Retrieve the address within the display file.

        INC     HL                                      ; Advance to the next column within the display file.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L767B                                ; Repeat for both columns.

        JP      L76B1                                   ; Jump to draw the wave.

; Reset Display and Restart Level
; -------------------------------
; Determine how many columns the player graphic spans.

L7689:  LD      C,$02                                   ; Assume the player occupies 2 columns.
        LD      A,(L6409)                               ; Fetch the player flags.
        BIT     5,A                                     ; Is the player flying via the vulture and so occupies 3 columns?
        JR      Z,L7693                                 ; Jump ahead if not.

        INC     C                                       ; Increment the column count to 3.

; Blank out the splash, which results in a 2 or 3 column wide gap in the wave.

L7693:  LD      HL,(L6401)                              ; Fetch the address within the display file of the player.

L7696:  LD      B,$10                                   ; There 16 lines to draw.
        LD      DE,L5DA9                                ; Use the first column of the Vulture Flying Left #1 graphic since it is completely blank.
        PUSH    HL                                      ; Save the display file address.
        CALL    L64DA                                   ; Copy a block into the display file.
        POP     HL                                      ; Retrieve the display file address.
        INC     HL                                      ; Advance to the column to the right.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L7696                                ; Repeat for all columns.

; If required, tidy up after being caught by the Bubloid.

        LD      A,(L6409)                               ; Fetch the player flags.
        BIT     4,A                                     ; Has the player been caught by the Bubloid?
        JR      NZ,L76AE                                ; Jump ahead if not.

        CALL    L6CFD                                   ; Restore the background graphics for the Bubloid.

L76AE:  JP      L7857                                   ; Jump to restart the level.

; -------------
; Draw the wave
; -------------
; Comes here after handling movement of the Bubloid.

L76B1:  LD      HL,L640A                                ; Point to the wave timer.
        LD      A,(FRAMES)                              ; Fetch the low byte of the FRAMES system variable, which decrements every 50th of a second.
        CP      (HL)                                    ; Is it time to toggle the wave?
        JP      P,L68CF                                 ; Jump if not to handle movement of the player.

; It is time to toggle the wave.

        SUB     $0A                                     ; Determine the next threshold when the wave will be toggled again.
        LD      (HL),A                                  ; Store the new value.

; Draw the wave along the bottom of the screen. The wave is not drawn in the last column of the display because the following code requires
; an odd number of columns for the toggling of L640B to work correctly.

        LD      C,$1F                                   ; There are 31 columns to fill.
        LD      HL,L5732                                ; Line 176 column 0.

L76C3:  PUSH    HL                                      ; Save the address within the display file.

        LD      A,(L640B)                               ; Fetch the wave phase indicator.
        RLC     A                                       ; Rotate the value to the left. This results in alternate values of %10101010 and %01010101, and the carry flag toggle upon each cycle.
        LD      (L640B),A                               ; Save the new phase indicator value.
        JR      NC,L76D3                                ; Jump if phase 2.

; Draw wave in phase 1.

        LD      DE,L6216                                ; Point to the wave graphic (column 1).
        JR      L76D6                                   ; Jump ahead to continue.

; Draw wave in phase 2.

L76D3:  LD      DE,L621A                                ; Point to the wave graphic (column 2).

L76D6:  LD      B,$04                                   ; There are 4 lines to draw.
        CALL    L64DA                                   ; Copy a column of wave graphic into the display file.

        POP     HL                                      ; Retrieve the address within the display file.
        INC     HL                                      ; Advance to the next column within the display file.
        DEC     C                                       ; Decrement the column count.
        JR      NZ,L76C3                                ; Loop back to the draw the next part of the wave.

; Update the bonus.

        CALL    L77CE                                   ; Decrement the bonus and update the value in the display file.

; Is the player flying?

        LD      A,(L6409)                               ; Fetch the player flags.
        AND     $60                                     ; Flying via a rocket pack or a vulture?
        JR      Z,L7745                                 ; Jump ahead if not to continue the game loop.

; The player is flying via a rocket pack or a vulture.

        LD      HL,L688C                                ; Point at the store holding the fuel usage timer.
        DEC     (HL)                                    ; Decrement it.
        JR      NZ,L7745                                ; Jump ahead if not expired to check whether Pause has been requested.

        LD      (HL),$0A                                ; Reset the fuel usage timer.

; Decrement the fuel level.

        LD      HL,L688D                                ; Point to the location holding the fuel level.
        DEC     (HL)                                    ; Decrement the fuel level.
        JR      NZ,L7741                                ; Jump ahead if the fuel has not completely run out.

; The fuel has completely run out and so the rocket pack or vulture will disappear. The graphics stored in the
; background buffer for the vulture or rocket pack are transferred as the background graphics for the player.

        PUSH    HL                                      ; Save the fuel level location.

        BIT     5,A                                     ; Is the player flying using the vulture or the rocket pack?
        JR      Z,L7722                                 ; Jump ahead if a rocket pack.

; The player is flying using the vulture. Set the background graphics for the player from the background graphics
; stored for the flying using the vulture graphics.

; Copy the display file for column 3 into the vulture background buffer.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        INC     HL
        INC     HL                                      ; Point 2 columns to the right.
        LD      DE,L5E99                                ; The vulture column 3 graphic background buffer.
        LD      B,$10                                   ; There are 16 lines.
        CALL    L64DA                                   ; Copy a block into the display file.

; Use the background buffer graphics for the vulture as the background graphics for the player.

        LD      HL,L5E89                                ; The vulture column 2 graphic background buffer.
        LD      DE,L5A39                                ; The right hand background buffer for the player.
        LD      BC,$0010                                ; There are 16 lines.
        LDIR                                            ; Copy the background graphics.

        LD      HL,L5E79                                ; The vulture column 1 graphic background buffer.
        LD      DE,L5949                                ; The left hand background buffer for the player.
        LD      BC,$0010                                ; There are 16 lines.
        LDIR                                            ; Copy the background graphics.

        JR      L7738                                   ; Jump ahead to continue.

; The player is flying using the rocket pack. Set the background graphics for the player from the background graphics
; stored for the flying using the rocket pack graphics.

L7722:  LD      HL,L5AF5                                ; The rocket pack right graphic background buffer.
        LD      DE,L5A39                                ; The right hand background buffer for the player.
        LD      BC,$0010                                ; There are 16 lines.
        LDIR                                            ; Copy the background graphics.

        LD      HL,L5B09                                ; The rocket pack left graphic background buffer.
        LD      DE,L5949                                ; The left hand background buffer for the player.
        LD      BC,$0010                                ; There are 16 lines.
        LDIR                                            ; Copy the background graphics.

L7738:  POP     HL                                      ; Retrieve the fuel level location.

; Mark the player as no longer flying.

        LD      A,(L6409)                               ; Fetch the player flags.
        AND     $9F                                     ; Discard the rocket pack and vulture flying flags.
        LD      (L6409),A                               ; Store the updated flags.

; Draw the fuel level
; -------------------
; Joins here when the fuel level has not completely run out.

L7741:  LD      C,(HL)                                  ; Fetch the amount of fuel remaining.
        CALL    L68AD                                   ; Draw the fuel gauge.

L7745:  JP      L7876                                   ; Jump to check whether SHIFT+H is being pressed to pause the game.

; Enter a loop to decrement the bonus and add its value to the score.
; Joins here after all diamonds have been collected.

L7748:  CALL    L77CE                                   ; Decrement the bonus and update the value in the display file.
        JR      C,L776D                                 ; Jump if the bonus has reached "0000".

; The bonus has not reached "0000" so increment the score by 1.

        LD      A,$01                                   ; The score increment.
        LD      B,$07                                   ; There are potentially 7 digits that need updating.
        LD      HL,L64B1                                ; Point to the last digit of the score.
        CALL    L6D6F                                   ; Add the increment to the score.

; Determine how many digits of the score were affected by the increment.

        LD      A,$08
        SUB     B                                       ; The number of digits affected by incrementing the score.
        CP      $08                                     ; Were all digits affected?
        JR      NZ,L775F                                ; Jump ahead if not.

        DEC     A                                       ; A=$07 as all digits need to be redrawn.

; Updated the score in the display file.

L775F:  LD      HL,L40C4+$0007                          ; Line 2 column 7.
        LD      BC,L64B1                                ; Point to the last digit of the score.
        
        NOP                                             ; [Redundant]
        NOP     

        CALL    L6515                                   ; Draw the score digits that were changed.

        NOP                                             ; [Redundant]     

        JR      L7748                                   ; Jump back to continue decrementing the bonus until it reaches "0000".

; The bonus has reached "0000". Check the score to determine how many extra lives to award.

L776D:  LD      HL,L64AB+$0002                          ; Point to the 10000s digit of the score.
        LD      A,(L640E)                               ; Fetch the previous value of the 10000s digit.
        LD      C,$00                                   ; A count of the number of extra lives to award.

L7775:  CP      (HL)                                    ; Has the player qualified for an extra life?
        JR      Z,L7782                                 ; Jump ahead if not.

; The player has qualified for an extra life.

        INC     A                                       ; Increment the previous 10000s digit value.
        CP      $0B                                     ; Has it overflowed, i.e. reached "10"?
        JR      NZ,L777F                                ; Jump if not.

        LD      A,$01                                   ; Reset it back to a value representing "0".

L777F:  INC     C                                       ; Increment the extra life count.
        JR      L7775                                   ; Jump back to check whether to award another extra life.

; The count of the number of extra lives has been made, so update the number of lives and the display.

L7782:  LD      (L640E),A                               ; Store the current value of the 10000s digit for testing against next time.

        LD      HL,L64BE                                ; Point to the store holding the number of lives.
        LD      A,(HL)                                  ; Fetch the number of lives.
        ADD     A,C                                     ; Apply the awarded lives.
        LD      (HL),A                                  ; Store the new number of lives.

        CALL    L6CFD                                   ; Configure caught by the Bubloid.

        NOP                                             ; [Redundant]
        NOP     
        NOP     
        NOP     

        JP      L66B4                                   ; Jump to start the next level.

; --------------
; All Lives Lost
; --------------

L7795:  LD      B,$07                                   ; There are 7 digits in the score.
        LD      DE,L64AB                                ; Point to the first digit of the score.
        LD      HL,L64B2                                ; Point to the first digit of the high score.

L779D:  LD      A,(DE)                                  ; Fetch a digit from the score.
        CP      (HL)                                    ; Is it the same as the high score?
        JR      NZ,L77A7                                ; Jump ahead if not.

        INC     HL                                      ; Advance to the next high score digit.
        INC     DE                                      ; Advance to the next score digit.
        DJNZ    L779D                                   ; Loop back for the next digit.

; The score is the same as the high score.

        JR      L77AF                                   ; Jump ahead to continue.

; The score digit is different to the corresponding high score digit.

L77A7:  JR      C,L77AF                                 ; Jump if the score digit is lower than the high score digit.

; The score digit is greater than the high score digit, so just copy over the remaining digits.

L77A9:  LD      A,(DE)                                  ; Fetch a digit from the score.
        LD      (HL),A                                  ; Fetch it into the high score.
        INC     HL                                      ; Advance to the next high score digit.
        INC     DE                                      ; Advance to the next score digit.
        DJNZ    L77A9                                   ; Loop back to copy all remaining digits.

; Copy the high score into the hi-res display file.

L77AF:  LD      HL,L40E5+$0013                          ; Line 3 column 19.
        LD      BC,L64B8                                ; Point to the last digit of the high score.
        LD      A,$07                                   ; There are 7 digits in the high score.
        CALL    L6515                                   ; Display the high score value in the display file.

; Wait for a key press.

        LD      DE,$FFFF                                ; The key code for 'no key pressed'.

L77BD:  LD      HL,(LAST_K)                             ; Fetch the key code from the last keyboard read.
        AND     A
        SBC     HL,DE                                   ; Was a key pressed?

L77C3:  JR      Z,L77BD                                 ; Loop back if not.

; A key has been pressed.

        JP      L653C                                   ; Jump to restart the game.

; --------
; Not Used
; --------

L77C8:  LD      D,D                                     ; [Redundant]
        JR      Z,L77C3

        JP      L653C

; -------------------------------
; Decrement and Display the Bonus
; -------------------------------
; Exit: Carry flag set if the bonus has reached "0000".

L77CE:  LD      HL,L64BC                                ; Point to the last digit of the bonus.
        LD      A,$01                                   ; Each digit of the bonus is based from 1, i.e. 1 represents 0.
        LD      B,$04                                   ; There are 4 digits.

L77D5:  CP      (HL)                                    ; Has this digit reached '0'?
        JR      NZ,L77DD                                ; Jump if not.

        DEC     HL                                      ; Point to the next digit.
        DJNZ    L77D5                                   ; Loop back to test the next digit.

        SCF                                             ; Set the carry flag to signal that the bonus has reached "0000".
        RET     

; Decrement the bonus.

L77DD:  LD      HL,L64BC                                ; Point to the last digit of the bonus value.
        LD      BC,$0400                                ; B=Number of digits of the bonus, C=Number of digits to redraw.
        CALL    L64CB                                   ; Decrement the bonus.

; Display the new bonus value in the display file.

        LD      A,C                                     ; Fetch the number of digits to update.
        LD      HL,L40E5+$001A                          ; Line 2 column 26.
        LD      BC,L64BC                                ; Point to the last digit of the bonus.
        CALL    L6515                                   ; Display the bonus value in the display file.

        AND     A                                       ; Reset the carry flag to signal that the bonus has not reached "0000".
        RET    

; -------------------
; End of BASIC Line 8
; -------------------

L77F2:  DEFB _NEWLINE

; ------------
; BASIC Line 9
; ------------

L77F3:  DEFB 9 >> 8
        DEFB 9 & $FF
        DEFW L7818 - 1 - $
        DEFB _REM

; ---------------------------
; Draw Fuel Can / Leg of Lamb
; ---------------------------

L77F8:  LD      HL,(L64BF)                              ; Fetch the location of the fuel can / leg of lamb.

        LD      A,(L6425)                               ; Fetch the level number.
        BIT     2,A                                     ; Level 1, 2 or 3?
        JR      Z,L780D                                 ; Jump ahead if so.

; Level 4, 5 or 6.

        LD      DE,$0042                                ; The leg of lamb is 2 lines shorter than the fuel can.
        ADD     HL,DE                                   ; Advance 2 lines lower in the display file.

        LD      B,$06                                   ; There are 6 lines to copy.
        LD      DE,L6296                                ; Point to the graphic for a leg of lamb.
        JR      L7812                                   ; Jump ahead to continue.

; Level 1, 2 or 3.

L780D:  LD      B,$08                                   ; There are 8 lines to copy.
        LD      DE,L628E                                ; Point to the graphic for a fuel can.

L7812:  CALL    L64DA                                   ; Copy the fuel can / leg of lamb graphic into the display file.

        JP      L7323                                   ; Jump to handle movement of the Bubloid.

; -------------------
; End of BASIC Line 9
; -------------------

L7818:  DEFB _NEWLINE

; -------------
; BASIC Line 10
; -------------

L7819:  DEFB 10 >> 8
        DEFB 10 & $FF
        DEFW L78AA - 1 - $
        DEFB _REM

; ----------------------------------------------------------------
; Determine Random Location for Fuel Can / Leg of Lamb (Continued)
; ----------------------------------------------------------------
; Entry: Selected display file position for the fuel can / leg of lamb to test.
; Exit : HL=Display file location verified as suitable for the fuel can / leg of lamb.

L781E:  PUSH    HL                                      ; Save the display file address of the fuel can / leg of lamb position.
        LD      B,$08                                   ; There are 8 lines of the fuel can / leg of lamb to test for an clash.

; Enter a loop to check whether the lines of the fuel can / leg of lamb clash with the player.

L7821:  EX      DE,HL                                   ; DE=Display file address of the line of the fuel can / leg of lamb being tested.
        PUSH    BC                                      ; Save the line count.

        LD      HL,(L6401)                              ; Fetch the address within the display file of the player.
        PUSH    HL                                      ; Stack it.

        INC     HL                                      ; Point at the right hand column of the player.
        PUSH    HL                                      ; Stack it.

        LD      BC,$01EE                                ; Offset of 14 lines and 32 columns.
        ADD     HL,BC                                   ; Advance to the left hand column 2 lines up from the bottom of the player.
        PUSH    HL                                      ; Stack it.

        INC     HL                                      ; Advance to the right hand column 2 lines up from the bottom of the player.
        PUSH    HL                                      ; Stack it.

        LD      B,$04                                   ; There are 4 corner positions of the player to test.

L7832:  POP     HL                                      ; Retrieve a test position.
        AND     A
        SBC     HL,DE                                   ; Is there a clash at this position?
        JR      Z,L7844                                 ; Jump if so.

        DJNZ    L7832                                   ; Repeat the test for the remaining corners.

; This line of the fuel can / leg of lamb does not clash with the player.

        LD      HL,$0021                                ; The length of a line of the display file.
        ADD     HL,DE                                   ; Advance to the line below in the fuel can / leg of lamb.
        POP     BC                                      ; Retrieve the line count.
        DJNZ    L7821                                   ; Repeat the test for all lines of the fuel can / leg of lamb.

; The fuel can / leg of lamb does not clash with the player.

        EXX                                             ; Restore the main registers (saved at L6635).
        POP     HL                                      ; Retrieve the display file address of the fuel can / leg of lamb position.
        RET     

; The fuel can / leg of lamb clashes with the player.

L7844:  DEC     B                                       ; Decrement the corner count.
        JR      Z,L784A                                 ; Jump if all corners were tested.

; Not all corners were tested and so the remaining test addresses on the stack need to be discarded.

L7847:  POP     HL                                      ; Discard the stacked test address.
        DJNZ    L7847                                   ; Repeat until all stacked test addresses have been discarded.

L784A:  POP     BC                                      ; Discard the line count.
        POP     HL                                      ; Discard the display file address of the fuel can / leg of lamb position.
        JP      L6636                                   ; Jump back to select a new random location for the fuel can / leg of lamb.

; ----------------------
; Display Splash Graphic
; ----------------------

L784F:  LD      A,$07                                   ; Set the splash animation movement rate.
        LD      (L6906+$0001),A

        JP      L7665                                   ; Jump to display the splash graphic.

; ---------------------
; Restart Current Level
; ---------------------

L7857:  LD      A,$02                                   ; Restore the player's movement rate.
        LD      (L6906+$0001),A

        JP      L683D                                   ; Jump to restart the current level.

; ------------------------------------
; Wait For a Key To Start Screen Level
; ------------------------------------

; Wait until there are no keys being pressed.

L785F:  LD      DE,$FFFF                                ; The key code for 'no key pressed'.

L7862:  LD      HL,(LAST_K)                             ; Fetch the key code from the last keyboard read.
        AND     A
        SBC     HL,DE                                   ; Is a key pressed?
        JR      NZ,L7862                                ; Loop back if so.

; Wait for a new key press.

L786A:  LD      HL,(LAST_K)                             ; Fetch the key code from the last keyboard read.
        AND     A
        SBC     HL,DE                                   ; Was a key pressed?
        JR      Z,L786A                                 ; Loop back if not.

; A key has been pressed.

        XOR     A                                       ; This value is used to set the low byte of FRAMES such that movement timers will expire on the next cycle.
        JP      L6846                                   ; Return back from the patch.

; ---------------------------
; Check SHIFT+H to Pause Game
; ---------------------------

L7876:  LD      DE,$DEBF                                ; The key code for 'SHIFT+H'.
        LD      HL,(LAST_K)                             ; Fetch the key code from the last keyboard read.
        AND     A
        SBC     HL,DE                                   ; Was 'SHIFT+H' pressed?
        JP      NZ,L68CF                                ; Jump if not to handle movement of the player.

; The game has been paused. Wait for SHIFT+S to resume the game.

        LD      DE,$FAFD                                ; The key code for 'SHIFT+S'.

L7885:  LD      HL,(LAST_K)                             ; Fetch the key code from the last keyboard read.
        AND     A
        SBC     HL,DE                                   ; Was 'SHIFT+S' pressed?
        JR      NZ,L7885                                ; Jump back if not.

; The game has been resumed.

        NOP                                             ; [Redundant]
        NOP     
        NOP     
        NOP     
        NOP     

        LD      HL,L6405                                ; Point at the store holding the player movement timer.
        LD      (HL),$FF                                ; The player will be moved vertically on the next cycle.
        INC     HL
        LD      (HL),$FF                                ; The player will be moved horizontally on the next cycle.

        LD      HL,L640A                                ; Point to the wave timer.
        LD      (HL),$FF                                ; The wave graphic will be animated on the next cycle.

        INC     HL
        INC     HL
        LD      (HL),$FF                                ; Value never used [Redundant].

        XOR     A
        LD      (FRAMES),A                              ; Set frames such that the player and wave movement timers expire on the next cycle.

        JP      L68CF                                   ; Jump ahead to handle movement of the player.

; --------------------
; End of BASIC Line 10
; --------------------

L78AA:  DEFB _NEWLINE

; ======================================================================================================================================================

; --------------
; BASIC Line 100
; --------------

; 100 SAVE "ROCKETMAn"

L78AB:  DEFB 100 >> 8
        DEFB 100 & $FF
        DEFW L78BB - 1 - $
        DEFB _SAVE, _QUOTE, _R, _O, _C, _K, _E, _T, _M, _A, _INVN, _QUOTE
L78BB:  DEFB _NEWLINE

; --------------
; BASIC Line 110
; --------------

; 110 PRINT USR 25916

L78BC:  DEFB 110 >> 8
        DEFB 110 & $FF
        DEFW L78CD - 1 - $
        DEFB _PRINT, _USR, _2, _5, _9, _1, _6, _NUMBER, $8F, $4A, $78, $00, $00
L78CD:  DEFB _NEWLINE

; ======================================================================================================================================================

; ------------
; Display File
; ------------

L78CE:  DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
        DEFB _NEWLINE    

; ======================================================================================================================================================

; --------------
; Variables Area
; --------------

L7BE7:  DEFB $80

; ======================================================================================================================================================

; ------------
; Editing Area
; ------------

L7BE8:

; ======================================================================================================================================================

; =============
; BASIC Program
; =============

;   1 REM {machine code}
;   2 REM {machine code}
;   3 REM {machine code}
;   4 REM {machine code}
;   5 REM {machine code}
;   7 REM {machine code}
;   8 REM {machine code}
;   9 REM {machine code}
;  10 REM {machine code}
; 100 SAVE "ROCKETMAn"
; 110 PRINT USR 25916

        END

